      subroutine nuclff(mass,mamo,q2,f1,f2)
c----------------------------------------------------------------------c
c Returns nucleon form factors f1 and f2
c Input: 
c        mass,mamo - mass and magnetic moment of a nucleon
c        q2 - squared four mometum transfer
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 20 Feb., 2014
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      real(dp) :: mass,mamo,q2
      complex(dp) :: f1,f2
      real(dp) :: gd,tau,ge,gm

      if (q2 < 0.d0) then
       gd=1.d0/(1.d0-q2/0.71d0)**2
       tau=-q2/(4.d0*mass**2)
       if (abs(mamo) > 2.7d0) then ! proton
        ge=gd
       else ! neutron
        ge=0.d0
       end if
       gm=mamo*gd
       f1=cmplx((ge+tau*gm)/(1.d0+tau),0.d0,dp)
       f2=cmplx((gm-ge)/(1.d0+tau)/(2.d0*mass),0.d0,dp)
      else
       f1=cmplx(1.d0,0.d0,dp)
       f2=cmplx(1.d0/(2.d0*mass),0.d0,dp)
      end if

      end
