      subroutine ppa_qp(ig,iwdth,sgn,u1,n1,p1,u2,n2,p2,u,n,p,g,
     &                                                       mass,wdth)
c----------------------------------------------------------------------c
c Returns a vector array being a contraction of a triple pi+pi-vector 
c vertex of the scalar QED, or other effective theory
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        iwdth=0 - fixed width of the vector particle,
c              1,2,... - running width of the vector particle,
c        sgn=+1, if scalars are writen in order pi-pi+,
c            -1, if scalars are writen in order pi+pi-,
c        u1,u2 - scalar arrays,
c        n1,n2,n - numbers of polarization states,
c        p1,p2 - momenta,
c        g - a triple pi+pi-photon coupling,
c        mass, wdth - mass and width of a vector particle.
c Output: u - vector array,
c         p - its momentum.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 30 April, 2009; modified on 7 May, 2009 (p changed)
c                             11 May, 2009 (sign of the photon propagator
c                                           corrected)
c 19 April, 2014 (propagator of a masive vector particle implemented)
c 2 June, 2014 (sign propagator of a masive vector particle corrected),
c 5 March, 2015 (s-dependent width o the vector meson implemented),
c 12 March, 2015 (call to subroutine runwidth added)
c 19 March, 2015 (call to subroutine runwidth changed,
c                 call to subroutine runcoupl implemented)
c 3 March, 2020 (quadruple precision version written)
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: ig,iwdth,n1,n2,n,i1,i2,i,sgn

      real(dp) :: mass,wdth
      real(qp), dimension(0:3) :: p1,p2,p,q
      complex(dp) :: g,mv2,rg
      complex(qp) :: u1(1,n1),u2(1,n2),u(0:3,n),gden

      p=-p1-p2

      if (ig == 0) then
       rg=g
      else
       call runcoupl_qp(ig,p,rg)
      end if

c      gden=-g/cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.d0,dp) ! 11 May, 2009
      if (mass > 0.d0) then
       if (iwdth == 0) then
        mv2=cmplx(mass**2,-mass*wdth,dp)
       else
        call runwidth_qp(iwdth,p,mass,wdth,mv2)
       end if
       gden=-rg/(cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.q0,qp)-mv2)
      else
       gden=-rg/cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.q0,qp)
      end if

      q=sgn*(p1-p2)

      i=0
      do i2=1,n2
       do i1=1,n1
        i=i+1
        if (mass > 0.d0) then
         u(:,i)=gden*u1(1,i1)*u2(1,i2)
     &        *(q-(q(0)*p(0)-q(1)*p(1)-q(2)*p(2)-q(3)*p(3))*p/mv2)
        else
         u(:,i)=gden*u1(1,i1)*u2(1,i2)*q
        end if
       end do
      end do

      end subroutine ppa_qp
