      subroutine genmat
c----------------------------------------------------------------------c
c Generates the matrix element
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 14 February, 2006; modified 23 February, 2006; 8 October, 2007;
c                             6/7 January, 2008 (reduction of the No of
c                             open files, where matrix elements are 
c                             added);
c                             20 March, 2008 (existing data files with
c                             topologies need not generated again)
c                             20 May, 2008
c                             24 May, 2009 (masses and widths of 
c                             intermediate particles stored in a separate
c                             file - unit 31)
c                             3 Feb., 2010 (A check if the existing topology 
c                             file is not empty implemented)
c                             25 Feb., 2010 (writing of kinematical routines
c                             modified: parts common for graphs of the same
c                             topology are not repeated any more, a new 
c                             module 'ntnkin.f' is generated)
c 18 March 2010: 
c writing of umodkk.f and ntnkin.f changed
c 17 May 2010: 
c writing of matrixel.f and mpol2.f changed as calculation of color 
c factors has been shifted to the stage of code generation
c 30 Sept. 2010:
c generation of repetitive kinematical routines eliminated
c 5 Oct. 2010:
c writng of ntnkin.f modified
c 17 Jan. 2011:
c a bug in writing kinematical routines for 2 -> 2 processes corrected
c 15-18 Feb. 2013:
c generation of colsqkk.f changed, matrixel.f and mpol2.f modified
c 22-23 January 2014:
c Modifications for imcs=1 introduced
c 23 July, 2014:
c check of U(1) gauge invariance for external photons implemented
c 31 Jan. 2016:
c a call to wrpartls changed
c 10 Feb. 2016:
c The EM charged pion form factor replaced with the fixed coupling if 
c the photon in gamma pi+ pi- vertex is external
c 14 July, 2016:
c Writing of subroutines psub*.f changed
c 12 Dec., 2016:
c A bug in writing module pmodkk corrected
c 8 Nov. 2018: calls to routines with quadrple precision four momenta 
c              implemented
c 5 Dec. 2019: calls to kingchnl limited to the case of igchnl=1
c 18 June, 2020: extra variables in writing of engymom.f added
c 22-23 June, 2020: modules mmodkk* and corresponding parts of mtdr*.f, 
c where amplitudes were added, included in subroutines mdkk*.f as parts
c of their intrinsic code
c 22-23 Feb. 2021: calls to quadruple precison versions of the helicity 
c amplitudes computation routines added
c 9 March 2021: bug in writing of kingchnl.f for isr=1 corrected
c 22 March 2021: 
c Calls to the FSR routines with neutrinos in a wrong place eliminated 
c 14 August 2021:
c Check of number of topologies in the topology file generated prior
c to the program execution implemented
c----------------------------------------------------------------------c

      use maxvals
      use mytypes
      use extpart
      use intpart
      use psnname
      use declends ! added on 18 March 2010
      use colinfo
      use schkins
      use tchkins
      use gchkins

      implicit none

      integer :: nun,i,id,idr,nkm,ipl,nf,l,ntin,ntop,ncls,
     &           npl,blad,plj,pli,plo,plg,plf,plct,pln,ple,
     &           nuni,nunf,icont,icon0,iconx,icl,icl0,icdec,iccon,
     &           lc,icla,plord(next),sgn,icon1,icony,temp(2),iphot,
     &           nking0=0,isl,isl0,iwrg,iwrg4,ntopol,i1,i2,i3

      character(80) :: cl,clx,ind,cdec,ccon,ind1,ind2,ind3,ind4,ind5,
     &                 indp,indq,sl,slx
      character(160) :: cl2
      character(240) :: cl3
      character(80), allocatable :: cla(:),matnm(:)
      integer, allocatable :: iclx(:),iclu(:)
      integer :: ifrst,icon
      character(80) :: decl,declnd,rest,sbstr1,sbstr2,clfsr,clwrg
      character(160) :: clkg,clkg4

ckkbla>
c       open(unit=987,file='top.0',status='replace',action='write')
c       open(unit=988,file='top.1',status='replace',action='write')
c       open(unit=989,file='top.2',status='replace',action='write')
ckkbla<


c Set initial values of the No of Feynman diagrams, reduced No of 
c the diagrams and No of intermediate particles

      id=0
      idr=0
      ipl=0
      nkm=0
      did=0       ! 30 Sept. 2010
      dnk=0       ! 30 Sept. 2010

      open(unit=11,file='matrixel.f',status='replace',action='write')
      open(unit=12,status='scratch',action='readwrite')
      open(unit=14,status='scratch',action='readwrite',recl=160)
      open(unit=15,status='scratch',action='readwrite')
      open(unit=70,status='scratch',action='readwrite')
c      open(unit=111,file='reddgs.tmp',status='replace',
c     &                                            action='readwrite')
      open(unit=111,file='scratch',status='replace',
     &                                            action='readwrite')
      open(unit=26,file='colsqkk.f',status='replace',action='write')
      open(unit=268,file='kingtmp4.dat',status='replace',action='write')

      icncl=0
      call begincolsq(26)

      call preamble(11)

      pl(:)%wr=0

      nf=0

c      open(13,file='colfs.txt',access='direct',form='formatted',
c     &                                                     recl=80)
      open(unit=13,status='scratch',access='direct',form='formatted',
     &                                   action='readwrite',recl=80)
c      open(18,file='calls.txt',access='direct',form='formatted',
c     &                                                     recl=160)
      open(18,status='scratch',access='direct',form='formatted',
     &                                                     recl=160)
c      open(20,file='orders.tmp',access='direct',form='formatted',
c     &                                                     recl=80)
      open(20,status='scratch',access='direct',form='formatted',
     &                                                     recl=80)
      open(21,file='decls.txt',access='direct',form='formatted',
     &                                                     recl=80)
      open(unit=28,file='kincls.f',status='replace',action='write')
      open(unit=289,file='kinclsnew.f',status='replace',action='write')
      open(unit=300,file='kinclsnew.txt',status='replace',
     &                                                  action='write')
      open(unit=285,file='kingchnl.f',status='replace',action='write')
      open(unit=287,file='engymom.f',status='replace',action='write')
      open(unit=29,file='psnkk.f',status='replace',action='write')
      open(31,status='scratch',access='direct',form='formatted',
     &                                                     recl=20)

      write(28,'(a)') '      subroutine kincls(ikin,imom,ecm,p,x,ndim,'
     &                                                  //'dlips)'
      write(289,'(a)') '      subroutine kinclsnew(ikin,imom,ecm,p,x,'
     &                                                 //'ndim,dlips)'

      write(28,'(a)') ' '
      write(28,'(a)') '      use partcls'
      write(289,'(a)') ' '
      write(289,'(a)') '      use partcls'

      if (next > 4) then    ! 17 Jan. 2011
       write(28,'(a)') '      use ntnkin'
      end if    ! 17 Jan. 2011
      write(28,'(a)') ' '
      write(28,'(a)') '      implicit none'
      write(28,'(a)') ' '
      write(28,'(a)') '      integer :: ikin,imom,ndim'
      write(289,'(a)') ' '
      write(289,'(a)') '      implicit none'
      write(289,'(a)') ' '
      write(289,'(a)') '      integer :: ikin,imom,ndim'

      write(indp,'(i16)') kind(1.d0)
      write(indq,'(i16)') kind(1.q0)
      if (iqprec == 1) then
       write(28,'(a)') '      real('//trim(adjustl(indp))//
     &                ') :: x(ndim),dlips'
       write(28,'(a)') '      real('//trim(adjustl(indq))//
     &                ') :: ecm,p(0:3,nepl)'
       write(289,'(a)') '      real('//trim(adjustl(indp))//
     &                ') :: x(ndim),dlips'
       write(289,'(a)') '      real('//trim(adjustl(indq))//
     &                ') :: ecm,p(0:3,nepl)'
      else
       write(28,'(a)') '      real('//trim(adjustl(indp))//
     &                ') :: ecm,p(0:3,nepl),x(ndim),dlips'
       write(289,'(a)') '      real('//trim(adjustl(indp))//
     &                ') :: ecm,p(0:3,nepl),x(ndim),dlips'
      end if
      write(28,'(a)') ' '
      write(289,'(a)') ' '

      write(285,'(a)') '      subroutine kingchnl(ikin,imom,ee,pp,'
     &                                                //'x,ndim,dlips)'
      write(285,'(a)') ' '
      write(285,'(a)') '      use engymom'
      write(285,'(a)') ' '
      write(285,'(a)') '      integer :: ikin,imom,ndim'
      write(285,'(a)') '      real(dp) :: x(ndim),dlips'
      if (iqprec == 1) then
       write(285,'(a)') '      real(qp) :: ee,pp(0:3,nepl)'
      else
       write(285,'(a)') '      real(dp) :: ee,pp(0:3,nepl)'
      end if
      write(285,'(a)') ' '
      write(285,'(a)') '      ecm=ee'
      write(285,'(a)') '      p=pp'

      write(287,'(a)') '      module engymom'
      write(287,'(a)') ' '
      write(287,'(a)') '      use inprms'
      write(287,'(a)') '      use partcls'
      write(287,'(a)') ' '
      if (iqprec == 1) then
       write(287,'(a)') '      real(qp) :: ecm,p(0:3,nepl),'
      else
       write(287,'(a)') '      real(dp) :: ecm,p(0:3,nepl),'
      end if
      write(287,'(a)') '     &            msum,reg,sp,sqsp,g,v(3),'
      write(287,'(a)') '     &            beta,ybta,dctg,'
      write(287,'(a)') '     &            q1(0:3),q2(0:3),q3(0:3),'//
     &                                   'q4(0:3),'
      write(287,'(a)') '     &            q23(0:3),q12(0:3),q34(0:3),'
     &                                 //'q234(0:3),'
      write(287,'(a)') '     &            sqsl1,sqsl2,sqsl3,sqsl4,'
      write(287,'(a)') '     &            sqsl23,sqsl12,sqsl34,sqsl234,'
      write(287,'(a)') '     &            sqsu1,sqsu2,sqsu3,sqsu4,'
      write(287,'(a)') '     &            sqsu23,sqsu12,sqsu34,sqsu234,'
      write(287,'(a)') '     &            s1,s2,s3,s4,'
      write(287,'(a)') '     &            s23,s12,s34,s234,'
      write(287,'(a)') '     &            sqs1,sqs2,sqs3,sqs4,'
      write(287,'(a)') '     &            sqs23,sqs12,sqs34,sqs234,'
      write(287,'(a)') '     &            qq1,b34g,e34g,v34g(3),g34g,'
      write(287,'(a)') '     &            e3,e7'
      write(287,'(a)') ' '
      write(287,'(a)') '      end module engymom'
      close(287)      

      nking=0

      write(29,'(a)') '      subroutine psnkk(ikin0,ecm,p,x,ndim,dlinv)'
      write(29,'(a)') ' '
      write(29,'(a)') '      use partcls'
      write(29,'(a)') '      use psnrmtest'
      write(29,'(a)') ' '
      write(29,'(a)') '      implicit none'
      write(29,'(a)') ' '
c      write(29,'(a)') '      integer, parameter :: dp=kind(1.d0),'//
c     &                       'imom=0'
c      write(29,'(a)') '      integer :: ikin0,ndim,ikin'
      write(29,'(a)') '      integer :: ikin0,ndim'
      if (iqprec == 1) then
       write(29,'(a)') '      real('//trim(adjustl(indp))//
     &                ') :: x(ndim),dlinv,dlips0'
       write(29,'(a)') '      real('//trim(adjustl(indq))//
     &                ') :: ecm,p(0:3,nepl)'
      else
       write(29,'(a)') '      real('//trim(adjustl(indp))//
     &                ') :: ecm,p(0:3,nepl),x(ndim),dlinv,dlips0'
      end if
c     &                     'dlinv,dlips,dlips0'
      write(29,'(a)') ' '
      write(29,'(a)') '      dlips0=dlinv'
      write(29,'(a)') '      dlinv=0.d0'
      write(29,'(a)') '      dlinvt=0.d0'
      write(29,'(a)') ' '
c      write(29,'(a)') '      do ikin=1,nkin'
c      write(29,'(a)') '       if (aw(ikin) > 0.d0) then'

c Added on 25 Feb. 2010
      open(unit=272,file='ntnkin.f',status='replace',action='write')
      write(272,'(a)') '      module ntnkin'
      write(272,'(a)') ' '

      do i=1,next
       ipl=ipl+1
       pl(ipl)%k=0
       iplord=ipl
c       pl(ipl)%k(0)=iplord
       write(20,'(26(i3))',rec=iplord) iplord
       allocate(pl(ipl)%ip(0),stat=blad)
       pl(ipl)%ct=f(ipl)%ct
       write(13,'(a)',rec=ipl) '1'
       if (f(ipl)%j < 21) then
        nf=nf+1
        write(ind,'(i16)') ipl
        write(ind1,'(i16)') f(ipl)%n
        cl='u'//trim(adjustl(ind))//'(4,'//trim(adjustl(ind1))//')'
        write(21,'(a)',rec=ipl) cl
        write(ind2,'(i16)') f(ipl)%i
        write(ind3,'(i16)') f(ipl)%o
        if (f(ipl)%mc == 0) then
         write(ind4,'(i16)') 0
         write(ind5,'(a)') '0.d0'
        else
         write(ind4,'(i16)') imcs
         write(ind5,'(i16)') f(ipl)%mc
         ind5='phi('//trim(adjustl(ind5))//')'
        end if
c        write(18,'(a)',rec=ipl) '      call spinorkk(p'//
c     &      trim(adjustl(ind))//','//
c     &      trim(f(ipl)%m)//','//trim(adjustl(ind2))//','//
c     &      trim(adjustl(ind3))//',u'//trim(adjustl(ind))//','//
c     &      trim(adjustl(ind1))//')'

        if (iqprec == 1) then
         write(18,'(a)',rec=ipl) '      call spinor_qp(p'//
     &      trim(adjustl(ind))//','//
     &      trim(f(ipl)%m)//','//trim(adjustl(ind2))//','//
     &      trim(adjustl(ind3))//','//trim(adjustl(ind4))//','
     &      //trim(adjustl(ind5))//',u'//trim(adjustl(ind))
     &      //','//trim(adjustl(ind1))//')'
        else
         write(18,'(a)',rec=ipl) '      call spinorkk(p'//
     &      trim(adjustl(ind))//','//
     &      trim(f(ipl)%m)//','//trim(adjustl(ind2))//','//
     &      trim(adjustl(ind3))//','//trim(adjustl(ind4))//','
     &      //trim(adjustl(ind5))//',u'//trim(adjustl(ind))
     &      //','//trim(adjustl(ind1))//')'
        end if
ckk>
        write(31,'(a10,a10)',rec=ipl) trim(f(ipl)%m),trim(f(ipl)%w)
ckk<
        pl(ipl)%wr=1
       else
c        if (f(ipl)%f > 0) then
        if (abs(f(ipl)%f) > 1) then
         write(ind,'(i16)') ipl
         write(ind1,'(i16)') f(ipl)%n
         if (f(ipl)%mc == 0) then
          write(ind4,'(i16)') 0
          write(ind5,'(a)') '0.d0'
         else
          write(ind4,'(i16)') imcs
          write(ind5,'(i16)') f(ipl)%mc
          ind5='phi('//trim(adjustl(ind5))//')'
         end if

         if (iqprec == 1) then
          write(18,'(a)',rec=ipl) '      call recpol_qp(p'//
     &      trim(adjustl(ind))//','//
     &      trim(f(ipl)%m)//','//trim(adjustl(ind4))//','
     &      //trim(adjustl(ind5))//',u'//trim(adjustl(ind))//','//
     &      trim(adjustl(ind1))//')'
         else
          write(18,'(a)',rec=ipl) '      call recpolkk(p'//
     &      trim(adjustl(ind))//','//
     &      trim(f(ipl)%m)//','//trim(adjustl(ind4))//','
     &      //trim(adjustl(ind5))//',u'//trim(adjustl(ind))//','//
     &      trim(adjustl(ind1))//')'
         end if
ckk>
         write(31,'(a10,a10)',rec=ipl) trim(f(ipl)%m),trim(f(ipl)%w)
ckk<
         cl='u'//trim(adjustl(ind))//'(0:3,'//trim(adjustl(ind1))//')'
         write(21,'(a)',rec=ipl) cl
        else
c         write(18,'(a)',rec=ipl) ' '
         write(ind,'(i16)') ipl
         write(ind1,'(i16)') f(ipl)%n
         cl='u'//trim(adjustl(ind))//'(1,'//trim(adjustl(ind1))//')'
         write(21,'(a)',rec=ipl) cl
         write(18,'(a)',rec=ipl) '      u'//trim(adjustl(ind))//'=onec'
ckk>
         write(31,'(a10,a10)',rec=ipl) trim(f(ipl)%m),trim(f(ipl)%w)
ckk<
        end if
        pl(ipl)%wr=1   ! This must work also for the Higgs
       end if
      end do

c      close(18)
c      close(20)
c      close(13)

      nexpl=ipl

      ntin=0
      ncls=0

      nuni=1
      nunf=2

      npl=next

      write(ind1,'(i16)') npl-1
      open(unit=nuni,file='topol'//trim(adjustl(ind1))//'.dat',
     &                           status='old',action='read',err=200)
ckk>
c Check if the topology file contain expected number of topologies
      ntopol=0
      do
       read(nuni,'(2(i4))',end=190) temp     ! 3 Feb. 2010
       ntopol=ntopol+1
       do i=1,temp(1)
        read(nuni,'(3(i3))',end=190) i1,i2,i3
       end do
       do i=1,temp(2)
        read(nuni,'(3(i3))',end=190) i1,i2,i3
       end do
      end do

 190  continue

      if (npl == 5 .and. ntopol == ntop_4) then
       rewind nuni
      else if (npl == 6 .and. ntopol == ntop_5) then
       rewind nuni
      else if (npl == 7 .and. ntopol == ntop_6) then
       rewind nuni
      else if (npl == 8 .and. ntopol == ntop_7) then
       rewind nuni
      else if (npl == 9 .and. ntopol == ntop_8) then
       rewind nuni
      else if (npl == 10 .and. ntopol == ntop_9) then
       rewind nuni
      else if (npl == 11 .and. ntopol == ntop_10) then
       rewind nuni
      else if (npl == 12 .and. ntopol == ntop_11) then
       rewind nuni
      else if (npl > 12) then
       write(6,'(a)')'That many external particles are not supported'
       stop 
      else
       go to 200
      end if
ckk<
c      if (npl > 4) then 
c       open(unit=3,file='topnos.dat',access='direct',form='formatted',
c     &                             status='old',action='read',recl=80)
c       read(3,'(i16)',rec=npl-4) ntin
c       close(3)
c      end if
      if (npl == 5) then
       ntin=ntop_4
      else if (npl == 6) then
       ntin=ntop_5
      else if (npl == 7) then
       ntin=ntop_6
      else if (npl == 8) then
       ntin=ntop_7
      else if (npl == 9) then
       ntin=ntop_8
      else if (npl == 10) then
       ntin=ntop_9
      else if (npl == 11) then
       ntin=ntop_10
      else if (npl == 12) then
       ntin=ntop_11
      end if
      call gentop(nuni,nunf,npl,ntin,ntop,ncls,id,idr,nkm,ipl)
      close(nuni)

      go to 300

 200  npl=4

      do 
       if (npl > 4) then
        write(ind1,'(i16)') npl-1
        open(unit=nuni,file='topol'//trim(adjustl(ind1))//'.dat',
     &                                  status='old',action='read')
        rewind nuni
       end if
       if (npl < next .or. npl == 4) then
        write(ind1,'(i16)') npl
        open(unit=nunf,file='topol'//trim(adjustl(ind1))//'.dat',
     &                            status='replace',action='write')
       end if

       call gentop(nuni,nunf,npl,ntin,ntop,ncls,id,idr,nkm,ipl)

       if (npl > 4) then
        close(nuni)
       end if
       if (npl < next .or. npl == 4) then
        close(nunf)
c        if (npl == 4) then
c         open(unit=3,file='topnos.dat',access='direct',form='formatted',
c     &                        status='replace',action='write',recl=80)
c        else
c         open(unit=3,file='topnos.dat',access='direct',form='formatted',
c     &                        status='old',action='write',recl=80)
c        end if
c        write(3,'(i16)',rec=npl-3) ntop
c        close(3)
       end if
       nun=nunf
       nunf=nuni
       nuni=nun
       ntin=ntop
       if (npl == next) exit
       npl=npl+1
      end do

 300  if (id == 0) then
       write(*,*)'No Feynman diagrams for this process'
       write(*,*)'Try another one!'
       stop
      end if


c      open(unit=17,file='partcls.f',status='replace',action='write')
c      call wrpartcls(17,next,nisp,nmcs,id,ickm,isr,iwtb,nkm,nphot,nhls)
c      close(17)


      open(16,file='partcls.tmp',access='direct',form='formatted',
     &                                                     recl=80)
      open(unit=19,file='partcls.dat',status='replace',action='write')

      write(*,'(a,i10,a,i10,a)') 'Of',ipl-next,' internal particles '//
     & 'created the following',ncls,' are used:'

      do i=next+1,ipl
       read(16,1612,rec=i) plj,pli,plo,plg,plf,plct,pln,ple
       if (i <= next) then
        if (plj > 20 .and. plj /= 24) then
c Restore old conventions for neutral bosons
         pli=1
        end if
        if (pl(i)%wr > 0) then
         write(*,3) i,plo*pli*plj,pli,plo,pln,ple 
        end if
       else
        if (plj > 20) then
c Restore old conventions for intermediate bosons
         if (plj == 24) then
          if (plo == 1) then
           pli=-pli
          end if
          plo=0
         else
          pli=1
          plo=0
         end if
        end if
        if (pl(i)%wr > 0) then
         write(6,3) i,plo*pli*plj,pli,plo,pln,ple 
        end if
       end if
      end do

 1612 format(i8,5(i3),i24,i3)
 3    format('pl(',i10,'):',5(i6))
 1614 format(i3,i3,i3,i3,i3,i24)


      write(19,'(a)') '      prvrsn='''//trim(adjustl(prvrsn))
     &                                                        //''''
      write(19,'(a)') '      prcsnm='''//trim(adjustl(prcsnm))
     &                                                        //''''

      do i=1,next
       write(ind,'(i16)') i
       write(19,'(a)') '      expc('//trim(adjustl(ind))//
     &                              ')='//''''//trim(expc(i))//''''
       write(19,'(a)') '      expl('//trim(adjustl(ind))//
     &                              ')='//"'"//expl(i)//"'"
       if(iqprec == 1) then
        write(19,'(a)') '      m('//trim(adjustl(ind))//
     &                               ')='//trim(f(i)%m)//'_qp'
       else
        write(19,'(a)') '      m('//trim(adjustl(ind))//
     &                                       ')='//trim(f(i)%m)
       end if
       write(19,'(a)') '      wdth('//trim(adjustl(ind))//
     &                                       ')='//trim(f(i)%w)
       write(19,'(a,i10)') '      mc_code('//trim(adjustl(ind))//
     &                                       ')=',f(i)%j
      end do

      close(16)
      close(19)

      write(*,*) ntop, ' topologies generated'
      write(*,*) id, ' Feynman diagrams generated'
      write(*,'(a,i10)') 'No of colored amplitudes reduced to', idr
      write(*,'(a)') '---------------------------------------------'//
     &  '--------------------'
      write(*,'(a)') 'Change to mc_computation in order to run the '//
     &  'Monte Carlo program.'
      write(*,'(a)') '---------------------------------------------'//
     &  '--------------------'

      rewind 70

      write(ind,'(i16)') idr
      write(ind1,'(i16)') nhel

      write(11,'(a)')'      integer, parameter :: rdgs='
     &               //trim(adjustl(ind))
      write(ind,'(i16)') nhip
      write(ind1,'(i16)') nidp
      write(ind2,'(i16)') ncip
      write(11,'(a)')'      integer, parameter :: nhip='
     &               //trim(adjustl(ind))//', nidp='
     &               //trim(adjustl(ind1))//', ncip='
     &               //trim(adjustl(ind2))
      if (iqprec == 1) then
       write(11,'(a)')'      real('//trim(adjustl(indq))//
     &                ') :: mpol2'
      else
       write(11,'(a)')'      real('//trim(adjustl(indp))//
     &                ') :: mpol2'
      end if
      open(unit=30,file='rcolms.f',status='replace',action='write')

      write(ind,'(i16)') idr
      write(30,'(a)') '      module rcolms'
      write(30,'(a)') ' '
      write(30,'(a)') '      integer, parameter :: rdgs='
     &                //trim(adjustl(ind))
      write(ind1,'(i16)') ncol
      write(30,'(a)') '      integer, parameter :: ncls='
     &                                 //trim(adjustl(ind1))
      write(30,'(a)') '      integer, parameter :: dp=kind(1.d0), '
     &                                              //'nc2=ncls**2-1'
      write(ind,'(d10.2)') rcold
      write(30,'(a)') '      real(dp), parameter :: rcold='
     &  //trim(adjustl(ind))
      write(30,'(a)') '      complex(dp), parameter :: '//
     &                  'ocom=(0.d0,0.d0),'
      write(30,'(a)') '     &                          '//
     &                  'onec=(1.d0,0.d0),'
      write(30,'(a)') '     &                          '//
     &                  'icom=(0.d0,1.d0)'
      write(30,'(a)') '      complex(dp) :: cfd(rdgs), '//
     &                  't(nc2,ncls,ncls),'
      write(30,'(a)') '     &                          '//
     &                  'f(nc2,nc2,nc2)'
      write(30,'(a)') '      real(dp) :: d(ncls,ncls)'

      write(30,'(a)') ' '
      write(30,'(a)') '      contains'
      write(30,'(a)') ' '
      write(30,'(a)') '      function tt(i,j,k,l)'
      write(30,'(a)') '      real(dp) :: tt'
      write(30,'(a)') '      integer :: i,j,k,l'
      write(30,'(a)') ' '
      write(30,'(a)') '      tt=-d(i,j)*d(k,l)/dble(2*ncls)'//
     &                                    '+d(i,l)*d(k,j)/2.d0'
      write(30,'(a)') ' '
      write(30,'(a)') '      end function tt'
      write(30,'(a)') ' '
      write(30,'(a)') '      function c(r,i,j,k,l)'
      write(30,'(a)') '      real(dp) :: c'
      write(30,'(a)') '      integer :: r,i,j,k,l,m'
      write(30,'(a)') '      c=0.d0'
      write(30,'(a)') '      do m=1,nc2'
      write(30,'(a)') '       if (r == 1) then'
      write(30,'(a)') '        c=c-f(i,j,m)*f(m,k,l)'
      write(30,'(a)') '       else if (r == 2) then'
      write(30,'(a)') '        c=c-f(i,k,m)*f(m,l,j)'
      write(30,'(a)') '       else if (r == 3) then'
      write(30,'(a)') '        c=c-f(i,l,m)*f(m,j,k)'
      write(30,'(a)') '       else'
      write(30,'(a)') '        write(*,*)''colsqkk.f: '//
     &                '       Wrong colour coeff.'',r'
      write(30,'(a)') '       end if'
      write(30,'(a)') '      end do'
      write(30,'(a)') '      end function c'
      write(30,'(a)') ' '
      write(30,'(a)') '      end module rcolms'

      close(30)



      open(unit=9,file='hmodkk.f',status='replace',action='write')

      write(9,'(a)') '      module hmodkk'
      write(9,'(a)') ' '
      write(ind,'(i16)') nhel
      write(9,'(a)')'      integer, parameter :: nhel='
     &               //trim(adjustl(ind))
      write(9,'(a)') ' '

      if (imcs == 1) then
       write(9,'(a)') 'c Here come declarations that '
     &                   //'are not written if imcs = 1'
      else
       icont=0
       icon0=0
       iconx=0
       clx=''
       cdec='      integer, dimension(nhel)::'
       ccon='     &'
       icdec=len_trim(cdec)
       iccon=len_trim(ccon)

       do i=1,ipl
        if (pl(i)%wr == 1) then
         if (iconx == icon0) then
          icl0=icdec
          icl=icdec+len_trim(clx)
          cl(1:icl)=trim(cdec)//trim(clx)
          icon0=icon0+1
         end if

         write(ind,'(i16)') i

         if (i == ipl) then
          clx='h'//trim(adjustl(ind))
         else
          clx='h'//trim(adjustl(ind))//','
         end if

         icl0=icl
         icl=icl+len_trim(clx)

         if (icl > 72) then
          icl=icl0
          if ((icont+1)/mxncnt == icon0) then
           write(9,'(a)') cl(1:icl-1)
           write(9,'(a)') ' '
          else
           write(9,'(a)') cl(1:icl)
          end if
          if (i == ipl) then
           icl=icdec+len_trim(clx)
           cl(1:icl)=trim(cdec)//trim(clx)
           write(9,'(a)') cl(1:icl)
          else
           icont=icont+1
           if (icont/mxncnt == icon0) then
            icl0=icdec
            icl=icdec+len_trim(clx)
            cl(1:icl)=trim(cdec)//trim(clx)
           else
            icl0=iccon
            icl=iccon+len_trim(clx)
            cl(1:icl)=trim(ccon)//trim(clx)
           end if
          end if
         else
          cl(icl0+1:icl)=clx
         end if
         iconx=icont/mxncnt
        end if

        if (i == ipl) then
         if (index(trim(cl(1:icl)),',',.true.) == icl) then
          write(9,'(a)') cl(1:icl-1)
         else
          write(9,'(a)') cl(1:icl)
         end if
        end if
       end do
      end if

      write(9,'(a)')' '
      write(9,'(a)') '      end module hmodkk'
      close(9)


      open(unit=8,file='pmodkk.f',status='replace',action='write')

      write(8,'(a)') '      module pmodkk'
      write(8,'(a)') ' '
c      write(8,'(a)') '      integer, parameter :: dp=kind(1.d0)'
c      write(8,'(a)') '      use mathprms'
c      write(8,'(a)') ' '

      icont=0
      icon0=0
      iconx=0
      clx=''
      if (iqprec == 1) then
       cdec='      real('//trim(adjustl(indq))//'), dimension(0:3)::'
      else
       cdec='      real('//trim(adjustl(indp))//'), dimension(0:3)::'
      end if
      ccon='     &'
      icdec=len_trim(cdec)
      iccon=len_trim(ccon)

      do i=1,ipl
       if (pl(i)%wr == 1) then
        if (iconx == icon0) then
         icl0=icdec
         icl=icdec+len_trim(clx)
         cl(1:icl)=trim(cdec)//trim(clx)
         icon0=icon0+1
        end if

        write(ind,'(i16)') i

        if (i == ipl) then
         clx='p'//trim(adjustl(ind))
        else
         clx='p'//trim(adjustl(ind))//','
        end if

        icl0=icl
        icl=icl+len_trim(clx)

        if (icl > 72) then
         icl=icl0
         if ((icont+1)/mxncnt == icon0) then
          write(8,'(a)') cl(1:icl-1)
          write(8,'(a)') ' '
         else
          if (i == ipl) then                 ! added on 12.12.2016
           write(8,'(a)') cl(1:icl-1)
          else                               !
           write(8,'(a)') cl(1:icl)          ! added on 12.12.2016
          end if                             !
         end if
         if (i == ipl) then
          icl=icdec+len_trim(clx)
          cl(1:icl)=trim(cdec)//trim(clx)
          write(8,'(a)') cl(1:icl)
          exit                               ! added on 12.12.2016
         else
          icont=icont+1
c Changed on 20 May, 2008:------------------------>
c          icl0=iccon
c          icl=iccon+len_trim(clx)
c          cl(1:icl)=trim(ccon)//trim(clx)
          if (icont/mxncnt == icon0) then
           icl0=icdec
           icl=icdec+len_trim(clx)
           cl(1:icl)=trim(cdec)//trim(clx)
          else
           icl0=iccon
           icl=iccon+len_trim(clx)
           cl(1:icl)=trim(ccon)//trim(clx)
          end if
c--------------------------------------------------<
         end if
        else
         cl(icl0+1:icl)=clx
        end if
        iconx=icont/mxncnt
       end if
       if (i == ipl) then
        if (index(trim(cl(1:icl)),',',.true.) == icl) then
         write(8,'(a)') cl(1:icl-1)
        else
         write(8,'(a)') cl(1:icl)
        end if
       end if
      end do

      write(8,'(a)') ' '
      write(8,'(a)') '      end module pmodkk'

      close(8)

ckk> Changed of 18 March 2010:
c      open(unit=7,file='umodkk.f',status='replace',action='write')
c
c      write(7,'(a)') '      module umodkk'
c      write(7,'(a)') ' '
c
c      icont=0
c      icon0=0
c      iconx=0
c      clx=''
c      cdec='      complex('//trim(adjustl(indp))//')::'
c      ccon='     &'
c      icdec=len_trim(cdec)
c      iccon=len_trim(ccon)
c
c      do i=1,ipl
c       if (pl(i)%wr == 1) then
c        if (iconx == icon0) then
c         icl0=icdec
c         icl=icdec+len_trim(clx)
c         cl(1:icl)=trim(cdec)//trim(clx)
c         icon0=icon0+1
c        end if
c
c        write(ind,'(i16)') i
c
c        read(21,'(a)',rec=i) clx
c
c        clx=trim(clx)//','
c
c        icl0=icl
c        icl=icl+len_trim(clx)
c
c        if (icl > 72) then
c         icl=icl0
c         if ((icont+1)/mxncnt == icon0) then
c          write(7,'(a)') cl(1:icl-1)
c          write(7,'(a)') ' '
c         else
c          write(7,'(a)') cl(1:icl)
c         end if
c         if (i == ipl) then
c          icl=icdec+len_trim(clx)
c          cl(1:icl)=trim(cdec)//trim(clx)
c          write(7,'(a)') cl(1:icl)
c         else
c          icont=icont+1
cc Changed on 20 May, 2008:------------------------>
cc          icl0=iccon
cc          icl=iccon+len_trim(clx)
cc          cl(1:icl)=trim(ccon)//trim(clx)
c          if (icont/mxncnt == icon0) then
c           icl0=icdec
c           icl=icdec+len_trim(clx)
c           cl(1:icl)=trim(cdec)//trim(clx)
c          else
c           icl0=iccon
c           icl=iccon+len_trim(clx)
c           cl(1:icl)=trim(ccon)//trim(clx)
c          end if
cc--------------------------------------------------<
c         end if
c        else
c         cl(icl0+1:icl)=clx
c        end if
c        iconx=icont/mxncnt
c       end if
c       if (i == ipl) then
c        if (index(trim(cl(1:icl)),',',.true.) == icl) then
c         write(7,'(a)') cl(1:icl-1)
c        else
c         write(7,'(a)') cl(1:icl)
c        end if
c       end if
c      end do
c
c      write(7,'(a)') ' '
c      write(7,'(a)') '      end module umodkk'
c
c      close(7)

      open(unit=7,file='umodkk.f',status='replace',action='write')
      write(7,'(a)') '      module umodkk'
      write(7,'(a)') ' '
      if (iqprec == 1) then
       decl='      complex('//trim(adjustl(indq))//') ::'
      else
       decl='      complex('//trim(adjustl(indp))//') ::'
      end if
      ifrst=1
      icon=0
      icon0=0
      rest=''
      do i=1,ipl
       if (pl(i)%wr == 1) then
        read(21,'(a)',rec=i) clx
        call wrdecls(7,clx)
       end if
      end do

      write(7,'(a)') trim(declnd)
      write(7,'(a)') ' '
      write(7,'(a)') '      end module umodkk'
      close(7)
ckk<
      close(21)

      icon1=0
      icony=0
c      clx=''
      if (iqprec == 1) then
       cdec='      complex('//trim(adjustl(indq))//')::'
      else
       cdec='      complex('//trim(adjustl(indp))//')::'
      end if
      ccon='     &'
      icdec=len_trim(cdec)
      iccon=len_trim(ccon)

      rewind 111

      do i=1,id
       read(111,'(3(i16))') sgn,l,lc
       if (icony == icon1) then
        icont=0
        icon0=0
        iconx=0
        clx=''
        slx=''
        icon1=icon1+1
        write(ind1,'(i16)') icon1
        open(unit=10,file='mdcskk'//trim(adjustl(ind1))//'.f',
     &                            status='replace',action='write')
        open(unit=107,file='ampskk'//trim(adjustl(ind1))//'.f',
     &                            status='replace',action='write')
c        write(10,'(a)') '      module mdcskk'//trim(adjustl(ind1)) ! 22 June, 2020
c        write(10,'(a)') ' '
        write(ind,'(i16)') lc
        write(107,3020) trim(adjustl(ind)),trim(adjustl(ind))
       end if

       if (iconx == icon0) then
        icl0=icdec
        icl=icdec+len_trim(clx)
        cl(1:icl)=trim(cdec)//trim(clx)
        isl0=iccon
        isl=isl0+len_trim(slx)
        sl(1:isl)=trim(ccon)//trim(slx)
        icon0=icon0+1
       end if
       write(ind,'(i16)') i
       if (i == id) then
        clx='m'//trim(adjustl(ind))
       else
        clx='m'//trim(adjustl(ind))//','
       end if
       if (sgn > 0) then
        slx='+m'//trim(adjustl(ind))
       else
        slx='-m'//trim(adjustl(ind))
       end if

       icl0=icl
       icl=icl+len_trim(clx)
       isl0=isl
       isl=isl+len_trim(slx)
       if (icl > 72) then
        icl=icl0
        isl=isl0
        if ((icont+1)/mxncnt == icon0) then
         write(10,'(a)') cl(1:icl-1)
         write(10,'(a)') ' '
         write(107,'(a)') sl(1:isl)
         write(ind,'(i16)') lc
         write(107,3020) trim(adjustl(ind)),trim(adjustl(ind))
 3020    format('      mat(',a,')=mat(',a,')')
        else
         write(10,'(a)') cl(1:icl)
         write(107,'(a)') sl(1:isl)
        end if
        if (i == id) then
c         if (icont == icon0) then            ! Modified on
         if ((icont+1)/mxncnt == icon0) then  ! 8 Oct., 2007
          icl=icdec+len_trim(clx)
          cl(1:icl)=trim(cdec)//trim(clx)
          isl=iccon+len_trim(slx)
          sl(1:isl)=trim(ccon)//trim(slx)
          write(10,'(a)') cl(1:icl)
          write(107,'(a)') sl(1:isl)
         else
          icl=iccon+len_trim(clx)
          cl(1:icl)=trim(ccon)//trim(clx)
          isl=iccon+len_trim(slx)
          sl(1:isl)=trim(ccon)//trim(slx)
          write(10,'(a)') cl(1:icl)
          write(107,'(a)') sl(1:isl)
         end if
c Modified on 8 Oct., 2007:
c         write(10,'(a)') ' '    ! 22 June, 2020
c         write(10,'(a)') '      end module mdcskk'//trim(adjustl(ind1))
         close(10)
         close(107)
         exit
c End of modification
        else
         icont=icont+1
c Changed on 20 May, 2008:------------------------>
c         icl0=iccon
c         icl=iccon+len_trim(clx)
c         cl(1:icl)=trim(ccon)//trim(clx)
         if (icont/mxncnt == icon0) then
          icl0=icdec
          icl=icdec+len_trim(clx)
          isl0=iccon
          isl=isl0+len_trim(slx)
          cl(1:icl)=trim(cdec)//trim(clx)
          sl(1:isl)=trim(ccon)//trim(slx)
         else
          icl0=iccon
          icl=iccon+len_trim(clx)
          isl0=iccon
          isl=iccon+len_trim(slx)
          cl(1:icl)=trim(ccon)//trim(clx)
          sl(1:isl)=trim(ccon)//trim(slx)
         end if
c--------------------------------------------------<
        end if
       else
        cl(icl0+1:icl)=clx
        sl(isl0+1:isl)=slx
       end if
       iconx=icont/mxncnt

       icony=i/mxcmdl

       if (icony == icon1 .or. i == id) then
        if (cl(icl:icl) == ',') then
         write(10,'(a)') cl(1:icl-1)
        else
         write(10,'(a)') cl(1:icl)
        end if
c        write(10,'(a)') ' '         ! 22 June, 2020
c        write(10,'(a)') '      end module mdcskk'//trim(adjustl(ind1))
        close(10)
        write(107,'(a)') sl(1:isl)
        close(107)
       end if
      end do

      write(11,'(a)') ' '
      write(11,'(a)') '      save first,comif'
      write(11,'(a)')' '

c      open(18,file='calls.txt',access='direct',form='formatted',
c     &                                                     recl=160)

      do i=1,next
       write(ind,'(i12)') i
       cl='      p'//trim(adjustl(ind))//'=p(:,'//trim(adjustl(ind))//
     &                                                         ')'
       write(11,'(a)') trim(cl)
      end do
      write(11,'(a)') ' '

      iphot=0

      do i=1,next
       read(18,'(a)',rec=i) cl2
       if (f(i)%j == 22) then    ! added on 23 July, 2014
        iphot=iphot+1
        write(ind,'(i12)') iphot
        write(11,'(a)') '      if (igauge == '//trim(adjustl(ind))//
     &                                                        ') then'
        write(ind,'(i16)') f(i)%n
        write(11,'(a)') '       do hl=1,'//trim(adjustl(ind))
        write(ind,'(i16)') i
        write(11,'(a)') '        u'//trim(adjustl(ind))//
     &       '(:,hl)=cmplx(p'//trim(adjustl(ind))//',0.d0,dp)'
        write(11,'(a)') '       end do'
        write(11,'(a)') '      else'
        write(11,'(a)') ' '//trim(cl2)
        write(11,'(a)') '      end if'
       else
        write(11,'(a)') trim(cl2)
       end if
      end do

      write(11,'(a)') ' '

      close(18,status='delete')
      close(31,status='delete')


      rewind 14

      icon0=0
      iconx=0

      do i=1,ncls
       if (iconx == icon0) then
        icon0=icon0+1
        write(ind,'(i16)') icon0
        open(unit=4,file='mtel'//trim(adjustl(ind))//'.f',
     &                            status='replace',action='write')
        write(4,'(a)') '      subroutine mtel'//trim(adjustl(ind))
        write(4,'(a)') ' '
        write(4,'(a)') '      use inprms'
        write(4,'(a)') '      use couplsmkk'
        write(4,'(a)') '      use pmodkk'
        write(4,'(a)') '      use umodkk'
        write(4,'(a)') ' '
        write(4,'(a)') '      implicit none'
        write(4,'(a)') ' '
        write(11,'(a)') '      call mtel'//trim(adjustl(ind))
       end if

       read(14,'(a)') cl2

       call change_string(cl2,'iapp'//',','0'//',','0'//',') ! 17 Feb. 2016
       call change_string(cl2,'iaapp'//',','iaapp1'//',','0'//',') ! 17 Feb. 2016

       call wrt72(4,cl2)

       iconx=i/mxcmdl

       if (iconx == icon0 .or. i == ncls) then
        write(4,'(a)') ' '
        write(4,'(a)') '      end subroutine mtel'//trim(adjustl(ind))
        close(4)
       end if
      end do

      close(14,status='delete')

      if (icncl == 0) then
       write(126,'(a)') ' '
       write(126,'(a)') '      end'
       close(126)
      end if

      call endcolsq(26)
      close(26)

      close(70,status='delete')

      write(11,'(a)') ' '

      write(11,'(a)') '      if (first == 1) then'
c      write(11,'($a)') '       nexh=(/'
      if (imcs /= 1) then
       write(11,'(a$)') '       nexh=(/'
       do i=1,next-1
        write(ind,'(i1)') f(i)%n
        write(11,'(a$)') trim(adjustl(ind))//','
       end do
       write(ind,'(i1)') f(next)%n
       write(11,'(a)') trim(adjustl(ind))//'/)'
      end if

      open(unit=5,file='hsubkk.f',status='replace',action='write')

      write(5,'(a)') '      subroutine hsubkk(next,nexh)'
      write(5,'(a)') ' '
      write(5,'(a)') '      use hmodkk'
      write(5,'(a)') ' '
      write(5,'(a)') '      implicit none'
      write(5,'(a)') ' '
      write(5,'(a)') '      integer :: next,nexh(next)'
      write(5,'(a)') ' '


      if (imcs == 1) then
       write(5,'(a)') 'c This subroutine is not empty only if imcs /= 1'
      else
       do i=1,ipl
        if (pl(i)%wr == 1) then
         read(20,'(26(i3))',rec=i) plord
         call wrordrs(5,plord,i)
        end if 
       end do
      end if
      write(5,'(a)') ' '
      write(5,'(a)') '      end subroutine hsubkk'

      close(5)
      close(20,status='delete')
      close(15,status='delete')

      if (imcs /= 1) then
       write(11,'(a)') '       call hsubkk(next,nexh)'
      end if
      write(11,'(a)') '       comif=1.d0/(nhip*nidp*ncip)'
      write(11,'(a)') '       first=0'
      write(11,'(a)') '      end if'
      write(11,*)' '
      write(11,'(a)') '      mat2=0.d0'
      write(11,*)' '
      write(11,'(a)') '      do hl=1,nhel'
      write(11,'(a)') '       mat2=mat2+mpol2(hl)'
      write(11,'(a)') '      end do'
      write(11,*)' '
      write(11,'(a)') '      mat2=comif*mat2'
      write(11,*)' '
      write(11,'(a)') '      end subroutine matrixel'
      close(11)

      open(unit=11,file='mpol2.f',status='replace',action='write')

      write(11,'(a)') '      function mpol2(h)'
c      write(11,'(a)') ' '
c      do i=1,icon1                 ! 22 June, 2020
c       write(ind,'(i16)') i
c       write(11,'(a)') '      use mdcskk'//trim(adjustl(ind))
c      end do
      write(11,*)' '
      write(11,'(a)') '      implicit none'
      write(11,'(a)') ' '
      write(ind,'(i16)') idr
      write(11,'(a)') '      integer, parameter :: rdgs='
     &               //trim(adjustl(ind))
      write(11,'(a)') '      integer :: h,i,k,k0,first=1'
      if (iqprec == 1) then
       write(11,'(a)') '      real('//trim(adjustl(indq))//
     &                 ') :: mpol2,mtmp2'
       write(11,'(a)') '      complex('//trim(adjustl(indq))//
     &                 ') :: mat(rdgs)'
      else
       write(11,'(a)') '      real('//trim(adjustl(indp))//
     &                 ') :: mpol2,mtmp2'
       write(11,'(a)') '      complex('//trim(adjustl(indp))//
     &                 ') :: mat(rdgs)'
      end if
      write(11,'(a)') '      include ''colredm.f'''
      write(11,'(a)') '      save first,ix,jx,iclm,c2fd'
      write(11,'(a)') ' '
      write(11,'(a)') '      if (first == 1) then'

      write(11,'(a)') '       open(unit=30,file=''colinds.dat'','//
     &                'status=''old'',action=''read'')'
      write(11,'(a)') '       do k=1,iclr'
      write(11,'(a)') '        read(30,''(i10,i10)'') ix(k),jx(k)'
      write(11,'(a)') '       end do'
      write(11,'(a)') '       close(30)'
      write(11,'(a)') '       open(unit=30,file=''colfcts.dat'','//
     &                'status=''old'',action=''read'')'
      write(11,'(a)') '       do k=1,iclrx'
      write(11,'(a)') '        read(30,''(i10,e20.12)'') iclm(k),'//
     &                'c2fd(k)'
      write(11,'(a)') '       end do'
      write(11,'(a)') '       close(30)'
      write(11,'(a)') '       first=0'
      write(11,'(a)') '      end if'
      write(11,'(a)') ' '
      if (iqprec == 1) then
       write(11,'(a)') '      mat=(0.q0,0.q0)'
      else
       write(11,'(a)') '      mat=(0.d0,0.d0)'
      end if
      write(11,'(a)') ' '

      rewind 12

      ccon='     &'

c      open(unit=4,file='mdkk.f',status='replace',action='write')

      icon0=0
      iconx=0

      do i=1,id
       if (iconx == icon0) then
        icon0=icon0+1
        write(ind,'(i16)') icon0
        open(unit=4,file='mdkk'//trim(adjustl(ind))//'.f',
     &                            status='replace',action='write')
        if (imcs == 1) then
         write(4,'(a)') '      subroutine mdkk'//trim(adjustl(ind))
     &                          //'(mat,dim)'
        else
         write(4,'(a)') '      subroutine mdkk'//trim(adjustl(ind))
     &                                  //'(mat,dim,h)'
        end if
        write(4,'(a)') ' '
        write(4,'(a)') '      use inprms'
        write(4,'(a)') '      use couplsmkk'
        if (imcs /= 1) then
         write(4,'(a)') '      use hmodkk'
        end if
c        write(4,'(a)') '      use mdcskk'//trim(adjustl(ind)) ! 22 June, 2020
        write(4,'(a)') '      use pmodkk'
        write(4,'(a)') '      use umodkk'
        write(4,'(a)') ' '
        write(4,'(a)') '      implicit none'
        if (imcs /= 1) then
         write(4,'(a)') ' '
         write(4,'(a)') '      integer :: h'
        end if
        write(4,'(a)') '      integer :: dim'
        if (iqprec == 1) then
         write(4,'(a)') '      complex('//trim(adjustl(indq))//
     &                 ') :: mat(dim)'
        else
         write(4,'(a)') '      complex('//trim(adjustl(indp))//
     &                 ') :: mat(dim)'
        end if
        write(4,'(a)') ' '
        write(4,'(a)') '      include ''mdcskk'//trim(adjustl(ind))
     &                        //'.f'''! 22 June, 2020
        write(4,'(a)') ' '
        if (imcs == 1) then
         write(11,'(a)') '      call mdkk'//trim(adjustl(ind))//
     &                         '(mat,rdgs)'
        else
         write(11,'(a)') '      call mdkk'//trim(adjustl(ind))//
     &                         '(mat,rdgs,h)'
        end if
       end if

       read(12,'(a)') cl3

       call change_string(cl3,'iapp'//',','0'//',','0'//',') ! 17 Feb. 2016
       call change_string(cl3,'iaapp'//',','iaapp1'//',','0'//',') ! 17 Feb. 2016

       call wrt72(4,cl3)

       iconx=i/mxcmdl

       if (iconx == icon0 .or. i == id) then
        write(4,'(a)') ' '
        write(4,'(a)') '      include ''ampskk'//trim(adjustl(ind))
     &                        //'.f'''! 23 June, 2020
        write(4,'(a)') ' '
        write(4,'(a)') '      end subroutine mdkk'//trim(adjustl(ind))
        close(4)
       end if
      end do

c      close(4)
      close(12,status='delete')


c Sum matrix elements with the same particle orders and colour factors

c      write(11,*)' '

c      rewind 111

c      allocate (cla(idr),stat=blad)
c      allocate (matnm(idr),stat=blad)
c      allocate (iclx(idr),stat=blad)
c      allocate (iclu(idr),stat=blad)
c
c      do i=1,idr
c       write(ind,'(i16)') i
c       matnm(i)='mtdr'//trim(adjustl(ind))//'.f'
c       open(unit=30,file=trim(matnm(i)),status='replace',action='write')
c       write(30,3010) trim(adjustl(ind))
c 3010  format('      mat(',a,')=')
c       close(30)
c       cla(i)=ccon
c       iclu(i)=0
c      end do

c      do i=1,id
c       read(111,'(3(i16))') sgn,l,lc
c       write(ind1,'(i16)') l
c       if (sgn > 0) then
c        clx=trim(cla(lc))//'+m'//trim(adjustl(ind1))
c       else
c        clx=trim(cla(lc))//'-m'//trim(adjustl(ind1))
c       end if
c       iclx(lc)=len_trim(clx)
c       if (iclx(lc) > 72) then
c        icla=len_trim(cla(lc))
c        open(unit=30,file=trim(matnm(lc)),status='old',
c     &                    position='append',action='write')
c        if (iclu(lc)/19 == 1) then
c         write(ind,'(i16)') lc
c         write(30,3020) trim(adjustl(ind)),trim(adjustl(ind))
c 3020    format('      mat(',a,')=mat(',a,')')
c         iclu(lc)=iclu(lc)-19
c        end if
c        write(30,'(a)') cla(lc)(1:icla)
c        close(30)
c        iclu(lc)=iclu(lc)+1
c        if (sgn > 0) then
c         cla(lc)='     &+m'//trim(adjustl(ind1))
c        else
c         cla(lc)='     &-m'//trim(adjustl(ind1))
c        end if
c       else
c        cla(lc)=clx
c       end if
c       iclx(lc)=len_trim(cla(lc))
c      end do
c
c      do i=1,idr
c       open(unit=30,file=trim(matnm(i)),status='old',
c     &                    position='append',action='write')
c       if (iclu(i)/19 == 1) then
c        write(ind,'(i16)') i
c        write(30,3020) trim(adjustl(ind)),trim(adjustl(ind))
c        iclu(i)=iclu(i)-19
c       end if
c       write(30,'(a)') cla(i)(1:iclx(i))
c       close(30)
c       iclu(i)=iclu(i)+1
c      end do
c
c      deallocate (cla,iclx,stat=blad)
c
c      do i=1,idr
c       write(11,3015) trim(matnm(i))
c 3015  format('      include ''',a,'''')
c      end do
c
c      deallocate (iclu,matnm,stat=blad)

      write(11,'(a)') ' '
      write(11,'(a)') '      mpol2=0.d0'
      write(11,'(a)') '      k0=1'
      write(11,'(a)')' '
      write(11,'(a)') '      do i=1,iclrx'
      write(11,'(a)') '       mtmp2=0.d0'
      write(11,'(a)') '       do k=k0,k0+iclm(i)-1'
      write(11,'(a)') '        if (jx(k) == ix(k)) then'
      write(11,'(a)') '         mtmp2=mtmp2+conjg(mat(ix(k)))'//
     &                '*mat(jx(k))'
      write(11,'(a)') '        else'
      write(11,'(a)') '         mtmp2=mtmp2+2.d0*conjg(mat(ix(k)))'
     &              //'*mat(jx(k))'
      write(11,'(a)') '        end if'
      write(11,'(a)') '       end do'
      write(11,'(a)') '       mpol2=mpol2+c2fd(i)*mtmp2'
      write(11,'(a)') '       k0=k0+iclm(i)'
      write(11,'(a)') '      end do'
      write(11,*) ' '
      write(11,'(a)') '      end function mpol2'

      close(111,status='delete')
c      close(111)
      close(11)
c      close(13)
      close(13,status='delete')

      if (ischnl == 1) then ! 14 Sept. 2020
       if (did > dnk) then             ! 30 Sept. 2010
c      if (did > dnk .or. (nexpl == 4 .and. nisp == 2)) then  ! 17 Jan. 2011
        open(unit=271,file='kinwrt.tmp',status='old',action='read')! 25 Feb. 2010
        do
         read(271,'(a)',end=100) cl
         write(27,'(a)') trim(cl)
        end do
 100    close(271)
       end if                  ! 30 Sept. 2010
      end if ! 14 Sept. 2020

      if (ischnl == 1) then ! 15 Sept. 2020
       if (next > 4) then   ! 17 Jan. 2011
        write(272,'(a)') trim(ntnend)
        write(272,'(a)') ' '
        if (npsub > ntop) then         ! > 5 Oct. 2010
         write(ind1,'(i16)') npsub
         write(ind2,'(i16)') ntop
         write(272,'(a)') '      integer, parameter :: nt'//
     &     trim(adjustl(ind1))//'=nt'//trim(adjustl(ind2))
         write(272,'(a)') ' '
        end if                         ! < 5 Oct. 2010
       end if                ! 17 Jan. 2011
      else
       write(272,'(a)') 'c This module is empty for ischnl /= 1'
       write(272,'(a)') ' '
      end if
      write(272,'(a)') '      end module ntnkin'
      close(272)
      if (ischnl == 1) then ! 14 Sept. 2020
       if (next > 4 .and. .not.newkinrout) then    ! 17 Jan. 2011, 29 Dec. 2021
        write(27,'(a)') '      end if'
       end if         ! 17 Jan. 2011
      end if ! 14 Sept. 2020

      write(27,'(a)') ' '
      write(27,'(a)') '      end'

      close(27)

      if(kinclse) then
       write(28,'(a)') kcls1
       write(28,'(a)') kcls2
      end if
      if (next > 4) then   ! 17 Jan. 2011
       if (ischnl == 1) then ! 14 Sept. 2020
        write(28,'(a)') '      end if'
       end if ! 14 Sept. 2020
      else     ! 17 Jan. 2011
       write(28,'(a)') '      call kinkk1'//
     &            '(ikin,imom,ecm,m,p,x,ndim,dlips)'
      end if   ! 17 Jan. 2011

      write(289,'(a)') '      end if'

      if (ischnl == 2) then
       write(28,'(a)') '      if (ikin <= nkins) then'
       write(28,'(a)') '       call kinschnl'//
     &                 '(ikin,imom,ecm,p,x,ndim,dlips)'
       write(28,'(a)') '      end if'
      end if

      if (itchnl == 1 .and. next > 4 .and. nkint > 0) then
       write(28,'(a)') '      if (ikin > nkins) then'
       if (iqprec == 1) then
        write(28,'(a)') '       call kintt3x2_qp'//
     &            '(ikin,imom,ecm,p,x,ndim,dlips)'
       else
        write(28,'(a)') '       call kintt3x2'//
     &            '(ikin,imom,ecm,p,x,ndim,dlips)'
       end if
       write(28,'(a)') '      end if'
      end if
      if (igchnl == 1 .and. next > 4 .and. nking >= 0) then
       if (igchnl0 == 1) then
        write(28,'(a)') '      if (ikin > nkinst) then'
        write(28,'(a)') '       call kingchnl'//
     &                 '(ikin,imom,ecm,p,x,ndim,dlips)'
        write(28,'(a)') '      end if'
       end if
      end if
      write(28,'(a)') ' '
      write(28,'(a)') '      end'

      close(28)
      close(289)
      close(300)
      close(268)
      if (igchnl == 1) then
       open(unit=286,file='kingtmp.dat',status='old',action='read')
       if (isr /= 1) then           ! 9 March 2021
        if (ikfsr == 0) then
         open(unit=288,file='kfsrinf.dat',status='old',action='read')
         nking0=nking
         nking=nking+nkfsr
        else
         open(unit=268,file='kingtmp4.dat',status='old',action='read')
        end if
       else                         ! 9 March 2021
        nking0=nking
       end if
       write(285,'(a)') ' '
       igf=0

       do i=1,nking
        if (ikfsr == 0) then
         if (i <= nking0) then
          read(286,'(a)') clkg
          open(unit=289,file='kfsrwrg.dat',status='old',action='read')
          iwrg=0
          do 
           read(289,'(a)',end=600) clwrg
           if (index(clkg,trim(adjustl(clwrg))) > 0) then
            iwrg=1
            exit
           end if
          end do
 600      close(289)
          if (iwrg == 0) then
           igf=igf+1
           write(ind,'(i16)') igf
           if (igf == 1) then
            write(285,'(a)') '      if (ikin == nkinst+'
     &                             //trim(adjustl(ind))//') then'
           else
            write(285,'(a)') '      else if (ikin == nkinst+'
     &                           //trim(adjustl(ind))//') then'
           end if
           call wrt72(285,' '//trim(clkg))
           write(285,'(a)') '     &               imom,x,ndim,dlips)'
          end if
         else
          read(288,'(a)') clfsr
          open(unit=289,file='kfsrwrg.dat',status='old',action='read')
          iwrg=0
          do 
           read(289,'(a)',end=700) clwrg
           if (index(clfsr,trim(adjustl(clwrg))) > 0) then
            iwrg=1
            exit
           end if
          end do
 700      close(289)
          if (iwrg == 0) then
           igf=igf+1
           write(ind,'(i16)') igf
           if (igf == 1) then
            write(285,'(a)') '      if (ikin == nkinst+'
     &                             //trim(adjustl(ind))//') then'
           else
            write(285,'(a)') '      else if (ikin == nkinst+'
     &                           //trim(adjustl(ind))//') then'
           end if
           write(285,'(a)')' '//trim(clfsr)
           write(285,'(a)') '     &               imom,x,ndim,dlips)'
          end if
         end if
        else
         read(286,'(a)') clkg
         open(unit=289,file='kfsrwrg.dat',status='old',action='read')
         iwrg=0
         do 
          read(289,'(a)',end=800) clwrg
          if (index(clkg,trim(adjustl(clwrg))) > 0) then
           iwrg=1
           exit
          end if
         end do
 800     close(289)
         if (iwrg == 0) then
          igf=igf+1
          write(ind,'(i16)') igf
          if (igf == 1) then
           write(285,'(a)') '      if (ikin == nkinst+'
     &                             //trim(adjustl(ind))//') then'
          else
           write(285,'(a)') '      else if (ikin == nkinst+'
     &                           //trim(adjustl(ind))//') then'
          end if
          call wrt72(285,' '//trim(clkg))
          write(285,'(a)') '     &               imom,x,ndim,dlips)'
          if (index(clkg,'fsr') > 0) then
c           igf=igf+1
c           write(ind,'(i16)') igf
c           write(285,'(a)') '      else if (ikin == nkinst+'
c     &                            //trim(adjustl(ind))//') then'
           read(268,'(a)') clkg4
           open(unit=289,file='kfsrwrg.dat',status='old',action='read')
           iwrg4=0
           do 
            read(289,'(a)',end=900) clwrg
            if (index(clkg4,trim(adjustl(clwrg))) > 0) then
             iwrg4=1
             exit
            end if
           end do
 900       close(289)
           if (iwrg4 == 0) then
            igf=igf+1
            write(ind,'(i16)') igf
            if (igf == 1) then
             write(285,'(a)') '      if (ikin == nkinst+'
     &                             //trim(adjustl(ind))//') then'
            else
             write(285,'(a)') '      else if (ikin == nkinst+'
     &                           //trim(adjustl(ind))//') then'
            end if
            call wrt72(285,' '//trim(clkg4))
            write(285,'(a)') '     &               imom,x,ndim,dlips)'
           end if
          end if
         end if
        end if
       end do
       write(285,'(a)') '      end if'
       close(286)
       close(288)
      else 
       write(285,'(a)') ' '
       write(285,'(a)') 'c This subroutine is not used if igchnl /= 1'
      end if
      write(285,'(a)') ' '
      write(285,'(a)') '      if (imom == 1) pp=p'
      write(285,'(a)') ' '
      write(285,'(a)') '      end'

      close(285)
      close(268)

      open(unit=17,file='partcls.f',status='replace',action='write')

      if (ischnl == 2) then
       nkm=nkins
      end if
      call wrpartcls(17,next,nisp,nmcs,id,ickm,isr,iwtb,nkm,nphot,nhls)
      close(17)

      if (.not.psubend) then ! 14 May 2010
       open(unit=4,file=trim(psnnm)//'.f',
     &           status='old',position='append',action='write')
       if (next > 4) then    ! 17 Jan. 2011
        if (ischnl == 1) then ! 14 Sept. 2020
         write(4,'(a)') psub1
         write(4,'(a)') '       if (aw(ikin) > 0.d0) then'
         write(4,'(a)') psub2
        end if ! 14 Sept. 2020
       else    ! 17 Jan. 2011
        if (ischnl == 1) then ! 14 Sept. 2020
         write(ind1,'(i16)') id
         write(4,'(a)') '      do ikin=1,'//trim(adjustl(ind1))
         write(4,'(a)') '       if (aw(ikin) > 0.d0) then'
         write(4,'(a)') '        call kinkk1'//
     &             '(ikin,imom,ecm,m,p,x,ndim,dlips)'
        end if ! 14 Sept. 2020
       end if    ! 17 Jan. 2011

       if (ischnl == 1) then ! 14 Sept. 2020
        write(4,'(a)') '        if (dlips > 0.d0) then'
        write(4,'(a)') '         dlinvt=dlinvt+aw(ikin)/dlips'
        write(4,'(a)') '         if (ikin == ikin0) then'
        write(4,'(a)') '          rlips=abs(dlips-dlips0)/dlips'
        write(4,'(a)') '          if (rlips > psnac) then'
        write(4,'(a)') '           iwkin(ikin)=iwkin(ikin)+1'
        write(4,'(a)') '           if (rlips > aw_w(ikin,1)) then'
        write(4,'(a)') '            aw_w(ikin,1)=rlips'
        write(4,'(a)') '            aw_w(ikin,2)=dlips0'
        write(4,'(a)') '            aw_w(ikin,3)=dlips'
        write(4,'(a)') '           end if'
        write(4,'(a)') '          end if'
        write(4,'(a)') '          dlips=dlips0'
        write(4,'(a)') '         end if'
        write(4,'(a)') '         dlinv=dlinv+aw(ikin)/dlips'
        write(4,'(a)') '        end if'
        write(4,'(a)') '       end if'
        write(4,'(a)') '      end do'
       end if ! 14 Sept. 2020

       if (ischnl == 2) then ! 20 May 2021
        write(4,'(a)') '      do ikin=1,nkins'
        write(4,'(a)') '       if (aw(ikin) > 0.d0) then'
        write(4,'(a)') '        call kinschnl'//
     &                   '(ikin,imom,ecm,p,x,ndim,dlips)'
        write(4,'(a)') '        if (dlips > 0.d0) then'
        write(4,'(a)') '         dlinvt=dlinvt+aw(ikin)/dlips'
        write(4,'(a)') '         if (ikin == ikin0) then'
        write(4,'(a)') '          rlips=abs(dlips-dlips0)/dlips'
        write(4,'(a)') '          if (rlips > psnac) then'
        write(4,'(a)') '           iwkin(ikin)=iwkin(ikin)+1'
        write(4,'(a)') '           if (rlips > aw_w(ikin,1)) then'
        write(4,'(a)') '            aw_w(ikin,1)=rlips'
        write(4,'(a)') '            aw_w(ikin,2)=dlips0'
        write(4,'(a)') '            aw_w(ikin,3)=dlips'
        write(4,'(a)') '           end if'
        write(4,'(a)') '          end if'
        write(4,'(a)') '          dlips=dlips0'
        write(4,'(a)') '         end if'
        write(4,'(a)') '         dlinv=dlinv+aw(ikin)/dlips'
        write(4,'(a)') '        end if'
        write(4,'(a)') '       end if'
        write(4,'(a)') '      end do'
       end if






       if (itchnl == 1 .and. next > 4 .and. nkint > 0) then
        write(4,'(a)') '      do ikin=nkins+1,nkinst'
        write(4,'(a)') '       if (aw(ikin) > 0.d0) then'
        if (iqprec == 1) then
         write(4,'(a)') '        call kintt3x2_qp'//
     &                   '(ikin,imom,ecm,p,x,ndim,dlips)'
        else
         write(4,'(a)') '        call kintt3x2'//
     &                   '(ikin,imom,ecm,p,x,ndim,dlips)'
        end if
        write(4,'(a)') '        if (dlips > 0.d0) then'
        write(4,'(a)') '         dlinvt=dlinvt+aw(ikin)/dlips'
        write(4,'(a)') '         if (ikin == ikin0) then'
        write(4,'(a)') '          rlips=abs(dlips-dlips0)/dlips'
        write(4,'(a)') '          if (rlips > psnac) then'
        write(4,'(a)') '           iwkin(ikin)=iwkin(ikin)+1'
        write(4,'(a)') '           if (rlips > aw_w(ikin,1)) then'
        write(4,'(a)') '            aw_w(ikin,1)=rlips'
        write(4,'(a)') '            aw_w(ikin,2)=dlips0'
        write(4,'(a)') '            aw_w(ikin,3)=dlips'
        write(4,'(a)') '           end if'
        write(4,'(a)') '          end if'
        write(4,'(a)') '          dlips=dlips0'
        write(4,'(a)') '         end if'
        write(4,'(a)') '         dlinv=dlinv+aw(ikin)/dlips'
        write(4,'(a)') '        end if'
        write(4,'(a)') '       end if'
        write(4,'(a)') '      end do'
       end if

       if (igchnl == 1 .and. next > 4 .and. nking >= 0 .and. 
     &                                           igchnl0 == 1) then ! 5 Dec. 2019
        write(4,'(a)') '      do ikin=nkinst+1,nkin'
        write(4,'(a)') '       if (aw(ikin) > 0.d0) then'
        write(4,'(a)') '        call kingchnl'//
     &                   '(ikin,imom,ecm,p,x,ndim,dlips)'
        write(4,'(a)') '        if (dlips > 0.d0) then'
        write(4,'(a)') '         dlinvt=dlinvt+aw(ikin)/dlips'
        write(4,'(a)') '         if (ikin == ikin0) then'
        write(4,'(a)') '          rlips=abs(dlips-dlips0)/dlips'
        write(4,'(a)') '          if (rlips > psnac) then'
        write(4,'(a)') '           iwkin(ikin)=iwkin(ikin)+1'
        write(4,'(a)') '           if (rlips > aw_w(ikin,1)) then'
        write(4,'(a)') '            aw_w(ikin,1)=rlips'
        write(4,'(a)') '            aw_w(ikin,2)=dlips0'
        write(4,'(a)') '            aw_w(ikin,3)=dlips'
        write(4,'(a)') '           end if'
        write(4,'(a)') '          end if'
        write(4,'(a)') '          dlips=dlips0'
        write(4,'(a)') '         end if'
        write(4,'(a)') '         dlinv=dlinv+aw(ikin)/dlips'
        write(4,'(a)') '        end if'
        write(4,'(a)') '       end if'
        write(4,'(a)') '      end do'
       end if
       
       write(4,'(a)') ' '
       write(4,'(a)') '      end subroutine '//trim(psnnm)
       close(4)
      end if

      write(29,'(a)') ' '
      write(29,'(a)') '      end'

      close(29)
ckkbla>
c       close(987)
c       close(988)
c       close(989)
ckkbla<

      contains

       subroutine wrordrs(iu,plorx,ix)
        integer :: iu,plorx(next),ix,ii,jj,iio,jjo,iorx,plox(nexpl)

        iorx=0

        do iio=ix-1,1,-1
         if (pl(iio)%wr == 1) then
          plox=0
          read(20,'(26(i3))',rec=iio) plox
          do jjo=1,next
           if (plox(jjo) == plorx(jjo) .and. plorx(jjo) /= 0) then
            if (jjo == next .or. 
     &          (plox(jjo+1) == 0 .and. plorx(jjo+1) == 0)) then
             iorx=iio
            end if
           else
            exit
           end if
          end do
          if (iorx > 0) then
           write(ind1,'(i16)') ix
           write(ind2,'(i16)') iorx
           write(iu,'(a)') '      h'//trim(adjustl(ind1))//'=h'//
     &                                trim(adjustl(ind2))
           exit
          end if
         end if
        end do

        if (iorx == 0) then

        jj=0
        do ii=1,next
         if (plorx(ii) == 0) then
          exit
         else
          jj=jj+1
         end if
        end do
c        write(iu,'($a)') '      call ix'
        write(iu,'(a$)') '      call ix'
        write(ind,'(i16)') jj
c        write(iu,'($a)') trim(adjustl(ind))//'(next,nexh,nhel,(/'
        write(iu,'(a$)') trim(adjustl(ind))//'(next,nexh,nhel,(/'

        do ii=1,jj-1
         if (plorx(ii) < 10) then
          write(iu,10) plorx(ii)
         else
          write(iu,20) plorx(ii)
         end if
        end do
        if (plorx(jj) < 10) then
c         write(iu,'($i1)') plord(jj)
         write(iu,'(i1$)') plord(jj)
        else
c         write(iu,'($i2)') plord(jj)
         write(iu,'(i2$)') plord(jj)
        end if
        write(ind,'(i16)') ix
        write(iu,'(a)') '/),h'//trim(adjustl(ind))//')'

        end if ! iorx

c 10     format($,i1,',')
c 20     format($,i2,',')
 10     format(i1,',',$)
 20     format(i2,',',$)

       end subroutine wrordrs

       subroutine wrdecls(nux,varx)
c----------------------------------------------------------------------c
c Writes declaration defined above 'decl' of varible 'varx' into a file 
c associated with an opened unit 'nux' in a fixed Fortran format, with 
c maximum No of continuation lines of 'mxncnt'
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 19 March, 2010; modified 
c----------------------------------------------------------------------c

      integer :: nux
      character(80) :: varx,dlx

      integer, parameter :: lcon=6,lline=72
      character(lcon), parameter :: con='     &'
      integer :: ldecl,lvarx,ldlx

      save dlx,ldlx,ldecl

      lvarx=len_trim(varx)+1

      if (ifrst == 1) then
       dlx=trim(decl)//trim(rest)
       ldecl=len_trim(decl)
       ldlx=len_trim(dlx)
       if (ldlx+lvarx > lline) then
        write(nux,'(a)') trim(dlx)
        icon=icon+1
        dlx=con//','//trim(varx)
       else
        if (ldlx > ldecl) then
         dlx=trim(dlx)//','//trim(varx)
        else
         dlx=trim(dlx)//' '//trim(varx)
        end if
       end if
       ldlx=len_trim(dlx)
       declnd=trim(dlx)
       ifrst=0
      else
       if (ldlx+lvarx > lline) then
        write(nux,'(a)') trim(dlx)
        if ((icon+1)/mxncnt == icon0+1) then
         icon0=icon0+1
         ifrst=1
         rest=' '//trim(varx)
        end if
        icon=icon+1
        dlx=con//','//trim(varx)
       else
        dlx=trim(dlx)//','//trim(varx)
       end if
       ldlx=len_trim(dlx)
       declnd=trim(dlx)
      end if

c      if ((icon+1)/mxncnt == icon0+1) then
c       icon0=icon0+1
c       ifrst=1
c      end if

      end subroutine wrdecls

      end
