      subroutine genpart(i1,i2,i3,k)
c----------------------------------------------------------------------c
c Generates particles that couple to particles pl(i1) and pl(i2) and
c possibly pl(i3) through a vertex. 
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 13 October, 2006; based on routines genvvv.f and genvvvv.f
c                   modified 20-25 October, 2006
c                            May 2007 (basic change of output writing),
c                            22 May, 2007 (momenta signs modified in 
c                                          the output writing)
c                            28 May, 2007 (wrong call to ggggkk fixed)
c                            29 Sept., 2008 (nffb -> nffba, 
c                                       nbbb -> nbbba, nbbbb -> nbbbba)
c                            7 May, 2009 (bug in the declaration of 
c                                         output of vvsskk corrcted)
c                            11 May, 2009 (calls to routines of scalar
c                                          QED modified)
c                            16 June, 2009 (bug in call sssskk corrected)
c 26 May, 2010:
c array bound mismatches in the assignments for 'pl%ip' and v1,v2,v3
c corrected
c 27 Jan., 2011:
c anomalous Wtb coupling implemented
c 6 May, 2013:
c a bug in implementation of the anomalous Wtb coupling corrected
c 28 Nov., 2013:
c effective NNa (nucleon-nucleon-photon) coupling implemented
c 22-23 January 2014:
c Modifications for imcs=1 introduced
c 20 March 2014:
c redundant variables f1,f2,f3 of type(particle) eliminated
c 25 Sept. 2014:
c a triple pi0-vector meson-vector meson coupling implemented
c 2 March 2015:
c a call to s-dependent width o the vector meson implemented in ppakk.f 
c implemented
c 12 March 2015:
c a call to s-dependent width o the vector meson implemented in pvvkk.f 
c and pppvkk.f implemented
c 24 March, 2015:
c a call to ppakk.f, papkk.f and appkk.f adapted for charged and neutral
c kaons
c 26 March, 2015:
c an if block that prevents two particles with mixing to be joined into
c a particle added
c 1 April, 2015:
c calls to s-dependent coupling in vvsskk, vvssmd, vsvskk, vsvsmd,
c vssvkk, vssvmd, svvskk, svvsmd, ssvvkk, ssvvmd, svsvkk and svsvmd
c implemented,
c calls to s-dependent coupling and width in vssvkk, ssvvkk and svsvkk
c implemented
c 7 April, 2015:
c calls to pvvkk, pvvmd, vvpkk and vvpmd corrected
c 19 May, 2015:
c zero size arrays pl(i)%im allocated
c 14 Oct. 2015:
c a criterion for the ISR and FSR added; a flag 'isr'
c 8 Jan. 2016:
c pl(k)%k(1) assigned for the gluon quartic vertex
c 8 Nov. 2018: calls to routines with quadrple precision four momenta 
c              implemented
c 29 Aug. 2019: a call to ffs_qp introduced
c 3 March 2020: calls to ppa_qp, pap_qp, app_qp, ssvv_qp, vvss_qp, 
c vsvs_qp, vssv_qp, svvs_qp, ppamd_qp, papmd_qp, appmd_qp, ssvvmd_qp, 
c vvssmd_qp, vsvsmd_qp, vssvmd_qp and svvsmd_qp introduced
c 5 March 2020: calls to svsv_qp, svsvmd_qp introduced
c 6-9 March 2020: calls to ann_qp, annmd_qp, btwan_qp, btwmd_qp, bwtmd_qp,
c twbmd_qp, fsf_qp, fvfan_qp, gggg_qp, nan_qp, nanmd_qp, nna_qp, 
c nnamd_qp, pppv_qp, pppvmd_qp, pvv_qp, pvvmd_qp, sff_qp, sss_qp, 
c ssss_qp, svv_qp, tbwan_qp, tbwmd_qp, vffan_qp, wbtmd_qp, vppp_qp,
c vpppmd_qp, vsv_qp, vvp_qp, vvpmd_qp, vvs_qp, vvv_qp, vvvmd_qp,
c vvvv_qp, wtbmd_qp introduced
c 22-23 Feb. 2021: calls to quadruple precison versions of the helicity 
c amplitudes computation routines added
c----------------------------------------------------------------------c

      use mytypes
      use maxvals
      use vertices
      use extpart
      use intpart
      use tchkins

      implicit none

      integer :: i1,i2,i3,i,j1,j2,j3,j4,k,k0,oif1,oif2,oif3,blad2,
     &           l,m,d1,d2,d3,nhiggs,npion,npi0,
     &           f1j,f1i,f1o,f1g,f1f,f1ct,f1n,f1e,
     &           f2j,f2i,f2o,f2g,f2f,f2ct,f2n,f2e,
     &           f3j,f3i,f3o,f3g,f3f,f3ct,f3n,f3e,
     &           plj,pli,plo,plg,plf,plct,pln,ple,
     &           first=1,abbbb(4),ibbbb(4),obbbb(4)

      integer, allocatable :: v1(:),v2(:),v3(:),
     &                        f1ix(:),f2ix(:),f3ix(:),plix(:)

      character(80) ind,ind1,ind2,ind3,cl,ci1,ci2,ci3,ci4,
     &              cn1,cn2,cn3,cn,indp,cq,ind0,cn0
      character(160) cl1,cl2
      character(2) cp1,cp2,cp3,cp0
      save first

      if (i3 == 0) then     ! Added on 26 March, 2015
       if(pl(i1)%mx+pl(i2)%mx > 3) return
      else
       if(pl(i1)%mx+pl(i2)%mx+pl(i3)%mx > 3) return
      end if

c      write(*,*)'genpart:',i1,i2,i3
c      write(*,*)'size:',size(pl(i1)%ip)
c      write(*,*)'in pl(',i1,'):',pl(i1)%ip
c      write(*,*)'in pl(',i2,'):',pl(i2)%ip
c      if (i3 > 0) then
c       write(*,*)'in pl(',i3,'):',pl(i3)%ip
c      end if

      open(16,file='partcls.tmp',access='direct',form='formatted',
     &                                                     recl=80)
      open(22,file='mdclsb.txt',
     &        access='direct',form='formatted',recl=160)

      read(16,1612,rec=i1) f1j,f1i,f1o,f1g,f1f,f1ct,f1n,f1e
      if (i2 > 0) then         ! 27 March 2014
       read(16,1612,rec=i2) f2j,f2i,f2o,f2g,f2f,f2ct,f2n,f2e
      end if

 1612 format(i8,5(i3),i24,i3)

c      if (isr == 1) then       ! added on 14 Oct. 2015
      if (isr == 1 .and. i3 == 0) then                    ! 18 Feb. 2016
       if (i1 <= next .and. f1j == 22) then
c        if (i2 > nisp .and. f2o > 0) then
        if (i2 > nisp .and. 
     &        (f2o > 0 .or. (f2g == 0 .and. f2i*f2o > 0))) then ! 18 Feb. 2016
         close(16)
         close(22)
         return
        end if
       else if (i2 <= next .and. f2j == 22) then
c        if (i1 > nisp .and. f1o > 0) then
        if (i1 > nisp .and. 
     &        (f1o > 0 .or. (f1g == 0 .and. f1i*f1o > 0))) then ! 18 Feb. 2016
         close(16)
         close(22)
         return
        end if
       end if
      else if (isr == 2 .and. i3 == 0) then
       if (i1 <= next .and. f1j == 22) then
        if (i2 <= nisp .or. 
     &      .not.(f2o > 0 .or. (f2g == 0 .and. f2i*f2o > 0))) then ! 18 Feb. 2016
         close(16)
         close(22)
         return
        end if
       else if (i2 <= next .and. f2j == 22) then
        if (i1 <= nisp .or. 
     &      .not.(f1o > 0 .or. (f1g == 0 .and. f1i*f1o > 0))) then ! 18 Feb. 2016
         close(16)
         close(22)
         return
        end if
       end if
      end if


      allocate(f1ix(f1e),stat=blad2)
      allocate(f2ix(f2e),stat=blad2)

      read(20,1620,rec=i1) f1ix
      if (i2 > 0) then         ! 27 March 2014
       read(20,1620,rec=i2) f2ix
      end if

 1620 format(26(i3))

      if (first == 1) then
       do i=1,next
        pl(i)%mx=0
        pl(i)%dm=0
       end do
       first=0
      end if

      k0=k

c      f1=pl(i1)
c      f2=pl(i2)

      if (i3 == 0) then

      if (i2 > 0) then

      if (f1j < 21) then            ! f1 is a fermion
       if (f2j < 21) then           ! f2 is a fermion too
        oif1=f1o*f1i*f1j
        oif2=f2o*f2i*f2j

        do i=1,nffba
         if ((oif1 == ffb(i,1) .and. oif2 == ffb(i,2))
     &       .or. (oif2 == ffb(i,1) .and. oif1 == ffb(i,2))) then

          k=k+1
          if (max(pl(i1)%mx,pl(i2)%mx) == 2) then
           pl(k)%mx=2
          else
           pl(k)%mx=0
          end if
          pl(k)%k(1)=i
          plj=abs(ffb(i,3))
          plo=-1                ! an internal boson is always incoming 
                                ! to the vertex
          pli=ffb(i,3)/(plo*plj)
c          if (plj == 24 .or. plj == 211) then
          call conveyfg(plj,plg,plf)
          if (plf < 0) then
           plo=-plo           ! therefore a charged boson should be 
                              ! considered as 
                              ! the outgoing particle from the other vertex
          end if
          plct=cffb(i,3)
          pln=f1n*f2n
          ple=f1e+f2e
          call convert
          if (oif1 > 0) then
           write(ind,'(i16)') k
           write(ind1,'(i16)') pln
           write(ci1,'(i16)') i1
           write(cn1,'(i16)') f1n
           write(ci2,'(i16)') i2
           write(cn2,'(i16)') f2n
           if (f1o > 0) then
            cp1='p'
           else
            cp1='-p'
           end if
           if (f2o > 0) then
            cp2='p'
           else
            cp2='-p'
           end if

           cq=trim(cp1)//trim(adjustl(ci1))//
     &            '+('//trim(cp2)//trim(adjustl(ci2))//')'

           if (abs(plf) == 1) then
            cl='u'//trim(adjustl(ind))//'(1,'//trim(adjustl(ind1))//')' 
            if (iqprec == 1) then
             cl1='      call ffs_qp('
             cl2='      call ffsmd_qp('
            else
             cl1='      call ffskk('
             cl2='      call ffsmd('
            end if
           else
            cl='u'//trim(adjustl(ind))//'(0:3,'//
     &                                        trim(adjustl(ind1))//')'
            if (f1j == 9 .or. f1j == 10) then
             write(indp,'(i16)') f2i   ! +1 particle, -1 antiparticle
             if (iqprec == 1) then
              cl1='      call nna_qp('//trim(adjustl(indp))//','//
     &           trim(mffb(i,1))//','
              cl2='      call nnamd_qp('//trim(adjustl(indp))//','
     &                   //trim(mffb(i,1))//','//trim(adjustl(cq))//','
             else
              cl1='      call nnakk('//trim(adjustl(indp))//','//
     &           trim(mffb(i,1))//','
              cl2='      call nnamd('//trim(adjustl(indp))//','
     &                   //trim(mffb(i,1))//','//trim(adjustl(cq))//','
             end if
            else
             if (iqprec == 1) then
              cl1='      call ffv_qp('
              cl2='      call ffvmd_qp('
             else
              cl1='      call ffvkk('
              cl2='      call ffvmd('
             end if
            end if
            if (iwtb == 1) then
             if (f1j == 5 .and. f2j == 6) then
              if (iqprec == 1) then
               cl1='      call btwan_qp('
               cl2='      call btwmd_qp('//trim(adjustl(cq))//','
              else
               cl1='      call btwan('
               cl2='      call btwmd('//trim(adjustl(cq))//','
              end if
             else if (f1j == 6 .and. f2j == 5) then
              if (iqprec == 1) then
               cl1='      call tbwan_qp('
               cl2='      call tbwmd_qp('//trim(adjustl(cq))//','
              else
               cl1='      call tbwan('
               cl2='      call tbwmd('//trim(adjustl(cq))//','
              end if
             end if
            end if
           end if
           write(21,'(a)',rec=k) cl
           cl1=trim(cl1)//'u'//trim(adjustl(ci1))//','//
     &        trim(adjustl(cn1))//','//trim(cp1)//trim(adjustl(ci1))//
     &        ','//'u'//trim(adjustl(ci2))//','//
     &        trim(adjustl(cn2))//','//trim(cp2)//trim(adjustl(ci2))//
     &        ','//'u'//trim(adjustl(ind))//','//trim(adjustl(ind1))//
     &        ','//'p'//trim(adjustl(ind))//','//trim(gffb(i))//','//
     &        trim(mffb(i,3))//','//trim(wffb(i,3))//')'

           write(18,'(a)',rec=k) trim(cl1)
           write(31,'(a10,a10)',rec=k) trim(mffb(i,3)),trim(wffb(i,3))
           write(ind1,'(i16)') i1
           write(ind2,'(i16)') i2

           allocate (plix(ple),stat=blad2)
           plix(:f1e)=f1ix
           plix(f1e+1:)=f2ix
c           call chckord
           write(20,1620,rec=k) plix

c           write(inh1,'(i16)') pl(i1)%k(0)
c           write(inh2,'(i16)') pl(i2)%k(0)

           if (imcs == 1) then
            write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//',u'//trim(adjustl(ind2))//','
           else
            write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),'
           end if
           pl(k)%k(2)=i1
           pl(k)%k(3)=i2
           pl(k)%k(4)=0
           call colinf(i1,i2,k)
c           end if
          else 
           write(ind,'(i16)') k
           write(ind1,'(i16)') pln
           write(ci1,'(i16)') i2
           write(cn1,'(i16)') f2n
           write(ci2,'(i16)') i1
           write(cn2,'(i16)') f1n
           if (f2o > 0) then
            cp1='p'
           else
            cp1='-p'
           end if
           if (f1o > 0) then
            cp2='p'
           else
            cp2='-p'
           end if

           cq=trim(cp1)//trim(adjustl(ci1))//
     &            '+('//trim(cp2)//trim(adjustl(ci2))//')'
           write(ind,'(i16)') k
           write(ind1,'(i16)') pln

           if (abs(plf) == 1) then
            cl='u'//trim(adjustl(ind))//'(1,'//trim(adjustl(ind1))//')'
            if (iqprec == 1) then
             cl1='      call ffs_qp('
             cl2='      call ffsmd_qp('
            else
             cl1='      call ffskk('
             cl2='      call ffsmd('
            end if
           else
            cl='u'//trim(adjustl(ind))//'(0:3,'//trim(adjustl(ind1))
     &                                                          //')'
            if (f1j == 9 .or. f1j == 10) then
             write(indp,'(i16)') f1i   ! +1 particle, -1 antiparticle
             if (iqprec == 1) then
              cl1='      call nna_qp('//trim(adjustl(indp))//','//
     &           trim(mffb(i,1))//','
              cl2='      call nnamd_qp('//trim(adjustl(indp))//','
     &                   //trim(mffb(i,1))//','//trim(adjustl(cq))//','
             else
              cl1='      call nnakk('//trim(adjustl(indp))//','//
     &           trim(mffb(i,1))//','
              cl2='      call nnamd('//trim(adjustl(indp))//','
     &                   //trim(mffb(i,1))//','//trim(adjustl(cq))//','
             end if
            else
             if (iqprec == 1) then
              cl1='      call ffv_qp('
              cl2='      call ffvmd_qp('
             else
              cl1='      call ffvkk('
              cl2='      call ffvmd('
             end if
            end if
            if (iwtb == 1) then
             if (f1j == 5 .and. f2j == 6) then
              if (iqprec == 1) then
               cl1='      call tbwan_qp('
               cl2='      call tbwmd_qp('//trim(adjustl(cq))//','
              else
               cl1='      call tbwan('
               cl2='      call tbwmd('//trim(adjustl(cq))//','
              end if
             else if (f1j == 6 .and. f2j == 5) then
              if (iqprec == 1) then
               cl1='      call btwan_qp('
               cl2='      call btwmd_qp('//trim(adjustl(cq))//','
              else
               cl1='      call btwan('
               cl2='      call btwmd('//trim(adjustl(cq))//','
              end if
             end if
            end if
           end if
           write(21,'(a)',rec=k) cl

           cl1=trim(cl1)//'u'//trim(adjustl(ci1))//','//
     &        trim(adjustl(cn1))//','//trim(cp1)//trim(adjustl(ci1))//
     &        ','//'u'//trim(adjustl(ci2))//','//
     &        trim(adjustl(cn2))//','//trim(cp2)//trim(adjustl(ci2))//
     &        ','//'u'//trim(adjustl(ind))//','//trim(adjustl(ind1))//
     &        ','//'p'//trim(adjustl(ind))//','//trim(gffb(i))//','//
     &        trim(mffb(i,3))//','//trim(wffb(i,3))//')'

           write(18,'(a)',rec=k) cl1
           write(31,'(a10,a10)',rec=k) trim(mffb(i,3)),trim(wffb(i,3))

           write(ind1,'(i16)') i2
           write(ind2,'(i16)') i1

           allocate (plix(ple),stat=blad2)
           plix(:f2e)=f2ix
           plix(f2e+1:)=f1ix
c           call chckord
c           write(inh1,'(i16)') pl(i2)%k(0)
c           write(inh2,'(i16)') pl(i1)%k(0)

           if (imcs == 1) then
            write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//',u'//trim(adjustl(ind2))//','
           else
            write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),'
           end if
           write(20,1620,rec=k) plix
           pl(k)%k(2)=i2
           pl(k)%k(3)=i1
           pl(k)%k(4)=0
           call colinf(i2,i1,k)
c           end if
          end if
c          call convert
         end if
        end do

       else if (f2j > 20) then   ! f2 is a boson
        oif1=f1o*f1i*f1j
        oif2=f2o*f2i*f2j
        do i=1,nffba
         if (oif1 == ffb(i,1) .and. oif2 == ffb(i,3)) then
          k=k+1
          if (max(pl(i1)%mx,pl(i2)%mx) == 2) then
           pl(k)%mx=2
          else
           pl(k)%mx=0
          end if
          pl(k)%k(1)=i
          pli=f1i
          plo=f1o
          plj=abs(ffb(i,2))
          plct=cffb(i,2)
          pln=f1n*f2n
          ple=f1e+f2e
          call conveyfg(plj,plg,plf)
          call convert

          write(ind,'(i16)') k
          write(ind1,'(i16)') pln
          write(ind2,'(i16)') pli
          write(ci1,'(i16)') i1
          write(cn1,'(i16)') f1n
          write(ci2,'(i16)') i2
          write(cn2,'(i16)') f2n

c          if (f2o == 0) then
c           if (f1o > 0) then
c            cp2='p'
c           else
c            cp2='-p'
c           end if
c          else
c           if (f1o*f2o < 0) then
c            cp2='-p'
c           else
c            cp2='p'
c           end if
c          end if

          cp1='p'
          if (i2 > nexpl) then
           if (f1o > 0) then
            cp2='p'
           else
            cp2='-p'
           end if
          else
           if (f1o*f2o > 0) then
            cp2='p'
           else
            cp2='-p'
           end if
          end if


c          write(18,3,rec=k) gffb(i),i1,i2,k
c 3        format('      call  fvkk(',a4,',',i6,',',i6,',',i8,')')

          write(ind,'(i16)') k
          write(ind1,'(i16)') pln
          cl='u'//trim(adjustl(ind))//'(4,'//trim(adjustl(ind1))//')'
          write(21,'(a)',rec=k) cl

          if (abs(f2f) == 1) then
           if (iqprec == 1) then
            cl1='      call fsf_qp('
            cl2='      call fsfmd_qp('
           else
            cl1='      call fsfkk('
            cl2='      call fsfmd('
           end if
          else
           if (f1j == 9 .or. f1j == 10) then
            if (iqprec == 1) then
             cl1='      call nan_qp(' ! particle index transferred as ind2
             cl2='      call nanmd_qp('//trim(adjustl(ind2))//','//
     &        trim(mffb(i,2))//','//trim(cp2)//trim(adjustl(ci2))//',' 
            else
             cl1='      call nankk(' ! particle index transferred as ind2
             cl2='      call nanmd('//trim(adjustl(ind2))//','//
     &        trim(mffb(i,2))//','//trim(cp2)//trim(adjustl(ci2))//',' 
            end if
           else
            if (iqprec == 1) then
             cl1='      call fvf_qp('
             cl2='      call fvfmd_qp('
            else
             cl1='      call fvfkk('
             cl2='      call fvfmd('
            end if
           end if
           if (iwtb == 1) then
            if (f1j == 5 .and. f2j == 24) then
             if (iqprec == 1) then
              cl1='      call fvfan_qp('
              cl2='      call bwtmd_qp('//trim(cp2)//trim(adjustl(ci2))
     &           //','
             else
              cl1='      call fvfan('
              cl2='      call bwtmd('//trim(cp2)//trim(adjustl(ci2))
     &           //','
             end if
            else if (f1j == 6 .and. f2j == 24) then
             if (iqprec == 1) then
              cl1='      call fvfan_qp('
              cl2='      call twbmd_qp('//trim(cp2)//trim(adjustl(ci2))
     &           //','
             else
              cl1='      call fvfan('
              cl2='      call twbmd('//trim(cp2)//trim(adjustl(ci2))
     &           //','
             end if
            end if
           end if
          end if
          cl1=trim(cl1)//'u'//trim(adjustl(ci1))//','//
     &        trim(adjustl(cn1))//','//trim(cp1)//trim(adjustl(ci1))//
     &        ','//'u'//trim(adjustl(ci2))//','//
     &        trim(adjustl(cn2))//','//trim(cp2)//trim(adjustl(ci2))//
     &        ','//'u'//trim(adjustl(ind))//','//trim(adjustl(ind1))//
     &        ','//'p'//trim(adjustl(ind))//','//trim(adjustl(ind2))//
     &        ','//trim(gffb(i))//','//
     &        trim(mffb(i,2))//','//trim(wffb(i,2))//')'

          write(18,'(a)',rec=k) cl1
          write(31,'(a10,a10)',rec=k) trim(mffb(i,2)),trim(wffb(i,2))

          write(ind1,'(i16)') i1
          write(ind2,'(i16)') i2

          allocate (plix(ple),stat=blad2)
          plix(:f1e)=f1ix
          plix(f1e+1:)=f2ix
c          call chckord
c          write(inh1,'(i16)') pl(i1)%k(0)
c          write(inh2,'(i16)') pl(i2)%k(0)

          if (imcs == 1) then
           write(22,'(a)',rec=k) trim(cl2)
     &             //trim(gffb(i))//',u'//
     &             trim(adjustl(ind1))//',u'//trim(adjustl(ind2))//','
          else
           write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),'
          end if
          write(20,1620,rec=k) plix
          pl(k)%k(2)=i1
          pl(k)%k(3)=i2
          pl(k)%k(4)=0
          call colinf(i1,i2,k)
         else if (oif1 == ffb(i,2) .and. oif2 == ffb(i,3)) then
          k=k+1
          if (max(pl(i1)%mx,pl(i2)%mx) == 2) then
           pl(k)%mx=2
          else
           pl(k)%mx=0
          end if
          pl(k)%k(1)=i
          pli=f1i
          plo=f1o
          plj=abs(ffb(i,1))
          plct=cffb(i,1)
          pln=f1n*f2n
          ple=f1e+f2e
          call conveyfg(plj,plg,plf)
          call convert

c          write(18,5,rec=k) gffb(i),i2,i1,k
c 5        format('      call  vfkk(',a4,',',i6,',',i6,',',i8,')')

          write(ind,'(i16)') k
          write(cn3,'(i16)') pln
          write(ci1,'(i16)') pli
          write(ind1,'(i16)') i2
          write(cn1,'(i16)') f2n
          write(ind2,'(i16)') i1
          write(cn2,'(i16)') f1n
c          if (f1o == 0) then
c           if (f2o > 0) then
c            cp1='p'
c           else
c            cp1='-p'
c           end if
c          else
c           if (f1o*f2o < 0) then
c            cp1='-p'
c           else
c            cp1='p'
c           end if
c          end if
          cp2='p'
          if (i2 > nexpl) then
           if (f1o > 0) then
            cp1='p'
           else
            cp1='-p'
           end if
          else
           if (f1o*f2o > 0) then
            cp1='p'
           else
            cp1='-p'
           end if
          end if
          if (abs(f2f) == 1) then
           if (iqprec == 1) then
            cl1='      call sff_qp('
            cl2='      call sffmd_qp('
           else
            cl1='      call sffkk('
            cl2='      call sffmd('
           end if
          else
           if (f1j == 9 .or. f1j == 10) then
            if (iqprec == 1) then
             cl1='      call ann_qp(' ! particle index transferred as ind2
             cl2='      call annmd_qp('//trim(adjustl(ci1))//','//
     &       trim(mffb(i,1))//','//trim(cp1)//trim(adjustl(ind1))//','
            else
             cl1='      call annkk(' ! particle index transferred as ind2
             cl2='      call annmd('//trim(adjustl(ci1))//','//
     &       trim(mffb(i,1))//','//trim(cp1)//trim(adjustl(ind1))//','
            end if
           else
            if (iqprec == 1) then
             cl1='      call vff_qp('
             cl2='      call vffmd_qp('
            else
             cl1='      call vffkk('
             cl2='      call vffmd('
            end if
           end if
           if (iwtb == 1) then
            if (f1j == 5 .and. f2j == 24) then
             if (iqprec == 1) then
              cl1='      call vffan_qp('
              cl2='      call wbtmd_qp('//trim(cp1)//
     &             trim(adjustl(ind1))//','
             else
              cl1='      call vffan('
              cl2='      call wbtmd('//trim(cp1)//
     &             trim(adjustl(ind1))//','
             end if
            else if (f1j == 6 .and. f2j == 24) then
             if (iqprec == 1) then
              cl1='      call vffan_qp('
              cl2='      call wtbmd_qp('//trim(cp1)//
     &             trim(adjustl(ind1))//','
             else
              cl1='      call vffan('
              cl2='      call wtbmd('//trim(cp1)//
     &             trim(adjustl(ind1))//','
             end if
            end if
           end if
          end if
          write(18,'(a)',rec=k) trim(cl1)//'u'//trim(adjustl(ind1))//','
     &             //trim(adjustl(cn1))//','//trim(cp1)//
     &             trim(adjustl(ind1))//','
     &             //'u'//trim(adjustl(ind2))//','
     &             //trim(adjustl(cn2))//','//trim(cp2)
     &             //trim(adjustl(ind2))//','
     &             //'u'//trim(adjustl(ind))//','
     &             //trim(adjustl(cn3))//',p'//trim(adjustl(ind))//','
     &             //trim(adjustl(ci1))//','//trim(gffb(i))//','//
     &             trim(mffb(i,1))//','//trim(wffb(i,1))//')'

          write(31,'(a10,a10)',rec=k) trim(mffb(i,1)),trim(wffb(i,1))

          allocate (plix(ple),stat=blad2)
          plix(:f2e)=f2ix
          plix(f2e+1:)=f1ix
c          call chckord
c          write(inh1,'(i16)') pl(i2)%k(0)
c          write(inh2,'(i16)') pl(i1)%k(0)

          if (imcs == 1) then
           write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//',u'//trim(adjustl(ind2))//','
          else
           write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),'
          end if
          write(ind,'(i16)') k
          write(ind1,'(i16)') pln
          cl='u'//trim(adjustl(ind))//'(4,'//trim(adjustl(ind1))//')'
          write(21,'(a)',rec=k) cl

          write(20,1620,rec=k) plix
          pl(k)%k(2)=i2
          pl(k)%k(3)=i1
          pl(k)%k(4)=0
          call colinf(i2,i1,k)
c          call convert
         end if
        end do
c Tu wyciąłem
       end if
      else if (f1j > 20) then    ! f1 is a boson
       if (f2j < 21) then        ! f2 is a fermion
        oif1=f1o*f1i*f1j
        oif2=f2o*f2i*f2j
        do i=1,nffba
         if (oif1 == ffb(i,3) .and. oif2 == ffb(i,1)) then
          k=k+1
          if (max(pl(i1)%mx,pl(i2)%mx) == 2) then
           pl(k)%mx=2
          else
           pl(k)%mx=0
          end if
          pl(k)%k(1)=i
          pli=f2i
          plo=f2o
          plj=abs(ffb(i,2))
          plct=cffb(i,2)
          pln=f1n*f2n
          ple=f1e+f2e
          call conveyfg(plj,plg,plf)
          call convert
          write(ind,'(i16)') k
          write(ind1,'(i16)') pln
          write(ind2,'(i16)') pli
          write(ci1,'(i16)') i2
          write(cn1,'(i16)') f2n
          write(ci2,'(i16)') i1
          write(cn2,'(i16)') f1n
c          if (f2o == 0) then
c           if (f1o > 0) then
c            cp2='p'
c           else
c            cp2='-p'
c           end if
c          else
c           if (f1o*f2o < 0) then
c            cp2='-p'
c           else
c            cp2='p'
c           end if
c          end if

          cp1='p'
          if (i1 > nexpl) then
           if (f2o > 0) then
            cp2='p'
           else
            cp2='-p'
           end if
          else
           if (f1o*f2o > 0) then
            cp2='p'
           else
            cp2='-p'
           end if
          end if

c          open(unit=13,status='scratch',action='readwrite')
c          write(13,3) i2,gffb(i),i1,k
c          write(18,3,rec=k) gffb(i),i2,i1,k

          write(ind,'(i16)') k
          write(ind1,'(i16)') pln
          cl='u'//trim(adjustl(ind))//'(4,'//trim(adjustl(ind1))//')'
          write(21,'(a)',rec=k) cl
          if (abs(f1f) == 1) then
           if (iqprec == 1) then
            cl1='      call fsf_qp('
            cl2='      call fsfmd_qp('
           else
            cl1='      call fsfkk('
            cl2='      call fsfmd('
           end if
          else
           if (f2j == 9 .or. f2j == 10) then
            if (iqprec == 1) then
             cl1='      call nan_qp(' ! particle index transferred as ind2
             cl2='      call nanmd_qp('//trim(adjustl(ind2))//','//
     &        trim(mffb(i,2))//','//trim(cp2)//trim(adjustl(ci2))//',' 
            else
             cl1='      call nankk(' ! particle index transferred as ind2
             cl2='      call nanmd('//trim(adjustl(ind2))//','//
     &        trim(mffb(i,2))//','//trim(cp2)//trim(adjustl(ci2))//',' 
            end if
           else
            if (iqprec == 1) then
             cl1='      call fvf_qp('
             cl2='      call fvfmd_qp('
            else
             cl1='      call fvfkk('
             cl2='      call fvfmd('
            end if
           end if
           if (iwtb == 1) then
c            if (f3j == 5 .and. f2j == 24) then 
            if (f2j == 5 .and. f1j == 24) then  ! 6 May, 2013
             if (iqprec == 1) then
              cl1='      call fvfan_qp('
              cl2='      call bwtmd_qp('//trim(cp2)//trim(adjustl(ci2))
     &                                                            //','
             else
              cl1='      call fvfan('
              cl2='      call bwtmd('//trim(cp2)//trim(adjustl(ci2))
     &                                                            //','
             end if
c            else if (f3j == 6 .and. f2j == 24) then
            else if (f2j == 6 .and. f1j == 24) then  ! 6 May, 2013
             if (iqprec == 1) then
              cl1='      call fvfan_qp('
              cl2='      call twbmd_qp('//trim(cp2)//trim(adjustl(ci2))
     &                                                          //','
             else
              cl1='      call fvfan('
              cl2='      call twbmd('//trim(cp2)//trim(adjustl(ci2))
     &                                                          //','
             end if
            end if
           end if
          end if
          cl1=trim(cl1)//'u'//trim(adjustl(ci1))//','//
     &        trim(adjustl(cn1))//','//trim(cp1)//trim(adjustl(ci1))//
     &        ','//'u'//trim(adjustl(ci2))//','//
     &        trim(adjustl(cn2))//','//trim(cp2)//trim(adjustl(ci2))//
     &        ','//'u'//trim(adjustl(ind))//','//trim(adjustl(ind1))//
     &        ','//'p'//trim(adjustl(ind))//','//trim(adjustl(ind2))//
     &        ','//trim(gffb(i))//','//
     &        trim(mffb(i,2))//','//trim(wffb(i,2))//')'

          write(18,'(a)',rec=k) cl1
          write(31,'(a10,a10)',rec=k) trim(mffb(i,2)),trim(wffb(i,2))
          write(ind1,'(i16)') i2
          write(ind2,'(i16)') i1
          allocate (plix(ple),stat=blad2)
          plix(:f2e)=f2ix
          plix(f2e+1:)=f1ix
c          call chckord
c          write(inh1,'(i16)') pl(i2)%k(0)
c          write(inh2,'(i16)') pl(i1)%k(0)

          if (imcs == 1) then
           write(22,'(a)',rec=k) trim(cl2)//trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//',u'//trim(adjustl(ind2))//','
          else
           write(22,'(a)',rec=k) trim(cl2)//trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),'
          end if
c          call printinf(check)
c          if (check) then
          write(20,1620,rec=k) plix
          pl(k)%k(2)=i2
          pl(k)%k(3)=i1
          pl(k)%k(4)=0
          call colinf(i2,i1,k)
c          call convert
c          end if
         else if (oif1 == ffb(i,3) .and. oif2 == ffb(i,2)) then
          k=k+1
          if (max(pl(i1)%mx,pl(i2)%mx) == 2) then
           pl(k)%mx=2
          else
           pl(k)%mx=0
          end if
          pl(k)%k(1)=i
          pli=f2i
          plo=f2o
          plj=abs(ffb(i,1))
          plct=cffb(i,1)
          pln=f1n*f2n
          ple=f1e+f2e

          write(ind,'(i16)') k
          write(cn3,'(i16)') pln
          write(ci1,'(i16)') pli
          write(ind1,'(i16)') i1
          write(cn1,'(i16)') f1n
          write(ind2,'(i16)') i2
          write(cn2,'(i16)') f2n

          if (i1 > nexpl) then
           if (f2o > 0) then
            cp1='p'
           else
            cp1='-p'
           end if
          else
           if (f1o*f2o > 0) then
            cp1='p'
           else
            cp1='-p'
           end if
          end if
          cp2='p'

          if (abs(f1f) == 1) then
           if (iqprec == 1) then
            cl1='      call sff_qp('
            cl2='      call sffmd_qp('
           else
            cl1='      call sffkk('
            cl2='      call sffmd('
           end if
          else
           if (f2j == 9 .or. f2j == 10) then
            if (iqprec == 1) then
             cl1='      call ann_qp(' ! particle index transferred as ind2
             cl2='      call annmd_qp('//trim(adjustl(ci1))//','//
     &       trim(mffb(i,1))//','//trim(cp1)//trim(adjustl(ind1))//','
            else
             cl1='      call annkk(' ! particle index transferred as ind2
             cl2='      call annmd('//trim(adjustl(ci1))//','//
     &       trim(mffb(i,1))//','//trim(cp1)//trim(adjustl(ind1))//','
            end if
           else
            if (iqprec == 1) then
             cl1='      call vff_qp('
             cl2='      call vffmd_qp('
            else
             cl1='      call vffkk('
             cl2='      call vffmd('
            end if
           end if
           if (iwtb == 1) then
            if (f1j == 24 .and. f2j == 5) then
             if (iqprec == 1) then
              cl1='      call vffan_qp('
              cl2='      call wbtmd_qp('//trim(cp1)//trim(adjustl(ind1))
     &         //','
             else
              cl1='      call vffan('
              cl2='      call wbtmd('//trim(cp1)//trim(adjustl(ind1))
     &         //','
             end if
            else if (f1j == 24 .and. f2j == 6) then
             if (iqprec == 1) then
              cl1='      call vffan_qp('
              cl2='      call wtbmd_qp('//trim(cp1)//trim(adjustl(ind1))
     &         //','
             else
              cl1='      call vffan('
              cl2='      call wtbmd('//trim(cp1)//trim(adjustl(ind1))
     &         //','
             end if
            end if
           end if
          end if

          write(18,'(a)',rec=k) trim(cl1)//'u'//trim(adjustl(ind1))//','
     &             //trim(adjustl(cn1))//','//trim(cp1)//
     &             trim(adjustl(ind1))//','
     &             //'u'//trim(adjustl(ind2))//','
     &             //trim(adjustl(cn2))//','//trim(cp2)
     &             //trim(adjustl(ind2))//','
     &             //'u'//trim(adjustl(ind))//','
     &             //trim(adjustl(cn3))//',p'//trim(adjustl(ind))//','
     &             //trim(adjustl(ci1))//','//trim(gffb(i))//','//
     &             trim(mffb(i,1))//','//trim(wffb(i,1))//')'

          write(31,'(a10,a10)',rec=k) trim(mffb(i,1)),trim(wffb(i,1))

          allocate (plix(ple),stat=blad2)
          plix(:f1e)=f1ix
          plix(f1e+1:)=f2ix
c          call chckord
c          write(inh1,'(i16)') pl(i1)%k(0)
c          write(inh2,'(i16)') pl(i2)%k(0)

          if (imcs == 1) then
           write(22,'(a)',rec=k) trim(cl2)//trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//',u'//trim(adjustl(ind2))//','
          else
           write(22,'(a)',rec=k) trim(cl2)//trim(gffb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),'
          end if
          write(ind,'(i16)') k
          write(ind1,'(i16)') pln
          cl='u'//trim(adjustl(ind))//'(4,'//trim(adjustl(ind1))//')'
          write(21,'(a)',rec=k) cl

          write(20,1620,rec=k) plix
          pl(k)%k(2)=i1
          pl(k)%k(3)=i2
          pl(k)%k(4)=0
          call colinf(i1,i2,k)
          call conveyfg(plj,plg,plf)
          call convert
         end if
        end do
       else if (f2j > 20) then         ! f2 is a boson too
        oif1=f1o*f1i*f1j
        oif2=f2o*f2i*f2j
        do i=1,nbbba
         do j1=1,3
          if (oif1 == bbb(i,j1)) then
           do j2=1,3
            if (j1 /= j2 .and. oif2 == bbb(i,j2)) then
             do j3=1,3
              if (j1 /= j3 .and. j2 /= j3) then
               k=k+1
               if (max(pl(i1)%mx,pl(i2)%mx) == 2) then
                pl(k)%mx=2
               else
                pl(k)%mx=0
               end if
               pl(k)%k(1)=i
               plj=abs(bbb(i,j3))
               plo=-1              ! an internal boson is always incoming 
                                   ! to the vertex
               pli=bbb(i,j3)/(plo*plj)

c               if (plj == 24 .or. plj == 211) then
               call conveyfg(plj,plg,plf)
               if (plf < 0) then
                plo=-plo          ! therefore a charged boson should be 
                                  ! considered as 
                                  ! the outgoing particle from the other vertex
               end if
               plct=cbbb(i,j3)
               pln=f1n*f2n
               ple=f1e+f2e
               call convert
               allocate (plix(ple),stat=blad2)
               plix(:f1e)=f1ix
               plix(f1e+1:)=f2ix
c               call chckord

c> Added on 7 April, 2015:
               npion=0
               if (f1j == 211 .or. f1j == 311 .or. f1j == 321) 
     &                                                    npion=npion+1
               if (f2j == 211 .or. f2j == 311 .or. f2j == 321) 
     &                                                    npion=npion+1
               if (plj == 211 .or. plj == 311 .or. plj == 321) 
     &                                                    npion=npion+1
               npi0=0
               if (f1j == 111) npi0=npi0+1
               if (f2j == 111) npi0=npi0+1
               if (plj == 111) npi0=npi0+1
c<

               if (f1j /= 25 .and. f2j /= 25 .and. plj /= 25) then

                write(ind1,'(i16)') i1
                write(ind2,'(i16)') i2
                write(ind3,'(i16)') k
                write(ind,'(i16)') pln
                write(cn1,'(i16)') f1n
                write(cn2,'(i16)') f2n
                write(cn3,'(i16)') pln

                if (i1 > nexpl) then
                 cp1='-p'
                else
                 if (f1o < 0) then
                  cp1='p'
                 else
                  cp1='-p'
                 end if
                end if
                if (i2 > nexpl) then
                 cp2='-p'
                else
                 if (f2o < 0) then
                  cp2='p'
                 else
                  cp2='-p'
                 end if
                end if

c> Changed on 7 April, 2015:
                if (npion == 2) then
c                if (f1j == 211 .or. f2j == 211 .or. plj == 211 .or.
c     &              f1j == 311 .or. f2j == 311 .or. plj == 311 .or.  ! 24.03.15
c     &              f1j == 321 .or. f2j == 321 .or. plj == 321) then ! 24.03.15
c<
                 if (j1 == 1 .and. j2 == 2) then
                  if (iqprec == 1) then
                   cl1='      call ppa_qp(i'//trim(gbbb(i))//
     &                                ',i'//trim(wbbb(i,j3))//',-1,'
                   cl2='      call ppamd_qp(i'//trim(gbbb(i))//',-1,'
                  else
                   cl1='      call ppakk(i'//trim(gbbb(i))//
     &                                ',i'//trim(wbbb(i,j3))//',-1,'
                   cl2='      call ppamd(i'//trim(gbbb(i))//',-1,'
                  end if
                  cl='u'//trim(adjustl(ind3))//'(0:3,'//
     &                                        trim(adjustl(ind))//')'
                  write(21,'(a)',rec=k) cl
                 else if (j1 == 1 .and. j2 == 3) then
                  if (iqprec == 1) then
                   cl1='      call pap_qp(i'//trim(gbbb(i))//',-1,'
                   cl2='      call papmd_qp(i'//trim(gbbb(i))//',-1,'
                  else
                   cl1='      call papkk(i'//trim(gbbb(i))//',-1,'
                   cl2='      call papmd(i'//trim(gbbb(i))//',-1,'
                  end if
                  cl='u'//trim(adjustl(ind3))//'(1,'//
     &                                        trim(adjustl(ind))//')'
                  write(21,'(a)',rec=k) cl
                 else if (j1 == 2 .and. j2 == 1) then
                  if (iqprec == 1) then
                   cl1='      call ppa_qp(i'//trim(gbbb(i))//',i'//
     &                                      trim(wbbb(i,j3))//',1,'
                   cl2='      call ppamd_qp(i'//trim(gbbb(i))//',1,'
                  else
                   cl1='      call ppakk(i'//trim(gbbb(i))//',i'//
     &                                      trim(wbbb(i,j3))//',1,'
                   cl2='      call ppamd(i'//trim(gbbb(i))//',1,'
                  end if
                  cl='u'//trim(adjustl(ind3))//'(0:3,'//
     &                                        trim(adjustl(ind))//')'
                  write(21,'(a)',rec=k) cl
                 else if (j1 == 2 .and. j2 == 3) then
                  if (iqprec == 1) then
                   cl1='      call pap_qp(i'//trim(gbbb(i))//',1,'
                   cl2='      call papmd_qp(i'//trim(gbbb(i))//',1,'
                  else
                   cl1='      call papkk(i'//trim(gbbb(i))//',1,'
                   cl2='      call papmd(i'//trim(gbbb(i))//',1,'
                  end if
                  cl='u'//trim(adjustl(ind3))//'(1,'//
     &                                        trim(adjustl(ind))//')'
                  write(21,'(a)',rec=k) cl
                 else if (j1 == 3 .and. j2 == 1) then
                  if (iqprec == 1) then
                   cl1='      call app_qp(i'//trim(gbbb(i))//',-1,'
                   cl2='      call appmd_qp(i'//trim(gbbb(i))//',-1,'
                  else
                   cl1='      call appkk(i'//trim(gbbb(i))//',-1,'
                   cl2='      call appmd(i'//trim(gbbb(i))//',-1,'
                  end if
                  cl='u'//trim(adjustl(ind3))//'(1,'//
     &                                        trim(adjustl(ind))//')'
                  write(21,'(a)',rec=k) cl
                 else if (j1 == 3 .and. j2 == 2) then
                  if (iqprec == 1) then
                   cl1='      call app_qp(i'//trim(gbbb(i))//',1,'
                   cl2='      call appmd_qp(i'//trim(gbbb(i))//',1,'
                  else
                   cl1='      call appkk(i'//trim(gbbb(i))//',1,'
                   cl2='      call appmd(i'//trim(gbbb(i))//',1,'
                  end if
                  cl='u'//trim(adjustl(ind3))//'(1,'//
     &                                        trim(adjustl(ind))//')'
                  write(21,'(a)',rec=k) cl
                 end if
                 write(18,'(a)',rec=k) trim(cl1)//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &              ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ',p'//trim(adjustl(ind3))//','//trim(gbbb(i))//','//
     &              trim(mbbb(i,j3))//','//trim(wbbb(i,j3))//')'
                 write(31,'(a10,a10)',rec=k) trim(mbbb(i,j3)),
     &                                       trim(wbbb(i,j3))
                 if (imcs == 1) then
                  write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gbbb(i))//',u'//trim(adjustl(ind1))//','
     &              //trim(cp1)//trim(adjustl(ind1))//','//'u'//
     &              trim(adjustl(ind2))//','//trim(cp2)
     &              //trim(adjustl(ind2))//','
                 else
                  write(22,'(a)',rec=k) trim(cl2)//trim(gbbb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),'//trim(cp1)//trim(adjustl(ind1))//','//'u'//
     &              trim(adjustl(ind2))//'(:,h'//trim(adjustl(ind2))//
     &             '(h)),'//trim(cp2)//trim(adjustl(ind2))//','
                 end if
c> Changed on 7 April, 2015:
                else if (npi0 == 1 .or. npion == 1) then
c                else if (f1j == 111 .or. f2j == 111 .or. plj == 111) 
c     &                                                            then
c<
                 if (j3 == 1) then
                  if (iqprec == 1) then
                   cl1='      call vvp_qp(i'//trim(gbbb(i))//','
                   cl2='      call vvpmd_qp(i'//trim(gbbb(i))//','
                  else
                   cl1='      call vvpkk(i'//trim(gbbb(i))//','
                   cl2='      call vvpmd(i'//trim(gbbb(i))//','
                  end if
                  cl='u'//trim(adjustl(ind3))//'(1,'//
     &                                        trim(adjustl(ind))//')'
                  write(21,'(a)',rec=k) cl
                 else
                  if (iqprec == 1) then
                   cl1='      call pvv_qp(i'//trim(gbbb(i))//',i'//
     &                                            trim(wbbb(i,j3))//','
                   cl2='      call pvvmd_qp(i'//trim(gbbb(i))//','
                  else
                   cl1='      call pvvkk(i'//trim(gbbb(i))//',i'//
     &                                            trim(wbbb(i,j3))//','
                   cl2='      call pvvmd(i'//trim(gbbb(i))//','
                  end if
                  cl='u'//trim(adjustl(ind3))//'(0:3,'//
     &                                        trim(adjustl(ind))//')'
                  write(21,'(a)',rec=k) cl
                  if (j2 == 1) then
                   ind0=ind1
                   cn0=cn1
                   cp0=cp1
                   ind1=ind2
                   cn1=cn2
                   cp1=cp2
                   ind2=ind0
                   cn2=cn0
                   cp2=cp0
                  end if
                 end if
                 write(18,'(a)',rec=k) trim(cl1)//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &              ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ',p'//trim(adjustl(ind3))//','//trim(gbbb(i))//','//
     &              trim(mbbb(i,j3))//','//trim(wbbb(i,j3))//')'
                 write(31,'(a10,a10)',rec=k) trim(mbbb(i,j3)),
     &                                       trim(wbbb(i,j3))
                 if (imcs == 1) then
                  write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gbbb(i))//',u'//trim(adjustl(ind1))//','
     &              //trim(cp1)//trim(adjustl(ind1))//','//'u'//
     &              trim(adjustl(ind2))//','//trim(cp2)
     &              //trim(adjustl(ind2))//','
                 else
                  write(22,'(a)',rec=k) trim(cl2)//trim(gbbb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),'//trim(cp1)//trim(adjustl(ind1))//','//'u'//
     &              trim(adjustl(ind2))//'(:,h'//trim(adjustl(ind2))//
     &             '(h)),'//trim(cp2)//trim(adjustl(ind2))//','
                 end if
                else
                 if (iqprec == 1) then
                  if (j1 == 1 .and. j2 == 2) then
                   cl1='      call vvv_qp(1,'
                   cl2='      call vvvmd_qp(1,'
                  else if (j1 == 1 .and. j2 == 3) then
                   cl1='      call vvv_qp(2,'
                   cl2='      call vvvmd_qp(2,'
                  else if (j1 == 2 .and. j2 == 1) then
                   cl1='      call vvv_qp(3,'
                   cl2='      call vvvmd_qp(3,'
                  else if (j1 == 2 .and. j2 == 3) then
                   cl1='      call vvv_qp(4,'
                   cl2='      call vvvmd_qp(4,'
                  else if (j1 == 3 .and. j2 == 1) then
                   cl1='      call vvv_qp(5,'
                   cl2='      call vvvmd_qp(5,'
                  else if (j1 == 3 .and. j2 == 2) then
                   cl1='      call vvv_qp(6,'
                   cl2='      call vvvmd_qp(6,'
                  end if
                 else
                  if (j1 == 1 .and. j2 == 2) then
                   cl1='      call vvvkk(1,'
                   cl2='      call vvvmd(1,'
                  else if (j1 == 1 .and. j2 == 3) then
                   cl1='      call vvvkk(2,'
                   cl2='      call vvvmd(2,'
                  else if (j1 == 2 .and. j2 == 1) then
                   cl1='      call vvvkk(3,'
                   cl2='      call vvvmd(3,'
                  else if (j1 == 2 .and. j2 == 3) then
                   cl1='      call vvvkk(4,'
                   cl2='      call vvvmd(4,'
                  else if (j1 == 3 .and. j2 == 1) then
                   cl1='      call vvvkk(5,'
                   cl2='      call vvvmd(5,'
                  else if (j1 == 3 .and. j2 == 2) then
                   cl1='      call vvvkk(6,'
                   cl2='      call vvvmd(6,'
                  end if
                 end if
                 write(18,'(a)',rec=k) trim(cl1)//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &              ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ',p'//trim(adjustl(ind3))//','//trim(gbbb(i))//','//
     &              trim(mbbb(i,j3))//','//trim(wbbb(i,j3))//')'

                 write(31,'(a10,a10)',rec=k) trim(mbbb(i,j3)),
     &                                       trim(wbbb(i,j3))

                 if (imcs == 1) then
                  write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gbbb(i))//',u'//trim(adjustl(ind1))//','
     &              //trim(cp1)//trim(adjustl(ind1))//','//'u'//
     &              trim(adjustl(ind2))//','//trim(cp2)
     &              //trim(adjustl(ind2))//','
                 else
                  write(22,'(a)',rec=k) trim(cl2)
     &              //trim(gbbb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),'//trim(cp1)//trim(adjustl(ind1))//','//'u'//
     &              trim(adjustl(ind2))//'(:,h'//trim(adjustl(ind2))//
     &             '(h)),'//trim(cp2)//trim(adjustl(ind2))//','
                 end if
                 write(ind,'(i16)') k
                 write(ind1,'(i16)') pln
                 cl='u'//trim(adjustl(ind))//'(0:3,'//
     &                                        trim(adjustl(ind1))//')'
                 write(21,'(a)',rec=k) cl
                end if
               else if (f1j == 25 .and. f2j == 25 .and. plj == 25) 
     &                                                          then

c                write(18,9,rec=k) gbbb(i),i1,i2,k
 9              format('      call hhhkk(',a4,',',i6,',',i6,',',i8,
     &                                                             ')')
                write(ind1,'(i16)') i1
                write(ind2,'(i16)') i2
                write(ind3,'(i16)') k
                write(cn1,'(i16)') f1n
                write(cn2,'(i16)') f2n
                write(cn3,'(i16)') pln
c                write(inh1,'(i16)') pl(i1)%k(0)
c                write(inh2,'(i16)') pl(i2)%k(0)

c                if (f1o < 0) then
c                 cp1='p'
c                else
c                 cp1='-p'
c                end if
c                if (f2o < 0) then
c                 cp2='p'
c                else
c                 cp2='-p'
c                end if

                if (i1 > nexpl) then
                 cp1='-p'
                else
                 if (f1o < 0) then
                  cp1='p'
                 else
                  cp1='-p'
                 end if
                end if
                if (i2 > nexpl) then
                 cp2='-p'
                else
                 if (f2o < 0) then
                  cp2='p'
                 else
                  cp2='-p'
                 end if
                end if

                cl='u'//trim(adjustl(ind3))//'(1,'//
     &                                        trim(adjustl(cn3))//')'
                write(21,'(a)',rec=k) cl

                if (iqprec == 1) then
                 write(18,'(a)',rec=k) '      call sss_qp('//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &             ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ',p'//trim(adjustl(ind3))//','//trim(gbbb(i))//','//
     &              trim(mbbb(i,j3))//','//trim(wbbb(i,j3))//')'
                else
                 write(18,'(a)',rec=k) '      call ssskk('//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &             ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ',p'//trim(adjustl(ind3))//','//trim(gbbb(i))//','//
     &              trim(mbbb(i,j3))//','//trim(wbbb(i,j3))//')'
                end if
                write(31,'(a10,a10)',rec=k) trim(mbbb(i,j3)),
     &                                       trim(wbbb(i,j3))
                if (imcs == 1) then
                 if (iqprec == 1) then
                  write(22,'(a)',rec=k) '      call sssmd_qp('//
     &              trim(gbbb(i))//',u'//trim(adjustl(ind1))//',u'//
     &              trim(adjustl(ind2))//','
                 else
                  write(22,'(a)',rec=k) '      call sssmd('//
     &              trim(gbbb(i))//',u'//trim(adjustl(ind1))//',u'//
     &              trim(adjustl(ind2))//','
                 end if
                else
                 if (iqprec == 1) then
                  write(22,'(a)',rec=k) '      call sssmd_qp('//
     &              trim(gbbb(i))//',u'//trim(adjustl(ind1))
     &              //'(:,h'//trim(adjustl(ind1))//'(h)),u'//
     &              trim(adjustl(ind2))//'(:,h'//trim(adjustl(ind2))//
     &             '(h)),'
                 else
                  write(22,'(a)',rec=k) '      call sssmd('//
     &              trim(gbbb(i))//',u'//trim(adjustl(ind1))
     &              //'(:,h'//trim(adjustl(ind1))//'(h)),u'//
     &              trim(adjustl(ind2))//'(:,h'//trim(adjustl(ind2))//
     &             '(h)),'
                 end if
                end if
               else
                if (abs(f1f) == 1) then               ! Scalar is on the
                 if (iqprec == 1) then
                  cl1='      call svv_qp('
                  cl2='      call svvmd_qp('       ! 1st,
                 else
                  cl1='      call svvkk('
                  cl2='      call svvmd('          ! 1st,
                 end if
                 cl='(0:3,'
                else if (abs(f2f) == 1) then      
                 if (iqprec == 1) then
                  cl1='      call vsv_qp('
                  cl2='      call vsvmd_qp('       ! 2nd, or
                 else
                  cl1='      call vsvkk('
                  cl2='      call vsvmd('          ! 2nd, or
                end if
                 cl='(0:3,'
                else
                 if (iqprec == 1) then
                  cl1='      call vvs_qp('
                  cl2='      call vvsmd_qp('       ! 3rd place
                 else
                  cl1='      call vvskk('
                  cl2='      call vvsmd('          ! 3rd place
                 end if
                 cl='(1,'
                end if

                write(ind1,'(i16)') i1
                write(ind2,'(i16)') i2
                write(ind3,'(i16)') k
                write(cn1,'(i16)') f1n
                write(cn2,'(i16)') f2n
                write(cn3,'(i16)') pln
c                write(inh1,'(i16)') pl(i1)%k(0)
c                write(inh2,'(i16)') pl(i2)%k(0)

c                if (f1o < 0) then
c                 cp1='p'
c                else
c                 cp1='-p'
c                end if
c                if (f2o < 0) then
c                 cp2='p'
c                else
c                 cp2='-p'
c                end if

                if (i1 > nexpl) then
                 cp1='-p'
                else
                 if (f1o < 0) then
                  cp1='p'
                 else
                  cp1='-p'
                 end if
                end if
                if (i2 > nexpl) then
                 cp2='-p'
                else
                 if (f2o < 0) then
                  cp2='p'
                 else
                  cp2='-p'
                 end if
                end if

                write(18,'(a)',rec=k) trim(cl1)//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &             ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ',p'//trim(adjustl(ind3))//','//trim(gbbb(i))//','//
     &              trim(mbbb(i,j3))//','//trim(wbbb(i,j3))//')'
                write(31,'(a10,a10)',rec=k) trim(mbbb(i,j3)),
     &                                       trim(wbbb(i,j3))
                if (imcs == 1) then
                 write(22,'(a)',rec=k) trim(cl2)//trim(gbbb(i))//',u'//
     &              trim(adjustl(ind1))//',u'//trim(adjustl(ind2))//','
                else
                 write(22,'(a)',rec=k) trim(cl2)//trim(gbbb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),'
                end if
                cl='u'//trim(adjustl(ind3))//trim(cl)//
     &                                        trim(adjustl(cn3))//')'
                write(21,'(a)',rec=k) trim(cl)

               end if
               write(20,1620,rec=k) plix
               pl(k)%k(2)=i1
               pl(k)%k(3)=i2
               pl(k)%k(4)=0
               call colinf(i1,i2,k)
c               call convert
c               end if
               exit
              end if
             end do
             exit
            end if
           end do
           exit
          end if
         end do
        end do
       else
        write(*,*)'genvvv.f: Wrong MC index of particle 2: ',f2j
       end if
      else
       write(*,*)'genvvv.f: Wrong MC index of particle 1: ',f1j
      end if

      deallocate(f1ix,f2ix,plix,stat=blad2)

      else ! i2 = 0 <=> i1 is an external particle

c      pl(i1)%k(0)=i1
      pl(i1)%k(1:)=0

      end if ! i2 > 0 

      else ! i3 /= 0

      read(16,1612,rec=i3) f3j,f3i,f3o,f3g,f3f,f3ct,f3n,f3e

      if (isr == 1) then       ! added on 14 Oct. 2015
       if (i1 <= next .and. f1j == 22) then
        if (i2 > nisp .and. i3 > nisp) then
         close(16)
         close(22)
         return
        end if
       else if (i2 <= next .and. f2j == 22) then
        if (i1 > nisp .and. i3 > nisp) then
         close(16)
         close(22)
         return
        end if
       else if (i3 <= next .and. f3j == 22) then
        if (i1 > nisp .and. i1 > nisp) then
         close(16)
         close(22)
         return
        end if
       end if
      end if


      allocate(f3ix(f3e),stat=blad2)

      read(20,1620,rec=i3) f3ix
c      read(20,1620,rec=pl(i3)%k(0)) f3ix


c      f3=pl(i3)

      if (f1j > 20 .and. f2j > 20 .and. f3j > 20) then


      oif1=f1o*f1i*f1j
      oif2=f2o*f2i*f2j
      oif3=f3o*f3i*f3j

      do i=1,nbbbba
       do j1=1,4
        if (oif1 == bbbb(i,j1)) then
         do j2=1,4
          if (j1 /= j2 .and. oif2 == bbbb(i,j2)) then
           do j3=1,4
            if (j1 /= j3 .and. j2 /= j3 .and. oif3 == bbbb(i,j3)) then
             do j4=1,4
              if (j1 /= j4 .and. j2 /= j4 .and. j3 /= j4 .and.
     &            j1 /= j2 .and. j1 /= j3 .and. j2 /= j3) then
               if (f1ct == 1) then
                k=k+1
                if (max(pl(i1)%mx,pl(i2)%mx,pl(i3)%mx) == 2) then
                 pl(k)%mx=2
                else
                 pl(k)%mx=0
                end if
                pl(k)%k(1)=i
                plct=cbbbb(i,j4)
                write(13,'(a)',rec=k) '1'
                plj=abs(bbbb(i,j4))
                plo=-1         ! an internal boson is always incoming 
                                   ! to the vertex
                pli=bbbb(i,j4)/(plo*plj)

c                if (plj == 24 .or. plj == 211) then
                call conveyfg(plj,plg,plf)
                if (plf < 0) then
                 plo=-plo   ! therefore a charged boson should be 
                            ! considered as 
                            ! the outgoing particle from the other vertex
                end if
                pln=f1n*f2n*f3n
                ple=f1e+f2e+f3e
                allocate (plix(ple),stat=blad2)
                plix(:f1e)=f1ix
                plix(f1e+1:f1e+f2e)=f2ix
                plix(f1e+f2e+1:)=f3ix
c                call chckord
                write(20,1620,rec=k) plix
                pl(k)%k(2)=i1
                pl(k)%k(3)=i2
                pl(k)%k(4)=i3
                call convert
                nhiggs=0
                if (f1j == 25) nhiggs=nhiggs+1
                if (f2j == 25) nhiggs=nhiggs+1
                if (f3j == 25) nhiggs=nhiggs+1
                if (plj == 25) nhiggs=nhiggs+1
                npion=0
                if (f1j == 211) npion=npion+1
                if (f2j == 211) npion=npion+1
                if (f3j == 211) npion=npion+1
                if (plj == 211) npion=npion+1
                npi0=0
                if (f1j == 111) npi0=npi0+1
                if (f2j == 111) npi0=npi0+1
                if (f3j == 111) npi0=npi0+1
                if (plj == 111) npi0=npi0+1
                if (nhiggs == 0 .and. npion == 0 .and. npi0 == 0) then
                 write(ind1,'(i16)') i1
                 write(ind2,'(i16)') i2
                 write(ind3,'(i16)') i3
                 write(ind,'(i16)') k
                 write(ci1,'(i16)') j1
                 write(ci2,'(i16)') j2
                 write(ci3,'(i16)') j3
                 write(ci4,'(i16)') j4
                 write(cn1,'(i16)') f1n
                 write(cn2,'(i16)') f2n
                 write(cn3,'(i16)') f3n
                 write(cn,'(i16)') pln
                 if (i1 > nexpl) then
                  cp1='-p'
                 else
                  if (f1o < 0) then
                   cp1='p'
                  else
                   cp1='-p'
                  end if
                 end if
                 if (i2 > nexpl) then
                  cp2='-p'
                 else
                  if (f2o < 0) then
                   cp2='p'
                  else
                   cp2='-p'
                  end if
                 end if
                 if (i3 > nexpl) then
                  cp3='-p'
                 else
                  if (f3o < 0) then
                   cp3='p'
                  else
                   cp3='-p'
                  end if
                 end if

                 if (iqprec == 1) then
                  write(18,'(a)',rec=k) '      call vvvv_qp('//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &              ','//trim(cp1)//trim(adjustl(ind1))//
     &              ','//trim(adjustl(ci1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &              ','//trim(adjustl(ci2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ','//trim(cp3)//trim(adjustl(ind3))//
     &              ','//trim(adjustl(ci3))//
     &             ',u'//trim(adjustl(ind))//','//trim(adjustl(cn))//
     &             ',p'//trim(adjustl(ind))//','//trim(adjustl(ci4))
     &             //','//trim(gbbbb(i))//','//
     &              trim(mbbbb(i,j4))//','//trim(wbbbb(i,j4))//')'
                 else
                  write(18,'(a)',rec=k) '      call vvvvkk('//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &              ','//trim(cp1)//trim(adjustl(ind1))//
     &              ','//trim(adjustl(ci1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &              ','//trim(adjustl(ci2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ','//trim(cp3)//trim(adjustl(ind3))//
     &              ','//trim(adjustl(ci3))//
     &             ',u'//trim(adjustl(ind))//','//trim(adjustl(cn))//
     &             ',p'//trim(adjustl(ind))//','//trim(adjustl(ci4))
     &             //','//trim(gbbbb(i))//','//
     &              trim(mbbbb(i,j4))//','//trim(wbbbb(i,j4))//')'
                 end if

                 write(31,'(a10,a10)',rec=k) trim(mbbbb(i,j4)),
     &                                       trim(wbbbb(i,j4))
                 if (imcs == 1) then
                  if (iqprec == 1) then
                   write(22,'(a)',rec=k) '      call vvvvmd_qp('
     &              //trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))//','
     &              //trim(adjustl(ci1))
     &              //',u'//trim(adjustl(ind2))//','
     &              //trim(adjustl(ci2))
     &              //',u'//trim(adjustl(ind3))//','
     &              //trim(adjustl(ci3))//','
                  else
                   write(22,'(a)',rec=k) '      call vvvvmd('
     &              //trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))//','
     &              //trim(adjustl(ci1))
     &              //',u'//trim(adjustl(ind2))//','
     &              //trim(adjustl(ci2))
     &              //',u'//trim(adjustl(ind3))//','
     &              //trim(adjustl(ci3))//','
                  end if
                 else
                  if (iqprec == 1) then
                   write(22,'(a)',rec=k) '      call vvvvmd_qp('
     &              //trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))
     &              //'(:,h'//trim(adjustl(ind1))//'(h)),'
     &              //trim(adjustl(ci1))
     &              //',u'//trim(adjustl(ind2))
     &              //'(:,h'//trim(adjustl(ind2))//'(h)),'
     &              //trim(adjustl(ci2))
     &              //',u'//trim(adjustl(ind3))
     &              //'(:,h'//trim(adjustl(ind3))//'(h)),'
     &              //trim(adjustl(ci3))//','
                  else
                   write(22,'(a)',rec=k) '      call vvvvmd('
     &              //trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))
     &              //'(:,h'//trim(adjustl(ind1))//'(h)),'
     &              //trim(adjustl(ci1))
     &              //',u'//trim(adjustl(ind2))
     &              //'(:,h'//trim(adjustl(ind2))//'(h)),'
     &              //trim(adjustl(ci2))
     &              //',u'//trim(adjustl(ind3))
     &              //'(:,h'//trim(adjustl(ind3))//'(h)),'
     &              //trim(adjustl(ci3))//','
                  end if
                 end if

                 write(ind,'(i16)') k
                 write(ind1,'(i16)') pln
                 cl='u'//trim(adjustl(ind))//'(0:3,'//
     &                                        trim(adjustl(ind1))//')'
                 write(21,'(a)',rec=k) cl
ckk>
c                else if (f1j == 25 .and. f2j == 25 .and. f3j == 25
c     &                                       .and. plj == 25) then
                else if (nhiggs == 4) then
ckk<
                 write(ind1,'(i16)') i1
                 write(ind2,'(i16)') i2
                 write(ind3,'(i16)') i3
                 write(ind,'(i16)') k
                 write(cn1,'(i16)') f1n
                 write(cn2,'(i16)') f2n
                 write(cn3,'(i16)') f3n
                 write(cn,'(i16)') pln

                 if (i1 > nexpl) then
                  cp1='-p'
                 else
                  if (f1o < 0) then
                   cp1='p'
                  else
                   cp1='-p'
                  end if
                 end if
                 if (i2 > nexpl) then
                  cp2='-p'
                 else
                  if (f2o < 0) then
                   cp2='p'
                  else
                   cp2='-p'
                  end if
                 end if
                 if (i3 > nexpl) then
                  cp3='-p'
                 else
                  if (f3o < 0) then
                   cp3='p'
                  else
                   cp3='-p'
                  end if
                 end if

                 if (iqprec == 1) then
                  write(18,'(a)',rec=k) '      call ssss_qp('//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &             ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ',p'//trim(adjustl(ind3))//
     &             ',u'//trim(adjustl(ind))//','//trim(adjustl(cn))//
     &             ',p'//trim(adjustl(ind))//','//trim(gbbbb(i))//','//
     &              trim(mbbbb(i,j4))//','//trim(wbbbb(i,j4))//')'
                 else
                  write(18,'(a)',rec=k) '      call sssskk('//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &             ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ',p'//trim(adjustl(ind3))//
     &             ',u'//trim(adjustl(ind))//','//trim(adjustl(cn))//
     &             ',p'//trim(adjustl(ind))//','//trim(gbbbb(i))//','//
     &              trim(mbbbb(i,j4))//','//trim(wbbbb(i,j4))//')' ! 16 June,
c     &             ',p'//trim(adjustl(ind))//trim(gbbbb(i))//','//
c     &              trim(mbbb(i,j4))//','//trim(wbbb(i,j4))//')'  ! 2009
                 end if
ckk>
                 write(31,'(a10,a10)',rec=k) trim(mbbbb(i,j4)), ! 16 June, 2009
     &                                       trim(wbbbb(i,j4))
c                 write(31,'(a10,a10)',rec=k) trim(mbbb(i,j4)),
c     &                                       trim(wbbb(i,j4))
ckk<

                 if (imcs == 1) then
                  if (iqprec == 1) then
                   write(22,'(a)',rec=k) '      call ssssmd_qp('//
     &              trim(gbbbb(i))//',u'//
     &              trim(adjustl(ind1))//',u'//trim(adjustl(ind2))
     &              //',u'//trim(adjustl(ind3))//','
                  else
                   write(22,'(a)',rec=k) '      call ssssmd('//
     &              trim(gbbbb(i))//',u'//
     &              trim(adjustl(ind1))//',u'//trim(adjustl(ind2))
     &              //',u'//trim(adjustl(ind3))//','
                  end if
                 else
                  if (iqprec == 1) then
                   write(22,'(a)',rec=k) '      call ssssmd_qp('//
     &              trim(gbbbb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),u'//trim(adjustl(ind3))
     &              //'(:,h'//trim(adjustl(ind3))//'(h)),'
                  else
                   write(22,'(a)',rec=k) '      call ssssmd('//
     &              trim(gbbbb(i))//',u'//
     &              trim(adjustl(ind1))//'(:,h'//trim(adjustl(ind1))//
     &             '(h)),u'//trim(adjustl(ind2))//'(:,h'//
     &              trim(adjustl(ind2))//'(h)),u'//trim(adjustl(ind3))
     &              //'(:,h'//trim(adjustl(ind3))//'(h)),'
                  end if
                 end if
                 write(ind,'(i16)') k
                 write(ind1,'(i16)') pln

                 cl='u'//trim(adjustl(ind))//'(1,'//
     &                                        trim(adjustl(ind1))//')'
                 write(21,'(a)',rec=k) cl
                else if ((nhiggs == 2 .and. npion == 0 .and. npi0 == 0)
     &              .or. (nhiggs == 0 .and. npion == 2 .and. npi0 == 0)) 
     &                                                              then
                 write(ind,'(i16)') k
                 write(ind1,'(i16)') pln
                 if (abs(f1f) == 3 .and. abs(f2f) == 3) then
c 7 May, 2009:
c                  cl='u'//trim(adjustl(ind))//'(0:3,'//
c     &                                        trim(adjustl(ind1))//')'
                  cl='u'//trim(adjustl(ind))//'(1,'//
     &                                        trim(adjustl(ind1))//')'
                  if (iqprec == 1) then
                   cl1='      call vvssmd_qp(i'//trim(gbbbb(i))//','
                   cl2='      call vvss_qp(i'//trim(gbbbb(i))//','
                  else
                   cl1='      call vvssmd(i'//trim(gbbbb(i))//','
                   cl2='      call vvsskk(i'//trim(gbbbb(i))//','
                  end if
                 else if (abs(f1f) == 3 .and. abs(f3f) == 3) then
                  cl='u'//trim(adjustl(ind))//'(1,'//
     &                                        trim(adjustl(ind1))//')'
                  if (iqprec == 1) then
                   cl1='      call vsvsmd_qp(i'//trim(gbbbb(i))//','
                   cl2='      call vsvs_qp(i'//trim(gbbbb(i))//','
                  else
                   cl1='      call vsvsmd(i'//trim(gbbbb(i))//','
                   cl2='      call vsvskk(i'//trim(gbbbb(i))//','
                  end if
                 else if (abs(f1f) == 3 .and. abs(plf) == 3) then
                  cl='u'//trim(adjustl(ind))//'(0:3,'//
     &                                         trim(adjustl(ind1))//')'
                  if (iqprec == 1) then
                   cl1='      call vssvmd_qp(i'//trim(gbbbb(i))//','
                   cl2='      call vssv_qp(i'//trim(gbbbb(i))//',i'
     &                                     //trim(wbbbb(i,j4))//','
                  else
                   cl1='      call vssvmd(i'//trim(gbbbb(i))//','
                   cl2='      call vssvkk(i'//trim(gbbbb(i))//',i'
     &                                     //trim(wbbbb(i,j4))//','
                  end if
                 else if (abs(f2f) == 3 .and. abs(f3f) == 3) then
                  cl='u'//trim(adjustl(ind))//'(1,'//
     &                                        trim(adjustl(ind1))//')'
                  if (iqprec == 1) then
                   cl1='      call svvsmd_qp(i'//trim(gbbbb(i))//','
                   cl2='      call svvs_qp(i'//trim(gbbbb(i))//','
                  else
                   cl1='      call svvsmd(i'//trim(gbbbb(i))//','
                   cl2='      call svvskk(i'//trim(gbbbb(i))//','
                  end if
                 else if (abs(f3f) == 3 .and. abs(plf) == 3) then
                  cl='u'//trim(adjustl(ind))//'(0:3,'//
     &                                        trim(adjustl(ind1))//')'
                  if (iqprec == 1) then
                   cl1='      call ssvvmd_qp(i'//trim(gbbbb(i))//','
                   cl2='      call ssvv_qp(i'//trim(gbbbb(i))//',i'
     &                                     //trim(wbbbb(i,j4))//','
                  else
                   cl1='      call ssvvmd(i'//trim(gbbbb(i))//','
                   cl2='      call ssvvkk(i'//trim(gbbbb(i))//',i'
     &                                     //trim(wbbbb(i,j4))//','
                  end if
                 else if (abs(f2f) == 3 .and. abs(plf) == 3) then
                  cl='u'//trim(adjustl(ind))//'(0:3,'//
     &                                        trim(adjustl(ind1))//')'
                  if (iqprec == 1) then
                   cl1='      call svsvmd_qp(i'//trim(gbbbb(i))//','
                   cl2='      call svsv_qp(i'//trim(gbbbb(i))//',i'
     &                                     //trim(wbbbb(i,j4))//','
                  else
                   cl1='      call svsvmd(i'//trim(gbbbb(i))//','
                   cl2='      call svsvkk(i'//trim(gbbbb(i))//',i'
     &                                     //trim(wbbbb(i,j4))//','
                  end if
                 end if
                 write(21,'(a)',rec=k) cl
                 write(ind1,'(i16)') i1
                 write(ind2,'(i16)') i2
                 write(ind3,'(i16)') i3
                 write(ind,'(i16)') k
                 write(cn1,'(i16)') f1n
                 write(cn2,'(i16)') f2n
                 write(cn3,'(i16)') f3n
                 write(cn,'(i16)') pln

                 if (i1 > nexpl) then
                  cp1='-p'
                 else
                  if (f1o < 0) then
                   cp1='p'
                  else
                   cp1='-p'
                  end if
                 end if
                 if (i2 > nexpl) then
                  cp2='-p'
                 else
                  if (f2o < 0) then
                   cp2='p'
                  else
                   cp2='-p'
                  end if
                 end if
                 if (i3 > nexpl) then
                  cp3='-p'
                 else
                  if (f3o < 0) then
                   cp3='p'
                  else
                   cp3='-p'
                  end if
                 end if

                 write(18,'(a)',rec=k) trim(cl2)//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &             ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ','//trim(cp3)//trim(adjustl(ind3))//
     &             ',u'//trim(adjustl(ind))//','//trim(adjustl(cn))//
     &             ',p'//trim(adjustl(ind))//','//trim(gbbbb(i))//','//
     &              trim(mbbbb(i,j4))//','//trim(wbbbb(i,j4))//')'
                 write(31,'(a10,a10)',rec=k) trim(mbbbb(i,j4)),
     &                                       trim(wbbbb(i,j4))
                 if (imcs == 1) then
                  write(22,'(a)',rec=k) trim(cl1)//trim(gbbbb(i))
     &            //',u'//trim(adjustl(ind1))//
     &             ','//trim(cp1)//trim(adjustl(ind1))//
     &              ',u'//trim(adjustl(ind2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &              ',u'//trim(adjustl(ind3))//','
     &             //trim(cp3)//trim(adjustl(ind3))//','
                 else
                  write(22,'(a)',rec=k) trim(cl1)
     &              //trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))
     &              //'(:,h'//trim(adjustl(ind1))//'(h)),'
     &              //trim(cp1)//trim(adjustl(ind1))//','
     &              //'u'//trim(adjustl(ind2))
     &              //'(:,h'//trim(adjustl(ind2))//'(h)),'
     &              //trim(cp2)//trim(adjustl(ind2))//','
     &              //'u'//trim(adjustl(ind3))
     &              //'(:,h'//trim(adjustl(ind3))//'(h)),'
     &              //trim(cp3)//trim(adjustl(ind3))//','
                 end if
                else if (npion == 2 .and. npi0 == 1) then
                 abbbb(j1)=i1
                 abbbb(j2)=i2
                 abbbb(j3)=i3
                 abbbb(j4)=k
                 ibbbb(j1)=f1n
                 ibbbb(j2)=f2n
                 ibbbb(j3)=f3n
                 ibbbb(j4)=pln
                 obbbb(j1)=f1o
                 obbbb(j2)=f2o
                 obbbb(j3)=f3o
                 obbbb(j4)=0

                 write(ind,'(i16)') k
                 write(ind1,'(i16)') pln

                 if (plf == 3) then
                  cl='u'//trim(adjustl(ind))//'(0:3,'//
     &                                        trim(adjustl(ind1))//')'
                  write(ind1,'(i16)') abbbb(2)
                  write(ind2,'(i16)') abbbb(3)
                  write(ind3,'(i16)') abbbb(4)
                  write(cn1,'(i16)') ibbbb(2)
                  write(cn2,'(i16)') ibbbb(3)
                  write(cn3,'(i16)') ibbbb(4)
                  if (abbbb(2) > nexpl) then
                   cp1='-p'
                  else
                   if (obbbb(2) < 0) then
                    cp1='p'
                   else
                    cp1='-p'
                   end if
                  end if
                  if (abbbb(3) > nexpl) then
                   cp2='-p'
                  else
                   if (obbbb(3) < 0) then
                    cp2='p'
                   else
                    cp2='-p'
                   end if
                  end if
                  if (abbbb(4) > nexpl) then
                   cp3='-p'
                  else
                   if (obbbb(4) < 0) then
                    cp3='p'
                   else
                    cp3='-p'
                   end if
                  end if
                  if (iqprec == 1) then
                   cl1='      call pppvmd_qp(i'//trim(gbbbb(i))//','
                   cl2='      call pppv_qp(i'//trim(gbbbb(i))//',i'
     &                                     //trim(wbbbb(i,j4))//','
                  else
                   cl1='      call pppvmd(i'//trim(gbbbb(i))//','
                   cl2='      call pppvkk(i'//trim(gbbbb(i))//',i'
     &                                     //trim(wbbbb(i,j4))//','
                  end if
                 else
                  cl='u'//trim(adjustl(ind))//'(1,'//
     &                                        trim(adjustl(ind1))//')'
                  write(ind1,'(i16)') abbbb(1)
                  write(cn1,'(i16)') ibbbb(1)
                  if (abbbb(1) > nexpl) then
                   cp1='-p'
                  else
                   if (obbbb(1) < 0) then
                    cp1='p'
                   else
                    cp1='-p'
                   end if
                  end if
                  if (bbbb(i,j4) == -211) then
                   write(ind2,'(i16)') abbbb(2)
                   write(ind3,'(i16)') abbbb(4)
                   write(cn2,'(i16)') ibbbb(2)
                   write(cn3,'(i16)') ibbbb(4)
                   if (abbbb(2) > nexpl) then
                    cp2='-p'
                   else
                    if (obbbb(2) < 0) then
                     cp2='p'
                    else
                     cp2='-p'
                    end if
                   end if
                   if (abbbb(4) > nexpl) then
                    cp3='-p'
                   else
                    if (obbbb(4) < 0) then
                     cp3='p'
                    else
                     cp3='-p'
                    end if
                   end if
                   if (iqprec == 1) then
                    cl1='      call vpppmd_qp(i'//trim(gbbbb(i))//',-1,'
                    cl2='      call vppp_qp(i'//trim(gbbbb(i))//',-1,'
                   else
                    cl1='      call vpppmd(i'//trim(gbbbb(i))//',-1,'
                    cl2='      call vpppkk(i'//trim(gbbbb(i))//',-1,'
                   end if
                  else if (bbbb(i,j4) == 211) then
                   write(ind2,'(i16)') abbbb(2)
                   write(ind3,'(i16)') abbbb(3)
                   write(cn2,'(i16)') ibbbb(2)
                   write(cn3,'(i16)') ibbbb(3)
                   if (abbbb(2) > nexpl) then
                    cp2='-p'
                   else
                    if (obbbb(2) < 0) then
                     cp2='p'
                    else
                     cp2='-p'
                    end if
                   end if
                   if (abbbb(3) > nexpl) then
                    cp3='-p'
                   else
                    if (obbbb(3) < 0) then
                     cp3='p'
                    else
                     cp3='-p'
                    end if
                   end if
                   if (iqprec == 1) then
                    cl1='      call vpppmd_qp(i'//trim(gbbbb(i))//',1,'
                    cl2='      call vppp_qp(i'//trim(gbbbb(i))//',1,'
                   else
                    cl1='      call vpppmd(i'//trim(gbbbb(i))//',1,'
                    cl2='      call vpppkk(i'//trim(gbbbb(i))//',1,'
                   end if
                  else if (bbbb(i,j4) == 111) then
                   write(ind2,'(i16)') abbbb(3)
                   write(ind3,'(i16)') abbbb(4)
                   write(cn2,'(i16)') ibbbb(3)
                   write(cn3,'(i16)') ibbbb(4)
                   if (abbbb(3) > nexpl) then
                    cp2='-p'
                   else
                    if (obbbb(3) < 0) then
                     cp2='p'
                    else
                     cp2='-p'
                    end if
                   end if
                   if (abbbb(4) > nexpl) then
                    cp3='-p'
                   else
                    if (obbbb(4) < 0) then
                     cp3='p'
                    else
                     cp3='-p'
                    end if
                   end if
                   if (iqprec == 1) then
                    cl1='      call vpppmd_qp(i'//trim(gbbbb(i))//',1,'
                    cl2='      call vppp_qp(i'//trim(gbbbb(i))//',1,'
                   else
                    cl1='      call vpppmd(i'//trim(gbbbb(i))//',1,'
                    cl2='      call vpppkk(i'//trim(gbbbb(i))//',1,'
                   end if
                  end if
                 end if

                 write(21,'(a)',rec=k) cl
                 write(ind,'(i16)') k
                 write(cn,'(i16)') pln

                 write(18,'(a)',rec=k) trim(cl2)//
     &             'u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &             ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ','//trim(cp3)//trim(adjustl(ind3))//
     &             ',u'//trim(adjustl(ind))//','//trim(adjustl(cn))//
     &             ',p'//trim(adjustl(ind))//','//trim(gbbbb(i))//','//
     &              trim(mbbbb(i,j4))//','//trim(wbbbb(i,j4))//')'
                 write(31,'(a10,a10)',rec=k) trim(mbbbb(i,j4)),
     &                                       trim(wbbbb(i,j4))
                 if (imcs == 1) then
                  write(22,'(a)',rec=k) trim(cl1)//trim(gbbbb(i))
     &            //',u'//trim(adjustl(ind1))//
     &            ','//trim(cp1)//trim(adjustl(ind1))
     &            //',u'//trim(adjustl(ind2))//
     &            ','//trim(cp2)//trim(adjustl(ind2))
     &            //',u'//trim(adjustl(ind3))//
     &            ','//trim(cp3)//trim(adjustl(ind3))//','
                 else
                  write(22,'(a)',rec=k) trim(cl1)
     &              //trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))//','
     &              //trim(cp1)//trim(adjustl(ind1))
     &              //'(:,h'//trim(adjustl(ind1))//'(h)),'
     &              //'u'//trim(adjustl(ind2))//','
     &              //trim(cp2)//trim(adjustl(ind2))
     &              //'(:,h'//trim(adjustl(ind2))//'(h)),'
     &              //'u'//trim(adjustl(ind3))//','
     &              //trim(cp3)//trim(adjustl(ind3))
     &              //'(:,h'//trim(adjustl(ind3))//'(h)),'
                 end if
                end if
               else if (f1ct == 8) then
                l=0
                m=i
                do
                 l=l+1
                 m=m+1
                 if (l > 3) exit
                 k=k+1
                 if (max(pl(i1)%mx,pl(i2)%mx,pl(i3)%mx) == 2) then
                  pl(k)%mx=2
                 else
                  pl(k)%mx=0
                 end if
                 pl(k)%k(1)=i ! 8 Jan. 2016
                 plct=f1ct
                 write(ind1,'(i16)') i1
                 write(ind2,'(i16)') i2
                 write(ind3,'(i16)') i3
                 write(ind,'(i16)') k
                 if (l == 1) then
                  write(13,100,rec=k) l,trim(adjustl(ind1)),
     &                                  trim(adjustl(ind2)),
     &                                  trim(adjustl(ind3)),
     &                                  trim(adjustl(ind))
 100              format('c(',i1,',i',a,',i',a,',i',a,',i',a,')')
                 else if (l == 2) then
                  write(13,100,rec=k) l,trim(adjustl(ind1)),
     &                                  trim(adjustl(ind3)),
     &                                  trim(adjustl(ind)),
     &                                  trim(adjustl(ind2))
                 else if (l == 3) then
                  write(13,100,rec=k) l,trim(adjustl(ind1)),
     &                                  trim(adjustl(ind)),
     &                                  trim(adjustl(ind2)),
     &                                  trim(adjustl(ind3))
                 else
                  write(*,*)'genpart.f: No such colour structure', l
                 end if

                 plj=abs(bbbb(i,j4))
                 plo=-1      ! an internal boson is always incoming 
                             ! to the vertex
                 pli=bbbb(i,j4)/(plo*plj)

c                 if (plj == 24 .or. plj == 211) then
                 call conveyfg(plj,plg,plf)
                 if (plf < 0) then
                  plo=-plo   ! therefore a charged boson should be 
                              ! considered as the outgoing particle 
                              ! from the other vertex
                 end if
                 pln=f1n*f2n*f3n
                 ple=f1e+f2e+f3e
                 allocate (plix(ple),stat=blad2)
                 plix(:f1e)=f1ix
                 plix(f1e+1:f1e+f2e)=f2ix
                 plix(f1e+f2e+1:)=f3ix
c                call chckord
                 write(20,1620,rec=k) plix
                 pl(k)%k(2)=i1
                 pl(k)%k(3)=i2
                 pl(k)%k(4)=i3
                 call convert
                 write(ci1,'(i16)') l
                 write(ind1,'(i16)') i1
                 write(ind2,'(i16)') i2
                 write(ind3,'(i16)') i3
                 write(ind,'(i16)') k
                 write(cn1,'(i16)') f1n
                 write(cn2,'(i16)') f2n
                 write(cn3,'(i16)') f3n
                 write(cn,'(i16)') pln

                 if (i1 > nexpl) then
                  cp1='-p'
                 else
                  if (f1o < 0) then
                   cp1='p'
                  else
                   cp1='-p'
                  end if
                 end if
                 if (i2 > nexpl) then
                  cp2='-p'
                 else
                  if (f2o < 0) then
                   cp2='p'
                  else
                   cp2='-p'
                  end if
                 end if
                 if (i3 > nexpl) then
                  cp3='-p'
                 else
                  if (f3o < 0) then
                   cp3='p'
                  else
                   cp3='-p'
                  end if
                 end if

                 if (iqprec == 1) then
                  write(18,'(a)',rec=k) '      call gggg_qp('//
     &             trim(adjustl(ci1))//
     &             ',u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &              ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ','//trim(cp3)//trim(adjustl(ind3))//
     &             ',u'//trim(adjustl(ind))//','//trim(adjustl(cn))//
     &             ',p'//trim(adjustl(ind))//','//trim(gbbbb(i))//','//
     &              trim(mbbbb(i,j4))//','//trim(wbbbb(i,j4))//')'
                 else
                  write(18,'(a)',rec=k) '      call ggggkk('//
     &             trim(adjustl(ci1))//
     &             ',u'//trim(adjustl(ind1))//','//trim(adjustl(cn1))//
     &              ','//trim(cp1)//trim(adjustl(ind1))//
     &             ',u'//trim(adjustl(ind2))//','//trim(adjustl(cn2))//
     &             ','//trim(cp2)//trim(adjustl(ind2))//
     &             ',u'//trim(adjustl(ind3))//','//trim(adjustl(cn3))//
     &             ','//trim(cp3)//trim(adjustl(ind3))//
     &             ',u'//trim(adjustl(ind))//','//trim(adjustl(cn))//
     &             ',p'//trim(adjustl(ind))//','//trim(gbbbb(i))//','//
     &              trim(mbbbb(i,j4))//','//trim(wbbbb(i,j4))//')'
                 end if
                 write(31,'(a10,a10)',rec=k) trim(mbbbb(i,j4)),
     &                                       trim(wbbbb(i,j4))

                 if (imcs == 1) then
                  if (iqprec == 1) then
                   write(22,'(a)',rec=k) '      call ggggmd_qp('
     &              //trim(adjustl(ci1))//','//trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))
     &              //',u'//trim(adjustl(ind2))
     &              //',u'//trim(adjustl(ind3))//','
                  else
                   write(22,'(a)',rec=k) '      call ggggmd('
     &              //trim(adjustl(ci1))//','//trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))
     &              //',u'//trim(adjustl(ind2))
     &              //',u'//trim(adjustl(ind3))//','
                  end if
                 else
                  if (iqprec == 1) then
                   write(22,'(a)',rec=k) '      call ggggmd_qp('
     &              //trim(adjustl(ci1))//','//trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))
     &              //'(:,h'//trim(adjustl(ind1))//'(h))'
     &              //',u'//trim(adjustl(ind2))
     &              //'(:,h'//trim(adjustl(ind2))//'(h))'
     &              //',u'//trim(adjustl(ind3))
     &              //'(:,h'//trim(adjustl(ind3))//'(h)),'
                  else
                   write(22,'(a)',rec=k) '      call ggggmd('
     &              //trim(adjustl(ci1))//','//trim(gbbbb(i))
     &              //',u'//trim(adjustl(ind1))
     &              //'(:,h'//trim(adjustl(ind1))//'(h))'
     &              //',u'//trim(adjustl(ind2))
     &              //'(:,h'//trim(adjustl(ind2))//'(h))'
     &              //',u'//trim(adjustl(ind3))
     &              //'(:,h'//trim(adjustl(ind3))//'(h)),'
                  end if
                 end if
                 write(ind,'(i16)') k
                 write(ind1,'(i16)') pln
                 cl='u'//trim(adjustl(ind))//'(0:3,'//
     &                                        trim(adjustl(ind1))//')'
                 write(21,'(a)',rec=k) cl
c                 call convert
                end do
               else 
                write(*,*)'genpart.f: Quartic couplings do not ',
     &                    'involve such a colour state',plct
               end if
               exit
              end if
             end do
             exit
            end if
           end do
           exit
          end if
         end do
         exit
        end if
       end do
      end do

      end if ! only bosons

      deallocate(f1ix,f2ix,f3ix,plix,stat=blad2)

      end if ! i3 == 0

      do l=k0+1,k
       if (i1 > nexpl) then
        d1=size(pl(i1)%ip)+1
        allocate(v1(d1),stat=blad2)
        if (d1-1 > 0) then
         v1(1:d1-1)=pl(i1)%ip
        end if
        v1(d1)=i1
       else
        d1=0
       end if
       if (i2 > nexpl) then
        d2=size(pl(i2)%ip)+1
        allocate(v2(d2),stat=blad2)
        if (d2-1 > 0) then
         v2(1:d2-1)=pl(i2)%ip
        end if
        v2(d2)=i2
       else
        d2=0
       end if
       if (i3 > nexpl) then
        d3=size(pl(i3)%ip)+1
        allocate(v3(d3),stat=blad2)
        if (d3-1 > 0) then
         v3(1:d3-1)=pl(i3)%ip
        end if
        v3(d3)=i3
       else
        d3=0
       end if
       allocate(pl(l)%ip(d1+d2+d3),stat=blad2)
       if (d1 > 0) then
c        pl(l)%ip(1:d1-1)=v1
c        pl(l)%ip(d1)=i1
        pl(l)%ip(1:d1)=v1    ! 26 May, 2010  
       end if
       if (d2 > 0) then
c        pl(l)%ip(d1+1:d1+d2-1)=v2
c        pl(l)%ip(d1+d2)=i2
        pl(l)%ip(d1+1:d1+d2)=v2    ! 26 May, 2010  
       end if
       if (d3 > 0) then
c        pl(l)%ip(d1+d2+1:d1+d2+d3-1)=v3
c        pl(l)%ip(d1+d2+d3)=i3
        pl(l)%ip(d1+d2+1:d1+d2+d3)=v3    ! 26 May, 2010  
       end if
       deallocate(v1,v2,v3,stat=blad2)
      end do

      close(16)
      close(22)

      do i=k0+1,k                                  ! > Added on 19 May, 2015
       pl(i)%dm=0                                  !
       allocate (pl(i)%im(pl(i)%dm),stat=blad2)    !
      end do                                       ! <

      contains

       subroutine convert

       pl(k)%ct=plct

       write(16,1612,rec=k) plj,pli,plo,plg,plf,plct,pln,ple
 1612 format(i8,5(i3),i24,i3)

       end subroutine convert

c       subroutine printinf(check)

c       logical :: check

c       rewind 13
c       read(13,'(a)') pl(k)%cl
c       close(13,status='delete')

c       check=.true.

c       do i=1,k0
c        if (index(pl(i)%cl,pl(l)%cl(1:index(pl(k)%cl,',',.true.))) > 0) 
c     &                                                             then
c         check=.false.
c         exit
c        end if
c        if (.not. check) exit
c       end do

c       end subroutine printinf

       subroutine colinf(p1,p2,p0)
c Store color information

       integer :: p1,p2,p0
       character(80) ind0,ind1,ind2

       if (plct == 1) then
        if (pl(p1)%ct == 1) then
         write(13,'(a)',rec=k) '1'
        else 
         write(ind1,'(i16)') p1
         write(ind2,'(i16)') p2
c         write(13,10,rec=k) p1,p2
c 10      format('d(i(',i6,'),i(',i6,'))')
         write(13,10,rec=k) trim(adjustl(ind1)),trim(adjustl(ind2))
 10      format('d(i',a,',i',a,')')
 11      format('d(j,i',a,')')
 12      format('d(i',a,',j)')
        end if
       else if (plct == 3) then
        if (pl(p1)%ct == 1) then
c         write(13,10,rec=k) p0,p2
         write(ind0,'(i16)') p0
         write(ind2,'(i16)') p2
         if (plo*pli*plj > 0) then
          write(13,10,rec=k) trim(adjustl(ind2)),trim(adjustl(ind0))
         else
          write(13,10,rec=k) trim(adjustl(ind0)),trim(adjustl(ind2))
         end if
        else if (pl(p1)%ct == 3) then
         if (pl(p2)%ct == 1) then
c          write(13,10,rec=k) p0,p1
          write(ind0,'(i16)') p0
          write(ind1,'(i16)') p1
          if (plo*pli*plj > 0) then
           write(13,10,rec=k) trim(adjustl(ind1)),trim(adjustl(ind0))
          else
           write(13,10,rec=k) trim(adjustl(ind0)),trim(adjustl(ind1))
          end if
         else if (pl(p2)%ct == 8) then
c          write(13,20,rec=k) p2,p1,p0
          write(ind1,'(i16)') p1
          write(ind2,'(i16)') p2
          write(ind0,'(i16)') p0
          write(13,20,rec=k) trim(adjustl(ind2)),trim(adjustl(ind1)),
     &                       trim(adjustl(ind0))
         end if
        else if (pl(p1)%ct == 8) then
c         write(13,20,rec=k) p1,p0,p2
         write(ind1,'(i16)') p1
         write(ind2,'(i16)') p2
         write(ind0,'(i16)') p0
         write(13,20,rec=k) trim(adjustl(ind1)),trim(adjustl(ind0)),
     &                      trim(adjustl(ind2))
c 20      format('t(i(',i6,'),i(',i6,'),i(',i6,'))')
 20      format('t(i',a,',i',a,',i',a,')')
        end if
       else if (plct == 8) then
        if (pl(p1)%ct == 3) then
c         write(13,20,rec=k) p0,p1,p2
         write(ind1,'(i16)') p1
         write(ind2,'(i16)') p2
         write(ind0,'(i16)') p0
         write(13,20,rec=k) trim(adjustl(ind0)),trim(adjustl(ind1)),
     &                      trim(adjustl(ind2))
        else if (pl(p1)%ct == 8) then
         if (j1 == 1 .and. j2 == 2) then
c          write(13,30,rec=k) p1,p2,p0
          write(ind1,'(i16)') p1
          write(ind2,'(i16)') p2
          write(ind0,'(i16)') p0
          write(13,30,rec=k) trim(adjustl(ind1)),trim(adjustl(ind2)),
     &                       trim(adjustl(ind0))
         else if (j1 == 1 .and. j2 == 3) then
c          write(13,30,rec=k) p1,p0,p2
          write(ind1,'(i16)') p1
          write(ind2,'(i16)') p2
          write(ind0,'(i16)') p0
          write(13,30,rec=k) trim(adjustl(ind1)),trim(adjustl(ind0)),
     &                       trim(adjustl(ind2))
         else if (j1 == 2 .and. j2 == 1) then
c          write(13,30,rec=k) p2,p1,p0
          write(ind1,'(i16)') p1
          write(ind2,'(i16)') p2
          write(ind0,'(i16)') p0
          write(13,30,rec=k) trim(adjustl(ind2)),trim(adjustl(ind1)),
     &                       trim(adjustl(ind0))
         else if (j1 == 2 .and. j2 == 3) then
c          write(13,30,rec=k) p0,p1,p2
          write(ind1,'(i16)') p1
          write(ind2,'(i16)') p2
          write(ind0,'(i16)') p0
          write(13,30,rec=k) trim(adjustl(ind0)),trim(adjustl(ind1)),
     &                       trim(adjustl(ind2))
         else if (j1 == 3 .and. j2 == 1) then
c          write(13,30,rec=k) p2,p0,p1
          write(ind1,'(i16)') p1
          write(ind2,'(i16)') p2
          write(ind0,'(i16)') p0
          write(13,30,rec=k) trim(adjustl(ind2)),trim(adjustl(ind0)),
     &                       trim(adjustl(ind1))
         else if (j1 == 3 .and. j2 == 2) then
c          write(13,30,rec=k) p0,p2,p1
          write(ind1,'(i16)') p1
          write(ind2,'(i16)') p2
          write(ind0,'(i16)') p0
          write(13,30,rec=k) trim(adjustl(ind0)),trim(adjustl(ind2)),
     &                       trim(adjustl(ind1))
         end if
c 30      format('icom*f(i(',i6,'),i(',i6,'),i(',i6,'))')
c 30      format('icom*f(i',a,',i',a,',i',a,')')
 30      format('f(i',a,',i',a,',i',a,')')
        else
         write(*,*)'genpart.f: Wrong color state (1)',pl(p2)%ct
         stop
        end if
       else
        write(*,*)'genpart.f: Wrong color state (2)',plct
        stop
       end if

       end subroutine colinf

c       subroutine chckord

c       integer :: iio,jjo,iorx,plox(nexpl)

c       do iio=iplord,1,-1
c        plox=0
c        iorx=0
c        read(20,'(26(i3))',rec=iio) plox
c        do jjo=1,ple
c         if (plox(jjo) == plix(jjo)) then
c          if (jjo == ple .and. plox(ple+1) == 0) then
c           iorx=iio
c          end if
c         else
c          exit
c         end if
c        end do
c        if (iorx > 0) then
c         pl(k)%k(0)=iorx
c         exit
c        end if
c       end do

c       if (iorx == 0) then
c        iplord=iplord+1
c        pl(k)%k(0)=iplord
c        write(20,'(26(i3))',rec=iplord) plix
c       end if

c       end subroutine chckord

      end subroutine genpart
