      subroutine readpart(process)
c----------------------------------------------------------------------c
c Reads particles from process
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 9 February, 2006; modified 12 October, 2006 (particle reading improved)
c                            6 November, 2006 (wrong restoration of old
c                                              conventions removed),
c                            12 May, 2007 (mass and widths assignments
c                                          added)
c                            18 October, 2007 (MC summing over 
c                                              polarizations implemented)
c                            14 April, 2008 (determination of the 
c                            statistical factor for identical particles
c                            in the final state corrected)
c                            4 Feb., 2010 (information about generated 
c                            mappings printed)
c 28 February, 2012: short name for the process created automatically
c 22-23 January 2014:
c Modifications for imcs=1 introduced
c 18 March, 2015
c Format of the integer representing a particle number in the MC 
c numbering scheme changed from i5 to i8
c 14 Oct. 2015:
c a criterion for the ISR and FSR added; a flag 'isr'
c 10 Feb. 2016:
c indices of the external photons stored in array exphot
c 25 June 2018:
c indices of the external particles forming t-channel poles stored in 
c arrays ixtch1,ixtch2
c 22 March 2021: 
c Calls to the FSR routines with neutrinos in a wrong place eliminated 
c 24 March 2021: 
c t-channel kinematics will be generated only if both initial state 
c particles are present in the final state
c----------------------------------------------------------------------c

      use maxvals
      use mytypes
      use extpart
      use vertices
      use declends
      use modpart
      use schkins
      use tchkins
      use gchkins

      implicit none

      character(80) :: process,proc,cline

      character(1), parameter :: space=' ',separ='>'
      integer :: i,j,k,l,n,blad,iprt(mxprt,5),
     &           allmod,inmod,ii,difp,ifrmn,addhls,iphot,itch1,itch2,
     &           igvb,icolon
      character(10) :: cprt(mxprt,4),ind1,ind2,ind3,ind4
      character(1) :: lprt(mxprt)

      integer, allocatable :: idpf(:)
      character(10), allocatable :: expd(:)
      character(9) :: prtcl(4)

      proc=adjustl(process)

      j=0
      l=index(proc,separ)

      if (l == 0) then
       write(*,*)'Wrong separator, or process not entered.'
       write(*,*)'Please, try again!'
       stop
      end if


c Read initial state particles

      do 
       n=index(proc,space)
       j=j+1
       expc(j)=proc(1:n-1)
       f(j)%o=-1
       proc=adjustl(proc(n:l))
       if (proc(2:2) == separ) exit
      end do

c No of the initial state particles

      nisp=j

c Read final state 

      k=l+2

      proc=adjustl(process(l+1:))

      do 
       n=index(proc,space)
       if (n > 1) then
        j=j+1
        f(j)%o=1
        expc(j)=proc(1:n-1)
       else
        exit
       end if
       proc=adjustl(proc(n:))
      end do

c No of the external particles

      next=j

      open(unit=19,file='tchcalls.f',status='replace',action='write')
c      open(unit=20,file='gchcalls.f',status='replace',action='write')

c Check for a photon or gluon in the final state:
      igvb=0
      igchnl0=igchnl
      do i=nisp+1,next,1
       if (expc(i) == 'a' .or. expc(i) == 'g') then
        igind=i
        igvb=igvb+1
       end if
      end do
      if (igvb > 0) then
       if (igvb == 1) then
        igchnl=1
       else
        igchnl=0
       end if
      else
       igchnl=0
      end if
      if (igchnl == 1) then
       write(indg,'(i4)') next-nisp-1
       clgisr='      call king'//trim(adjustl(indg))//'isr('
       clgfsr='      call king'//trim(adjustl(indg))//'fsr('
      end if

c      close(20)

      if (itchnl == 1) then
c Check for particles forming t-channel poles:
       itch1=0
       itch2=0
       ixtch1=0
       ixtch2=0
       do j=3,next
        if (expc(j) == expc(1)) then
         itch1=itch1+1
         ixtch1(itch1)=j
        else if (expc(j) == expc(2)) then
         itch2=itch2+1
         ixtch2(itch2)=j
        end if
       end do
       if (itch1*itch2 == 0) itchnl=0
c       nkint=3*itch1*itch2  ! Only for ff~ pairs
       nkint=0
c       if (nkint > 0) then ! Commented on 21 Jan 2019
       if (next > 4) then
c        if (itch1 > 0 .or. itch2 > 0) then
        if (itch1 > 0 .and. itch2 > 0) then
         write(ind1,'(i4)') next-4
         write(19,'(a)') '      integer :: ix('//trim(adjustl(ind1))
     &                                                         //')'
         write(19,'(a)') ' '
        end if
        if (itch1 > 0 .and. itch2 > 0) then
         do i=1,itch1
          write(ind1,'(i4)') ixtch1(i)
          do j=1,itch2
           nkint=nkint+1
           write(ind3,'(i4)') nkint
           if (nkint == 1) then
            write(19,'(a)') '      if (ikin == nkins+'
     &             //trim(adjustl(ind3))//') then'
           else
            write(19,'(a)') '      else if (ikin == nkins+'
     &            //trim(adjustl(ind3))//') then'
           end if
           write(ind2,'(i4)') ixtch2(j)
           l=0
           do k=3,next
            if (k /= ixtch1(i) .and. k /= ixtch2(j)) then
             write(ind3,'(i4)') k
             l=l+1
             write(ind4,'(i4)') l
             write(19,'(a)') '       ix('//trim(adjustl(ind4))//')='
     &                       //trim(adjustl(ind3))
            end if
           end do
           if (l == 0) ind4='0'
           write(19,'(a)') '       call kintt3y2(1,'
     &          //trim(adjustl(ind1))//',1,2,'//trim(adjustl(ind2))//
     &         ',1,'//trim(adjustl(ind4))//')'
           if (l > 1) then
            nkint=nkint+1
            write(ind3,'(i4)') nkint
            write(19,'(a)') '      else if (ikin == nkins+'
     &            //trim(adjustl(ind3))//') then'
            l=0
            do k=3,next
             if (k /= ixtch1(i) .and. k /= ixtch2(j)) then
              write(ind3,'(i4)') k
              l=l+1
              write(ind4,'(i4)') l
              write(19,'(a)') '       ix('//trim(adjustl(ind4))//')='
     &                        //trim(adjustl(ind3))
             end if
            end do
            if (l == 0) ind4='0'
            write(19,'(a)') '       call kintt3y2(1,'
     &          //trim(adjustl(ind1))//',1,2,'//trim(adjustl(ind2))//
     &         ',0,'//trim(adjustl(ind4))//')'
            nkint=nkint+1
            write(ind3,'(i4)') nkint
            write(19,'(a)') '      else if (ikin == nkins+'
     &            //trim(adjustl(ind3))//') then'
            l=0
            do k=3,next
             if (k /= ixtch1(i) .and. k /= ixtch2(j)) then
              write(ind3,'(i4)') k
              l=l+1
              write(ind4,'(i4)') l
              write(19,'(a)') '       ix('//trim(adjustl(ind4))//')='
     &                        //trim(adjustl(ind3))
             end if
            end do
            if (l == 0) ind4='0'
            write(19,'(a)') '       call kintt3y2(1,'
     &          //trim(adjustl(ind1))//',0,2,'//trim(adjustl(ind2))//
     &         ',1,'//trim(adjustl(ind4))//')'
           end if
          end do
         end do
        end if
c        if (itch1 > 0 .or. itch2 > 0) then
        if (itch1 > 0 .and. itch2 > 0) then
         write(19,'(a)') '      end if'
        end if
       else
         write(19,'(a)') 'c This file is empty for next <= 4'
       end if
c       else                ! Commented on 21 Jan 2019
       if (nkint == 0) then ! Added on 21 Jan 2019
        write(19,'(a)') 'c Assignements below should not play '//
     &                  'a role for nkint <= 0'
        write(ind1,'(i4)') next-2
        write(19,'(a)') '      integer :: ix('//trim(adjustl(ind1))
     &                                                         //')'
        l=0
        do k=3,next
         l=l+1
         write(ind1,'(i4)') l
         write(ind2,'(i4)') k
         write(19,'(a)') '       ix('//trim(adjustl(ind1))//')='
     &                      //trim(adjustl(ind2))
        end do
       end if
      else
       write(19,'(a)') 'c Assignements below should not play '//
     &                  'a role for itchnl /= 1'
       write(ind1,'(i4)') next-2
       write(19,'(a)') '      integer :: ix('//trim(adjustl(ind1))
     &                                                         //')'
       l=0
       do k=3,next
        l=l+1
        write(ind1,'(i4)') l
        write(ind2,'(i4)') k
        write(19,'(a)') '       ix('//trim(adjustl(ind1))//')='
     &                      //trim(adjustl(ind2))
       end do
      end if

      close(19)

c      if (itch1 > 0 .or. itch2 > 0) then
c       sumfspm=0.d0
c       do j=3,next
c        sumfspm=sumfspm+m(j)
c       end do
c      end if

c Create short name for the process

      if (trim(prcsnm) == 'auto') then
       prcsnm=''
       do j=1,nisp
        prcsnm=trim(adjustl(prcsnm))//trim(adjustl(expc(j)))
       end do
       prcsnm=trim(adjustl(prcsnm))//'_'
       do j=nisp+1,next
        prcsnm=trim(adjustl(prcsnm))//expc(j)(1:1)
       end do
      end if

c No of identical particles in the final state

      allocate (idpf(next-nisp),stat=blad)
      allocate (expd(next-nisp),stat=blad)

      idpf=1
      difp=1
      expd(difp)=expc(nisp+1)

      do j=nisp+2,next
       do ii=difp,1,-1
        if (expc(j) == expd(ii)) then
         idpf(ii)=idpf(ii)+1
         exit
        else
         if (ii == 1) then
          difp=difp+1
          expd(difp)=expc(j)
         end if
        end if
       end do
      end do

      nidp=1

      do j=1,difp
       do ii=1,idpf(j)
        nidp=nidp*ii
       end do
      end do

      deallocate (idpf,expd,stat=blad)

c      open(66,file='modpart.dat',access='direct',form='formatted',
c     &                                                     recl=80)
      open(unit=66,file='modpart.dat',status='old',action='read')

c Read comments

      do 
       read(66,'(a80)') cline
       if (index(cline,'Begin input') > 0 ) exit
      end do

c Read particles

      ipmod=0

      do 
       ipmod=ipmod+1
       read(66,'(4(a9),4(i5),i8,a1)',err=600) cprt(ipmod,:),
     &                                    iprt(ipmod,:),lprt(ipmod)
      end do

 600  ipmod=ipmod-1

      close(66)

      do i=1,ipmod
       prtf(i)=iprt(i,1)
       prtg(i)=iprt(i,2)
       prtj(i)=iprt(i,5)
      end do

c Check if all the external particles belong to the model

      allmod=1

      do j=1,next
       inmod=0
       do i=1,ipmod
        if (trim(expc(j)) == trim(cprt(i,1))
     &     .or. trim(expc(j)) == trim(cprt(i,2))) then
         inmod=1
         exit
        end if
       end do
       allmod=allmod*inmod
       if (allmod == 0) then
        write(*,*) trim(expc(j)),' is not in the model'
        stop 
       end if
      end do

      nmcs=0
      nhip=1
      iphot=0

      do j=1,next
       f(j)%e=1
       allocate (f(j)%ix(f(j)%e),stat=blad)
       allocate (f(j)%ip(0),stat=blad)
       f(j)%ix(f(j)%e)=j
       do i=1,ipmod
        if (trim(expc(j)) == trim(cprt(i,1))
     &     .or. trim(expc(j)) == trim(cprt(i,2))) then
         expl(j)=lprt(i)
         if (trim(expc(j)) == trim(cprt(i,1))
     &      .and. trim(expc(j)) == trim(cprt(i,2))) then  ! neutral boson
          if (f(j)%o == -1) then
           f(j)%i=-1
          else
           f(j)%i=1
          end if
         else
          if (trim(expc(j)) == trim(cprt(i,1))) then
           f(j)%i=1
          else
           f(j)%i=-1
          end if
         end if
         f(j)%j=iprt(i,5)
         f(j)%f=iprt(i,1)
         f(j)%g=iprt(i,2)
         f(j)%n=iprt(i,4)
         f(j)%ct=iprt(i,3)
         f(j)%m=trim(cprt(i,3))
         f(j)%w=trim(cprt(i,4))
        end if
       end do
       if (f(j)%o == -1) then
        nhip=nhip*f(j)%n
       end if
       if (imcs == 1) then
        if (f(j)%n > 1) then
         nmcs=nmcs+1
         f(j)%mc=nmcs
         f(j)%n=1
        else
         f(j)%mc=0
        end if
       else
        f(j)%mc=0
       end if
       if (f(j)%j == 22) then
        iphot=iphot+1
        exphot(iphot)=j       ! 10 Feb. 2016
       end if
      end do

      nphot=iphot

c Check if there are nucleons present among the external particles

      inucl=0

      do j=1,next
       if (f(j)%j == 9 .or. f(j)%j == 10) then
        inucl=1
        exit
       end if
      end do

c Define the SM vertices, using the notation
c f = 1-neutrino, 2-charged lepton, 3-up quark, 4-down quark;
c b = 0-Higgs, 1-photon, 2-Z, 3-W, 4-gluon.

c Read fermion-fermion-boson vertices of the SM and the corresponding couplings

      if (ickm == 1) then
       open(unit=66,file='vertices.dat.ckm',status='old',action='read')
      else
       open(unit=66,file='vertices.dat',status='old',action='read')
      end if

      nffba=0
      nbbba=0
      nbbbba=0
      nbba=0
      addhls=0

      open(unit=19,file='vrtxinf.dat',status='replace',action='write')

      nhls=0     ! 2 Feb. 2016

      write(19,'(a)') 'The following Standard Model interaction '//
     &                'vertices are included:'

      do 
       read(66,'(a80)',end=700) cline
       icolon=index(cline,':')
       if (cline(1:1) == '#') then
        if (icolon == len_trim(cline) .and. addhls == 0) then
         write(19,'(a)') adjustl(cline(2:))
        end if
       else 
        if (icolon < len_trim(cline) .and. addhls == 0) then
         write(19,'(a)') trim(cline)
        end if
       end if
       if (cline(1:1) == '#') then
        if (addhls == 0 .and. index(cline,'HLS') > 0) then
         write(19,'(a)') 'The following HLS rules are included in '
     &   //'addition to the Standard Model:'
         addhls=1
        end if
       else
        if (addhls == 1 .and. icolon < len_trim(cline)) then
         write(19,'(a)') trim(cline)
        end if
        cline=adjustl(cline)
        if (addhls == 1) then
c         write(19,'(a)') '#  '//trim(cline)
         nhls=nhls+1     ! 2 Feb. 2016
        end if
        j=0
        l=index(cline,':')
        if (l == 0) then
         write(*,*)'Wrong separator in vertices.dat. It should be ":"'
         stop
        end if
        proc=cline(1:l-1)
        do 
         n=index(proc,space)
         if (n > 1) then
          j=j+1
          prtcl(j)=proc(1:n-1)
         else
          exit
         end if
         proc=adjustl(proc(n:))
        end do
        if (j == 4) then
         nbbbba=nbbbba+1
         if (nbbbba > nbbbb) then
          write(*,*)'Too many quartic couplings in vertices.dat.'//
     &              ' Increase nbbbb in maxvals.f.'
         end if
         do k=1,4
          do i=1,ipmod
           if (trim(prtcl(k)) == trim(cprt(i,1))) then
            bbbb(nbbbba,k)=iprt(i,5)
            exit
           else if (trim(prtcl(k)) == trim(cprt(i,2))) then
            bbbb(nbbbba,k)=-iprt(i,5)
            exit
           else
            if (i == ipmod) then
             write(*,*)'readpart.f: No such particle in modpart.f'
             stop
            end if
           end if
          end do
         end do
         gbbbb(nbbbba)=trim(adjustl(cline(l+1:)))
        else if (j == 3) then
         ifrmn=0
         do k=1,3
          do i=1,ipmod
           if (trim(prtcl(k)) == trim(cprt(i,1))
     &         .or. trim(prtcl(k)) == trim(cprt(i,2))) then
            if (iprt(i,2) /= 0) then
             ifrmn=1
             exit
            end if
           end if
          end do
          if (ifrmn == 1) exit
         end do
         if (ifrmn == 1) then
          nffba=nffba+1
          if (nffba > nffb) then
           write(*,*)'Too many fermion-fermion-boson couplings'//
     &               ' in vertices.dat.'
           write(*,*)'Increase nffb in maxvals.f.'
          end if
          do k=1,3
           do i=1,ipmod
            if (trim(prtcl(k)) == trim(cprt(i,1))) then
             ffb(nffba,k)=iprt(i,5)
             exit
            else if (trim(prtcl(k)) == trim(cprt(i,2))) then
             ffb(nffba,k)=-iprt(i,5)
             exit
            else
             if (i == ipmod) then
              write(*,*)'readpart.f: No such particle in modpart.f'
              stop
             end if
            end if
           end do
          end do
          gffb(nffba)=trim(adjustl(cline(l+1:)))
         else
          nbbba=nbbba+1
          if (nbbba > nbbb) then
           write(*,*)'Too many triple-boson couplings in vertices.dat.'
           write(*,*)'Increase nbbb in maxvals.f.'
          end if
          do k=1,3
           do i=1,ipmod
            if (trim(prtcl(k)) == trim(cprt(i,1))) then
             bbb(nbbba,k)=iprt(i,5)
             exit
            else if (trim(prtcl(k)) == trim(cprt(i,2))) then
             bbb(nbbba,k)=-iprt(i,5)
             exit
            else
             if (i == ipmod) then
              write(*,*)'readpart.f: No such particle in modpart.f'
              stop
             end if
            end if
           end do
          end do
          gbbb(nbbba)=trim(adjustl(cline(l+1:)))
         end if
        else if (j == 2) then
         nbba=nbba+1
         if (nbba > mxbb) then
          write(*,*)'Too many boson-boson mixings in vertices.dat.'
          write(*,*)'Increase mxbb in maxvals.f.'
         end if
         do k=1,2
          do i=1,ipmod
           if (trim(prtcl(k)) == trim(cprt(i,1))) then
            mcbb(nbba,k)=iprt(i,5)
            exit
           else if (trim(prtcl(k)) == trim(cprt(i,2))) then
            mcbb(nbbba,k)=-iprt(i,5)
            exit
           else
            if (i == ipmod) then
             write(*,*)'readpart.f: No such particle in modpart.f'
             stop
            end if
           end if
          end do
         end do
         gbb(nbba)=trim(adjustl(cline(l+1:)))
        else
         write(*,*)'readpart.f: Only triple and quartic couplings'//
     &             ' are possible'
         stop
        end if
       end if
      end do

      close(19)
 700  close(66)

      do i=1,nffba
       do j=1,3
        do k=1,ipmod
         if (abs(ffb(i,j)) == iprt(k,5)) then
          cffb(i,j)=iprt(k,3)
          mffb(i,j)=cprt(k,3)
          wffb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do

      do i=1,nbbba
       do j=1,3
        do k=1,ipmod
         if (abs(bbb(i,j)) == iprt(k,5)) then
          cbbb(i,j)=iprt(k,3)
          mbbb(i,j)=cprt(k,3)
          wbbb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do

      do i=1,nbbbba
       do j=1,4
        do k=1,ipmod
         if (abs(bbbb(i,j)) == iprt(k,5)) then
          cbbbb(i,j)=iprt(k,3)
          mbbbb(i,j)=cprt(k,3)
          wbbbb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do

      do i=1,nbba
       do j=1,2
        do k=1,ipmod
         if (abs(mcbb(i,j)) == iprt(k,5)) then
          clbb(i,j)=iprt(k,3)
          msbb(i,j)=cprt(k,3)
          wdbb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do

      nhel=1
c      nhip=1
      ncip=1

      do j=1,next
       nhel=nhel*f(j)%n
       if (f(j)%o == -1) then
c        nhip=nhip*f(j)%n
        ncip=ncip*f(j)%ct
       end if
      end do

      open(16,file='partcls.tmp',access='direct',form='formatted',
     &                                                     recl=80)
c      open(20,file='orders.tmp',access='direct',form='formatted',
c     &                                                     recl=80)

      do j=1,next
       write(16,1612,rec=j) f(j)%j,f(j)%i,f(j)%o,f(j)%g,f(j)%f,f(j)%ct,
     &                      f(j)%n,f(j)%e
c       write(20,1620,rec=j) j
      end do
c 1612 format(6(i3),i24,i3)
 1612 format(i8,5(i3),i24,i3)
 1620 format(26(i3))

      close(16)
c      close(20)

      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      write(*,'(a)') 'Attempting the process'
      write(*,'(a)') ' '
      write(*,'(a)') '        '//trim(adjustl(process))
      write(*,'(a)') ' '
      write(ind1,'(i10)') ipmod
      write(*,'(a)') 'with '//trim(adjustl(prvrsn))//
     & ', in a model with '//trim(adjustl(ind1))//' particles,'
      write(ind1,'(i10)') nffba
      write(ind2,'(i10)') nbbba
      write(ind3,'(i10)') nbbbba
      write(*,'(a)') trim(adjustl(ind1))//' fermion-fermion-boson, '// 
     & trim(adjustl(ind2))//' triple-boson and '
     & //trim(adjustl(ind3))//' quartic-boson couplings and'
      write(ind1,'(i10)') nbba
      write(*,'(a)') trim(adjustl(ind1))//' boson-boson mixings, '// 
c      write(*,'(a)') 
     &'see ''modpart.dat'' and ''vertices.dat'''//
     & ' for details.'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      if (iwtb == 1) then ! Include the anomalous wtb coupling'
       write(*,'(a)') 'The anomalous Wtb coupling included'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (imcs == 1) then
       write(*,'(a)') 'MC summing over polarizations implemented'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (iphot == 0) isr=0 ! added on 14 Oct. 2015
      if (isr == 1) then    ! added on 14 Oct. 2015
       write(*,'(a)') 'Only the ISR taken into account'
       write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      else if (isr == 2) then
       write(*,'(a)') 'Only the FSR taken into account'
       write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (igvb > 1) then
       write(*,'(a,i2,a)')'There are', igvb,' massless vector bosons '
     &          //'radiated in the process.'
       write(*,'(a)')'The dedicated kinematics is not yet implemented,'
     &              //' hence igchnl set to 0'
       write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (igchnl0 == 0 .and. igchnl == 1) then
       write(*,'(a)')'Flag igchnl set to igchn=1 in order to '
     &              //'generate kinematics dedicated '
       write(*,'(a)')'to a photon or gluon radiation process'
       write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (ischnl == 1) then
       write(*,'(a,i4,a)') 'Propagators of subsystems with up to',mxmap,
     &              ' particles will be mapped'
       write(*,'(a)') 'in s-channel kinematical routines'
       write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (itchnl == 1) then
       write(*,'(a)') 't-channel kinematics for the multichannel MC'//
     & ' integration will be generated'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (ischnl == 0 .and. itchnl ==  0 .and. igchnl == 0) then
       write(*,'(a)')'The kinematics will not be generated if all '
     &             //'the flags:'
       write(*,'(a)')'ischnl, itchnl and igchnl are 0.'
       if (igvb > 1) then
        write(*,'(a)')'Set ischnl or itchnl to 1 and run the '
     &             //'program anew'
       else
        write(*,'(a)')'Set at least one of them to 1 and run the '
     &             //'program anew'
       end if
       stop
      end if

      write(*,'(a)') 'External particles:'
      do i=1,next
       if (f(i)%j == 9 .or. f(i)%j == 10) then
        write(*,1) f(i)%ix,trim(expc(i)),f(i)%f,f(i)%i,f(i)%o,
     &             f(i)%o*f(i)%i*f(i)%j,f(i)%n
       else
        write(*,1) f(i)%ix,trim(expc(i)),f(i)%j,f(i)%i,f(i)%o,
     &             f(i)%o*f(i)%i*f(i)%j,f(i)%n
       end if
      end do
 1    format(i4,'  ',a,' :',i8,2(i3),i8,i3)
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'

      ikfsr=0
      nkfsr=0
      if (igvb == 1) then
       open(unit=288,file='kfsrinf.dat',status='replace',action='write')
c Check for pairs of the final state particles that couple to the same triple ffb vertex
       do i=1,nffba
        do j=nisp+1,next
         if (f(j)%o*f(j)%i*f(j)%j == ffb(i,1)) then
          do k=nisp+1,next
           if (f(k)%o*f(k)%i*f(k)%j == ffb(i,2)) then
            write(ind1,'(i10)') j
            write(ind2,'(i10)') k
            if (f(j)%f /= 1) then
             nkfsr=nkfsr+1
             write(288,'(a)') trim(clgfsr)//trim(adjustl(ind1))//','//
     &                       trim(adjustl(ind2))//','//trim(mffb(i,3))//
     &                      ','//trim(wffb(i,3))//','
            end if
            if (f(k)%f /= 1) then
             nkfsr=nkfsr+1
             write(288,'(a)') trim(clgfsr)//trim(adjustl(ind2))//','//
     &                       trim(adjustl(ind1))//','//trim(mffb(i,3))//
     &                      ','//trim(wffb(i,3))//','
            end if
           end if
          end do
         end if
        end do
       end do
       do i=1,nbbba
        do j=nisp+1,next
         if (f(j)%o*f(j)%i*f(j)%j == bbb(i,1)) then
          do k=nisp+1,next
           if (f(k)%o*f(k)%i*f(k)%j == bbb(i,2)) then
            write(ind1,'(i10)') j
            write(ind2,'(i10)') k
            if (f(j)%f /= 1) then
             nkfsr=nkfsr+1
             write(288,'(a)') trim(clgfsr)//trim(adjustl(ind1))//','//
     &                       trim(adjustl(ind2))//','//trim(mbbb(i,3))//
     &                      ','//trim(wbbb(i,3))//','
            end if
            if (f(k)%f /= 1) then
             nkfsr=nkfsr+1
             write(288,'(a)') trim(clgfsr)//trim(adjustl(ind2))//','//
     &                       trim(adjustl(ind1))//','//trim(mbbb(i,3))//
     &                      ','//trim(wbbb(i,3))//','
            end if
           end if
          end do
         end if
        end do
       end do
       close(288)
      end if

c Check for neutrinos in a wrong place in calls to the FSR routines

      if (igchnl == 1) then
       open(unit=288,file='kfsrinf.dat',status='old',action='read')
       open(unit=289,file='kfsrwrg.dat',status='replace',action='write')
       do 
        read(288,'(a80)',end=800) cline
        do j=nisp+1,next
         if (f(j)%f == 1) then
          write(ind1,'(i4)') j
          if (index(cline,trim(adjustl(ind1))//',mw') > 0
     &     .or. index(cline,trim(adjustl(ind1))//',mz') > 0
     &     .or. index(cline,trim(adjustl(ind1))//',mh') > 0) then
           write(289,'(a)') cline
          end if
         end if
        end do
       end do
 800   close(288)
       close(289)
      end if

      if (ischnl /= 2) then
       call wrkinprms(next-nisp)
      end if

      if (ischnl /= 2) then
       open(unit=20,file='kinschnl.f',status='replace',action='write')
       write(20,'(a)') '      subroutine kinschnl'
       write(20,'(a)') 'c This subroutine is empty if ischnl /= 2'
       write(20,'(a)') '      end'
       close(20)
c       open(unit=20,file='kinparams.f',status='replace',action='write')
c       write(20,'(a)')'      module kinparams'
c       write(20,'(a)') 'c This module is empty if ischnl /= 2'
c       write(20,'(a)') '      end'
c       close(20)
      end if

      end
