      subroutine runwidth(iwdth,p,mass,wdth,mv2)
c----------------------------------------------------------------------c
c Returns a complex mass
c        mv2=cmplx(mass**2,-mass*rwdth,dp)
c with a running particle width rwdth
c
c Input: iwdth=1,2,3,... - running width of the vector particle,
c              supplement your own running width option below,
c        p - four momentum of a particle,
c        mass, wdth - mass and width of a particle.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 12 March, 2015; modified on 
c 19 March, 2015: a module inprms added
c----------------------------------------------------------------------c

      use inprms 

      implicit none

      integer :: iwdth
      real(dp) :: p(0:3),mass,wdth,s,sqs,rwdth
      complex(dp) :: mv2

      s=p(0)**2-p(1)**2-p(2)**2-p(3)**2

      if (s > 0.d0) then
       sqs=sqrt(p(0)**2-p(1)**2-p(2)**2-p(3)**2)
       if (iwdth == 1) then
        if (sqs < 2.d0*mpi) then
         rwdth=0.d0
        else
         rwdth=wdth*(mass/sqs)
     &       *(sqrt(sqs**2-4.d0*mpi**2)/sqrt(mass**2-4.d0*mpi**2))**3
        end if
       else if (iwdth == 2) then
        if (sqs < 2.d0*mpi) then
         rwdth=0.d0
        else
         rwdth=wdth*(mass/sqs)**2
     &       *(sqrt(sqs**2-4.d0*mpi**2)/sqrt(mass**2-4.d0*mpi**2))**3
        end if
       else if (iwdth == 3) then
        rwdth=wdth*(mass/sqs)**2
       else
        write(7,*)'runwidth.f: iwdth=',iwdth,' is not implemented'
       end if
      else
       rwdth=0.d0
      end if

      mv2=cmplx(mass**2,-mass*rwdth,dp)

      end subroutine runwidth
